<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'CrBcHelpers.php');
require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'CrBcCart.php');

class CrBcAttributes
{
    static $BACKORDERS_DISALLOWED_VISIBLE = 'DISALLOWED_VISIBLE';
    static $BACKORDERS_DISALLOWED_HIDDEN = 'DISALLOWED_HIDDEN';

    public static function copyProductAttributes($src_id, $dest_id){
        $config['key_id'] = 'id';

        $config['compare_key'] = 'product_id';
        $config['table'] = '#__breezingcommerce_product_attributes';
        $ids = self::copyTableRows($src_id, $dest_id, $config);

        $config['compare_key'] = 'product_id';
        
        $config['table'] = '#__breezingcommerce_product_attribute_items';
        foreach($ids As $id){
            self::copyTableRows(
                    $src_id, 
                    $dest_id, 
                    $config, 
                    array('product_attribute_id' => $id['insert_id']), 
                    array('product_attribute_id' => $id['original_id'])
            );
        }
    }

    public static function copyTableRows($src_id, $dest_id, $config, $property_overrides = array(), $compare_extras = array()){
        $ret = array();

        $original_id = 0;
        $insert_ids = array();
        $db = JFactory::getDBO();

        $and = '';
        foreach($compare_extras As $column => $value){
            $and .= " And ".$db->quoteName($column)." = ".$db->quote($value);
        }
        
        $db->setQuery( ' Select * From '. $config['table'] .
                '  Where '.$config['compare_key'].' = ' . intval($src_id) . $and);
        $result = $db->loadObjectList();

        //if(count($compare_extras)){
        //    echo $db->getQuery();
        //    exit;
        //}
        
        $i = 0;
        foreach($result As $obj)
        {
            foreach($property_overrides As $key => $property_override){
                $obj->{$key} = $property_override;
            }

            $ret[$i]['original_id'] = $obj->{$config['key_id']};
            unset($obj->{$config['key_id']});
            $obj->{$config['compare_key']} = intval($dest_id);

            $db->insertObject($config['table'], $obj);
            $ret[$i]['insert_id'] = $db->insertid();

            $i++;
        }
        return $ret;
    }

    public static function getAttributesForm($product_id, $amount, $selected = array()){

        if(!$amount){
            $amount = 1;
        }

        $db = JFactory::getDBO();
        
        $db->setQuery("
            Select
                items.*,
                items.product_id As item_product_id,
                attributes.`name` As attribute_name,
                attributes.`min_amount`,
                attributes.`allow_amount`,
                attributes.display_as As attribute_display_as,
                attributes.required As attribute_required,
                product_attribute_items.`name` As product_attribute_item_name,
                product_attribute_items.`id` As product_attribute_item_id,
                product_attribute_items.plus_price As product_attribute_item_plus_price,
                product_attribute_items.item_product_id As product_attribute_item_product_id,
                product_attribute_items.selected As product_attribute_item_selected,
                p.producttaxclass_id,
                p.stock,
                p.attribs,
                p.id As pid,
                p.virtual_product
            From
                #__breezingcommerce_attributes As attributes,
                #__breezingcommerce_attribute_items As items,
                #__breezingcommerce_product_attributes As product_attributes,
                #__breezingcommerce_product_attribute_items As product_attribute_items,
                #__breezingcommerce_products As p
            Where
                product_attributes.product_id = ".intval($product_id)."
            And
                product_attribute_items.product_attribute_id = product_attributes.id
            And
                attributes.id = product_attributes.attribute_id
            And
                items.id = product_attribute_items.attribute_item_id
            And
                attributes.id = items.attribute_id
            And
                p.id = product_attributes.product_id
            And
                product_attributes.published = 1
            And
                product_attribute_items.published = 1
            Order By
                product_attributes.`ordering`,
                product_attribute_items.`ordering`
                    
        ");
        $rows = $db->loadAssocList();

        $has_attributes = false;
        $attributes = array();

        $pform = new JRegistry('', JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');

        // used for global override
        $form = CrBcHelpers::getBcConfig();

        foreach($rows As $row){
            
            if(!isset($attributes[$row['attribute_id']])){
                $has_attributes = true;
                
                $translation = CrBcHelpers::loadTranslation($row['attribute_id'], 'attribute');
                
                if(is_object($translation)){
                    
                    $attributes[$row['attribute_id']]['name'] = $translation->title;
                }
                else
                {
                    $attributes[$row['attribute_id']]['name'] = $row['attribute_name'];
                }
                
                $attributes[$row['attribute_id']]['required'] = $row['attribute_required'];
                $attributes[$row['attribute_id']]['display_as'] = $row['attribute_display_as'];
                $attributes[$row['attribute_id']]['allow_amount'] = $row['allow_amount'];
                $attributes[$row['attribute_id']]['min_amount'] = $row['min_amount'];//product_attribute_item_id
                $attributes[$row['attribute_id']]['producttaxclass_id'] = $row['producttaxclass_id'];
                $attributes[$row['attribute_id']]['product_id'] = $row['pid'];
                $attributes[$row['attribute_id']]['attribs'] = $row['attribs'];
                $attributes[$row['attribute_id']]['virtual_product'] = $row['virtual_product'];
                $attributes[$row['attribute_id']]['items'] = array();
            }

            $name = $row['name'];
            
            $translation = CrBcHelpers::loadTranslation($row['id'], 'attribute_item_'.$row['attribute_id']);
            
            if(is_object($translation)){

                $name = $translation->title;
            }
            else
            {
                if($row['product_attribute_item_name'] != ''){
                    $name = $row['product_attribute_item_name'];
                }
            }
            
            $plus_price = 0;

            $disallowed = false;
            
            $aproduct = null;
            $stock_products = null;
            
            if($row['product_attribute_item_product_id'] || $row['item_product_id']){
                
                $p_virtual_product = false;
                $p_stock = 0;
                $p_id = 0;
                $p_attribs = '';
                
                if ($row['product_attribute_item_product_id']) {
                    $p_id = $row['product_attribute_item_product_id'];
                } else {
                    $p_id = $row['item_product_id'];
                }

                if ($p_id) {
                    
                    $db->setQuery("Select price, stock, virtual_product, attribs From #__breezingcommerce_products Where id = " . $p_id . " And published = 1");
                    $aproduct = $db->loadAssoc();
                    
                    // no result but an associated product? Then we need to skip displaying it entirely
                    if(!is_array($aproduct)){
                        continue;
                    }
                    
                    $stock_products[$row['product_attribute_item_id']] = $aproduct;
                    
                    if ($aproduct['price']) {
                        $plus_price = $aproduct['price'];
                    }
                    

                    $pform->loadString($aproduct['attribs']);

                    $disallowed = false;
                    
                    if( !$aproduct['virtual_product']
                            &&
                            $aproduct['stock'] <= 0
                            &&
                            $pform->get('backorders','') != ''
                         ){
                        if( $pform->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN ){
                              $disallowed = true;
                        }
                        
                    } else if( !$aproduct['virtual_product']
                            &&
                            $aproduct['stock'] <= 0
                            &&
                            $form->get('backorders','') != ''
                         ){
                        if( $form->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN ){
                              $disallowed = true;
                        }
                    }
                    
                    // disallowed and hidden ? then also no display if there is a product associated
                    if($disallowed){
                        continue;
                    }
                }
                
            }

            if($row['plus_price'] != 0 && !$row['product_attribute_item_product_id']){

                $plus_price = $row['plus_price'];

            }

            if($row['product_attribute_item_plus_price'] != 0){
                
                $plus_price = $row['product_attribute_item_plus_price'];

            }

            $pre_selected = false;
            if($row['product_attribute_item_selected'] && !count($selected)){
                $pre_selected = true;
            }

            if(!$disallowed){
                $attributes[$row['attribute_id']]['items'][] = array(
                    'id' => $row['product_attribute_item_id'],
                    'name' => $name,
                    'pre_selected' => $pre_selected,
                    'plus_price' => $plus_price
                );
            }
        }

        $out = '<div id="crbc-attributes-'.$product_id.'" class="crbc-attributes">';

        $required_elements = '';
        $minimum_elements = '';
        
        $cart = new CrBcCart( JFactory::getSession()->get('crbc_cart', array()) );

        // just in case we need the customer id for anything regarding prices
        
        $customer_id = 0;

        if($cart instanceof CrBcCart && JFactory::getUser()->get('id', 0) != 0){
            
            JFactory::getDbo()->setQuery("Select id From #__breezingcommerce_customers Where userid = " . intval(JFactory::getUser()->get('id', 0)) . " Limit 1");
            $customer_id = JFactory::getDbo()->loadResult();

        } else {
            
            $customer_id = isset($cart->cart['customer_id']) ? $cart->cart['customer_id'] : 0;
        }
        
        if(count($attributes)){
            $out .= '<h3 id="crbcAttributesTitle">'.JText::_('COM_BREEZINGCOMMERCE_PRODUCT_ACCESSORIES').'</h3>'."\n";
        }

        foreach($attributes As $attribute_id => $attribute){

            $attribute_id = $attribute_id.$product_id;
            
            $amount_of_attributes = -1;

            foreach ($attribute['items'] As $item){
                if (isset($selected[$item['id']]) || $item['pre_selected']) {
                    if (isset($selected[$item['id']])) {
                        $amount_of_attributes = $selected[$item['id']];
                    } else {
                        $amount_of_attributes = $attribute['min_amount'];
                    }
                    break;
                }
            }

            $select_first = false;
            if($amount_of_attributes == -1){
                $select_first = true;
            }

            $out .= '<div id="crbcAttributeWrapper'.$attribute_id.'" class="crbcAttributeWrapper crbc-well well">'."\n";
            $out .= '<div class="controls form-inline">';
            
            $out .= '<div id="crbcAttributeLabelWrapper"><label for="crbcAttributes'.$attribute_id.'">'.$attribute['name'].($attribute['required'] ? ' <i class="crbc-fa crbc-fa-star-o'.(JFactory::getApplication()->isAdmin() ? ' icon-asterisk' : '').'"></i>' : '').'</label></div>'."\n";
            // select-list
            // select-list-multiple
            // radio
            // checkbox
            switch($attribute['display_as']){
                case 'select-list-multiple':
                case 'select-list':
                case 'radio':
                case 'checkbox':
                    $multiple = '';
                    $toggle_amounts = ' onchange="crbcToggleAttributeAmount(this, '.intval($attribute_id).');crbcUpdatePriceByAttribute();"';

                    $head = '';

                    if(!$attribute['allow_amount']){
                        $toggle_amounts = '';
                    }

                    if($attribute['display_as'] == 'select-list-multiple' || $attribute['display_as'] == 'checkbox'){
                        $multiple = ' multiple="multiple"';
                        $toggle_amounts = '';
                    }
                    
                    if($attribute['display_as'] == 'select-list-multiple'){
                        $toggle_amounts = 'onchange="crbcUpdatePriceByAttribute()" ';
                    }
                    
                    if($attribute['display_as'] == 'select-list-multiple' || $attribute['display_as'] == 'select-list'){
                        
                        $select = '<select '.$toggle_amounts.' name="crbcAttributes['.$attribute_id.'][]" class="crbcAttributes" id="crbcAttributes'.$attribute_id.'"'.$multiple.'>'."\n";
                        if(!$attribute['required']){
                            $select .= '<option value="0"'.($select_first ? ' selected="selected"' : '').'>'.htmlentities(JText::_('COM_BREEZINGCOMMERCE_NONE'), ENT_QUOTES, 'UTF-8').'</option>'."\n";
                            $select_first = false;
                        }

                        $i = 0;
                        
                        foreach ($attribute['items'] As $item){

                            $add_amount = 0;
                            if($amount_of_attributes != -1){
                                $add_amount = $amount_of_attributes <= 0 ? 1 : $amount_of_attributes;
                            }else{
                                $add_amount = $attribute['min_amount'] <= 0 ? 1 : $attribute['min_amount'];
                            }

                            if($select_first){
                                $amount_of_attributes = $attribute['min_amount'];
                            }

                            $plus_price = '';
                            if($item['plus_price'] != 0){
                                $prefix = '+';
                                if($item['plus_price'] < 0){
                                    $prefix = ''; // added in CrBcCart::priceFormat()
                                }
                                $price = CrBcCart::getItemPrice( $item['plus_price'], $cart->currency_conversion_rate, $attribute['producttaxclass_id'], array(), array(), $amount * $add_amount, $customer_id);
                                $plus_price = ' (<span class="crbc-attr-amount"></span>'.$prefix.$cart->formatPrice($price['gross']).')';
                            }

                            $select .= '<option value="'.$item['id'].'"'.(isset($selected[$item['id']]) || $item['pre_selected'] || ($select_first && $i == 0 ) ? ' selected="selected"' : '').'>'.htmlentities($item['name'], ENT_QUOTES, 'UTF-8').$plus_price.(is_array($stock_products) && isset($stock_products[$item['id']]) && isset($stock_products[$item['id']]['virtual_product']) && !$stock_products[$item['id']]['virtual_product'] ? ' / ' . ($stock_products[$item['id']]['stock'] <= 0 ? JText::_('COM_BREEZINGCOMMERCE_OUT_OF_STOCK') : JText::_('COM_BREEZINGCOMMERCE_IN_STOCK') ) : '').'</option>'."\n";
                            $i++;
                        }

                        $select .= '</select>'."\n";
                        
                        if($attribute['required']){
                            $required_elements .= "\n".'<script type="text/javascript">'."\n".'<!--'."\n".'crbcRequiredElements.push(document.getElementById("crbcAttributes'.intval($attribute_id).'"));'."\n//-->\n</script>";
                        }

                        if($attribute['display_as'] == 'select-list'){
                            $minimum_elements .= "\n".'<script type="text/javascript">'."\n".'<!--'."\n".'crbcRequiredElementsMinimum.push({id: "crbcAttributes'.intval($attribute_id).'", min_amount: '.intval($attribute['min_amount']).'});'."\n//-->\n</script>";
                        }
                    }
                    else if($attribute['display_as'] == 'radio'){

                        $select = '<div class="crbcAttributesRadioWrapper" id="crbcAttributesRadioWrapper'.$attribute_id.'">'."\n";
                        $select .= '<div class="controls form-inline">';
                        
                        if(!$attribute['required']){
                            $select .= '<div class="crbcAttributeRadioWrap" id="crbcAttributeRadioWrap0"><input onchange="crbcUpdatePriceByAttribute()" type="radio"'.$toggle_amounts.' name="crbcAttributes['.$attribute_id.'][]" value="0" class="crbcAttributeRadio crbcAttributes" id="crbcAttributeRadio0"'.($select_first ? ' checked="checked"' : '').'/> <label for="crbcAttributeRadio0">'.htmlentities(JText::_('COM_BREEZINGCOMMERCE_NONE'), ENT_QUOTES, 'UTF-8')."</label></div>\n";
                            $select_first = false;
                        }

                        $i = 0;
                        foreach ($attribute['items'] As $item){

                            $add_amount = 0;
                            if($amount_of_attributes != -1){
                                $add_amount = $amount_of_attributes <= 0 ? 1 : $amount_of_attributes;
                            }else{
                                $add_amount = $attribute['min_amount'] <= 0 ? 1 : $attribute['min_amount'];
                            }

                            if($select_first){
                                $amount_of_attributes = $attribute['min_amount'];
                            }

                            $plus_price = '';
                            if($item['plus_price'] != 0){
                                $prefix = '+';
                                if($item['plus_price'] < 0){
                                    $prefix = ''; // added in CrBcCart::priceFormat()
                                }
                                $price = CrBcCart::getItemPrice( $item['plus_price'], $cart->currency_conversion_rate, $attribute['producttaxclass_id'], array(), array(), $amount * $add_amount, $customer_id);
                                $plus_price = ' (<span class="crbc-attr-amount"></span>'.$prefix.$cart->formatPrice($price['gross']).')';
                            }

                            $select .= '<div class="crbcAttributeRadioWrap" id="crbcAttributeRadioWrap'.$item['id'].'"><input onchange="crbcUpdatePriceByAttribute()" type="radio"'.$toggle_amounts.' name="crbcAttributes['.$attribute_id.'][]" class="crbcAttributeRadio crbcAttributes" id="crbcAttributeRadio'.$item['id'].'" value="'.$item['id'].'"'.(isset($selected[$item['id']]) || $item['pre_selected'] || ($select_first && $i == 0 )  ? ' checked="checked"' : '').'/> <label for="crbcAttributeRadio'.$item['id'].'">'.htmlentities($item['name'], ENT_QUOTES, 'UTF-8').$plus_price.(is_array($stock_products) && isset($stock_products[$item['id']]) && isset($stock_products[$item['id']]['virtual_product']) && !$stock_products[$item['id']]['virtual_product'] ? ' / ' . ($stock_products[$item['id']]['stock'] <= 0 ? JText::_('COM_BREEZINGCOMMERCE_OUT_OF_STOCK') : JText::_('COM_BREEZINGCOMMERCE_IN_STOCK') ) : '')."</label></div>\n";
                            $i++;
                        }

                        $select .= '</div></div>'."\n";

                        if($attribute['required']){
                            $required_elements .= "\n".'<script type="text/javascript">'."\n".'<!--'."\n".'crbcRequiredElements.push(crbcGetFormElementByName("crbcAttributes['.$attribute_id.'][]"));'."\n//-->\n</script>";
                        }

                        $minimum_elements .= "\n".'<script type="text/javascript">'."\n".'<!--'."\n".'crbcRequiredElementsMinimum.push({id: "crbcAttributes'.intval($attribute_id).'", min_amount: '.intval($attribute['min_amount']).'});'."\n//-->\n</script>";

                    }
                    else if($attribute['display_as'] == 'checkbox'){

                        $select = '<div class="crbcAttributesCheckboxWrapper" id="crbcAttributesCheckboxWrapper'.$attribute_id.'">'."\n";
                        
                        $i = 0;
                        foreach ($attribute['items'] As $item){

                            $add_amount = 0;
                            if($amount_of_attributes != -1){
                                $add_amount = $amount_of_attributes <= 0 ? 1 : $amount_of_attributes;
                            }else{
                                $add_amount = $attribute['min_amount'] <= 0 ? 1 : $attribute['min_amount'];
                            }

                            if($select_first){
                                $amount_of_attributes = $attribute['min_amount'];
                            }

                            $plus_price = '';
                            if($item['plus_price'] != 0){
                                $prefix = '+';
                                if($item['plus_price'] < 0){
                                    $prefix = ''; // added in CrBcCart::priceFormat()
                                }
                                $price = CrBcCart::getItemPrice( $item['plus_price'], $cart->currency_conversion_rate, $attribute['producttaxclass_id'], array(), array(), $amount * $add_amount, $customer_id);
                                $plus_price = ' (<span class="crbc-attr-amount"></span>'.$prefix.$cart->formatPrice($price['gross']).')';
                            }

                            $select .= '<div class="crbcAttributeCheckboxWrap" id="crbcAttributeCheckboxWrap'.$item['id'].'"><input onchange="crbcUpdatePriceByAttribute()" type="checkbox"'.$toggle_amounts.' name="crbcAttributes['.$attribute_id.'][]" class="crbcAttributeCheckbox crbcAttributes" id="crbcAttributeCheckbox'.$item['id'].'" value="'.$item['id'].'"'.(isset($selected[$item['id']]) || $item['pre_selected'] || ($select_first && $i == 0 )  ? ' checked="checked"' : '').'/> <label for="crbcAttributeCheckbox'.$item['id'].'">'.htmlentities($item['name'], ENT_QUOTES, 'UTF-8').$plus_price.(is_array($stock_products) && isset($stock_products[$item['id']]) && isset($stock_products[$item['id']]['virtual_product']) && !$stock_products[$item['id']]['virtual_product'] ? ' / ' . ($stock_products[$item['id']]['stock'] <= 0 ? JText::_('COM_BREEZINGCOMMERCE_OUT_OF_STOCK') : JText::_('COM_BREEZINGCOMMERCE_IN_STOCK') ) : '')."</label></div>\n";
                            $i++;
                        }

                        // not in checkboxes
                        //$minimum_elements .= 'crbcRequiredElementsMinimum.push({id: "crbcAttributes'.intval($attribute_id).'", min_amount: '.intval($add_amount).'});'."\n";

                        $select .= '</div>'."\n";

                        if($attribute['required']){
                            $required_elements .= "\n".'<script type="text/javascript">'."\n".'<!--'."\n".'crbcRequiredElements.push(crbcGetFormElementByName("crbcAttributes['.$attribute_id.'][]"));'."\n//-->\n</script>";
                        }
                    }

                    if($attribute['display_as'] == 'select-list-multiple' || $attribute['display_as'] == 'checkbox'){

                        $head = '<input type="hidden" name="crbcAttributesAmounts['.$attribute_id.']" id="crbcAttributesAmounts'.$attribute_id.'" value="'.$attribute['min_amount'].'"/>'."\n";
                        
                    }else{

                        $head = '<div class="crbcAttributeQtyWrapper">
<label id="crbcAttributesAmountsLabel'.$attribute_id.'" style="display:'.(($amount_of_attributes > -1 && $attribute['allow_amount']) ? 'inline' : 'none').';" class="control-label" for="crbcAttributesAmounts'.$attribute_id.'">'.JText::_('COM_BREEZINGCOMMERCE_QTY').'</label>
<div style="display:none;" id="crbcAttributesAmountsEmpty'.$attribute_id.'" style="display:'.(($amount_of_attributes == -1 || !$attribute['allow_amount']) ? 'block' : 'none').'">'.($attribute['min_amount']).'</div>
<div class="controls form-inline">
<div class="input-append">
<input onchange="if(this.value < '.$attribute['min_amount'].'){this.value = '.$attribute['min_amount'].'}; jQuery(\'.crbc-attr-amount\').html(parseInt(this.value) > 1 ? parseInt(this.value) + \' x \' : \'\'); crbcUpdatePriceByAttribute();" type="text" name="crbcAttributesAmounts['.$attribute_id.']" class="crbcAttributesAmounts" id="crbcAttributesAmounts'.$attribute_id.'" value="'.($amount_of_attributes > -1 && $attribute['allow_amount'] ? $amount_of_attributes : $attribute['min_amount']).'" style="display:'.(($amount_of_attributes > -1 && $attribute['allow_amount']) ? 'inline' : 'none').';"/>
<button style="display:'.(($amount_of_attributes > -1 && $attribute['allow_amount']) ? 'inline' : 'none').';" class="btn" type="button" onclick="crbcUpdateAttributeQty(+1,'.intval($attribute['min_amount']).',\'#crbcAttributesAmounts'.$attribute_id.'\')"><i class="icon-chevron-up"></i></button>
<button style="display:'.(($amount_of_attributes > -1 && $attribute['allow_amount']) ? 'inline' : 'none').';" class="btn" type="button" onclick="crbcUpdateAttributeQty(-1,'.intval($attribute['min_amount']).',\'#crbcAttributesAmounts'.$attribute_id.'\')"><i class="icon-chevron-down"></i></button>
</div>
</div>
<div>&nbsp;</div>
'.(($attribute['allow_amount'] && intval($attribute['min_amount']) > 0) ? JText::_('COM_BREEZINGCOMMERCE_MINIMUM_QUANTITY').': '.intval($attribute['min_amount']) : '').'
</div>'."\n";

                    }

                    $out .= $select.$head;
                    break;
            }
            $out .= '</div>'."\n";
            $out .= '</div>'."\n";
        }
        
        $out .= $required_elements."\n".$minimum_elements."\n";

        $out .= '</div>';
        
        return $out;
    }
    
    public static function getAttributeJs(){
        
        return '
var crbcTriggerCartValidations = false;
var crbcRequiredElements = new Array();
var crbcRequiredElementsMinimum = new Array();
var crbcTmpAttributeAmounts = {};
var crbcCartValidationErrorText = '.CrBcHelpers::jsonEncode(JText::_('COM_BREEZINGCOMMERCE_ACCESSORIES_ERROR')).';
var crbcCartValidationErrorElement = null;

function crbcToggleAttributeAmount(element, attribute_id){
    switch(element.type){
        case "select-one":
        case "radio":
            if( (typeof element.value != "undefined" && element.value == "0") || (typeof element.selectedIndex != "undefined" && (element.selectedIndex < 0 || element.options[element.selectedIndex].value == "0"))){
                document.getElementById("crbcAttributesAmounts"+attribute_id).blur();
                document.getElementById("crbcAttributesAmounts"+attribute_id).style.display = "none";
                document.getElementById("crbcAttributesAmountsLabel"+attribute_id).style.display = "none"; 
                //document.getElementById("crbcAttributesAmountsEmpty"+attribute_id).style.display = "inline";
                crbcTmpAttributeAmounts["crbcAttributesAmounts"+attribute_id] = document.getElementById("crbcAttributesAmounts"+attribute_id).value;
                document.getElementById("crbcAttributesAmounts"+attribute_id).value = 0;
                jQuery(document.getElementById("crbcAttributesAmounts"+attribute_id)).next(".btn").css("display","none");
                jQuery(document.getElementById("crbcAttributesAmounts"+attribute_id)).next(".btn").next(".btn").css("display","none");
            }else{
                document.getElementById("crbcAttributesAmounts"+attribute_id).style.display = "inline";
                document.getElementById("crbcAttributesAmountsLabel"+attribute_id).style.display = "block"; 
                document.getElementById("crbcAttributesAmounts"+attribute_id).focus();
                //document.getElementById("crbcAttributesAmountsEmpty"+attribute_id).style.display = "none";
                if(typeof crbcTmpAttributeAmounts["crbcAttributesAmounts"+attribute_id] != "undefined"){
                    document.getElementById("crbcAttributesAmounts"+attribute_id).value = crbcTmpAttributeAmounts["crbcAttributesAmounts"+attribute_id];
                }
                
                jQuery(document.getElementById("crbcAttributesAmounts"+attribute_id)).next(".btn").css("display","inline");
                jQuery(document.getElementById("crbcAttributesAmounts"+attribute_id)).next(".btn").next(".btn").css("display","inline");
            }
        break;
    }
}

function crbcUpdatePriceByAttribute(){
    if( typeof crbc_products != "undefined" ){
        for(var i = 0; i < crbc_products.length; i++){
            crbc_products[i].update_price();
        }
    }
    else if ( typeof crbc_product != "undefined" ){
         crbc_product.update_price();
    }
}

crbcUpdateAttributeQty = function(amount, min_amount, element){
        var qty = parseInt(jQuery(element).val());
        qty = isNaN(qty) ? min_amount : qty;
        var result = amount + qty;
        if(result < min_amount){
            result = qty;
        }
        jQuery(element).val(result);
        crbcUpdatePriceByAttribute();
    }
';
    }
    
    public static function getRequiredJs(){
        
        return '

function crbcGetFormElementByName(name){
    var nodeList = [];
    for(var i = 0; i < document.forms.length; i++){
        for(var j = 0; j < document.forms[i].elements.length; j++){
            if(document.forms[i].elements[j].name == name){
                nodeList.push(document.forms[i].elements[j]);
            }
        }
    }
    return nodeList;
}

function crbcValidateAttributes(){

    crbcCartValidationErrorElement = null;

    if(crbcTriggerCartValidations){
        
        for(var i = 0; i < crbcRequiredElementsMinimum.length; i++){
        
            var amount_obj = jQuery("#"+crbcRequiredElementsMinimum[i].id).closest(".crbcAttributeWrapper").find(".crbcAttributesAmounts");
            var amount = parseInt(jQuery(amount_obj).val());

            if(isNaN(amount) || amount < crbcRequiredElementsMinimum[i].min_amount){
    
                crbcCartValidationErrorElement = jQuery("#"+crbcRequiredElementsMinimum[i].id);
                return false;
            }
        }

        for(var i = 0; i < crbcRequiredElements.length; i++){
        
            if(typeof crbcRequiredElements[i] === "object" && crbcRequiredElements[i] && crbcRequiredElements[i] instanceof Array){
                var found = false;
                for(var j = 0; j < crbcRequiredElements[i].length; j++){
                    if(crbcRequiredElements[i][j].checked){
                        found = true;
                    }
                }
                if(!found){
                    crbcCartValidationErrorElement = crbcRequiredElements[i];
                    return false;
                }
            } else {
            
                switch(crbcRequiredElements[i].type){
                    case "select-multiple":
                    case "select-one":
                        if(crbcRequiredElements[i].selectedIndex == -1 || crbcRequiredElements[i].options[crbcRequiredElements[i].selectedIndex].value == "0"){
                            crbcCartValidationErrorElement = crbcRequiredElements[i];
                            return false;
                        }else{
                            
                            var amount_obj = jQuery(crbcRequiredElements[i]).closest(".crbcAttributeWrapper").find(".crbcAttributesAmounts");
                            var amount = parseInt(jQuery(amount_obj).val());
                            
                            for(var j = 0; j < crbcRequiredElementsMinimum.length; j++){

                                if(jQuery("#"+crbcRequiredElementsMinimum[j].id).get(0) == crbcRequiredElements[i] && ( isNaN(amount) || amount <= 0 || amount < crbcRequiredElementsMinimum[j].min_amount ) ){
                                    crbcCartValidationErrorElement = crbcRequiredElements[i];
                                    return false;
                                }
                            }
                        }
                    break;
                }
            }
            
        }
    }
    return true;
}
';
    }

    public static function importAttributes($product_ids, JModelLegacy $model, $reorder = true, $publish = false){

        $db = JFactory::getDBO();
        
        $atts_updated = 0;
        $att_items_updated = 0;
        $published = 0;
        if($publish){
            $published = 1;
        }

        $row		= $model->getTable('productattributes');
        $groupings	= array();

        foreach($product_ids As $product_id){

            $db->setQuery("Select id From #__breezingcommerce_attributes");
            $attribute_ids = $db->loadColumn();
            
            foreach($attribute_ids As $attribute_id){

                $db->setQuery("Select id From #__breezingcommerce_product_attributes Where attribute_id = " . $attribute_id . " And product_id = " . $product_id . " Limit 1");
                $product_attribute_id = $db->loadResult();

                if(!$product_attribute_id)
                {
                    $db->setQuery("Insert Into #__breezingcommerce_product_attributes (`product_id`,`attribute_id`,`published`) Values (".$product_id.",".$attribute_id.",$published)");
                    $db->query();
                    
                    $product_attribute_id = $db->insertid();
                    $atts_updated++;
                }


                $db->setQuery("Select id From #__breezingcommerce_attribute_items Where attribute_id = " . $attribute_id);
                $attribute_item_ids = $db->loadColumn();
            
                foreach($attribute_item_ids As $attribute_item_id){

                    $db->setQuery("Select id From #__breezingcommerce_product_attribute_items Where attribute_item_id = $attribute_item_id And attribute_id = " . $attribute_id . " And product_id = " . $product_id);
                    $product_attribute_item_id = $db->loadResult();

                    if(!$product_attribute_item_id)
                    {
                        $db->setQuery("Insert Into #__breezingcommerce_product_attribute_items (`product_id`,`attribute_item_id`,`product_attribute_id`,`attribute_id`,`published`) Values ($product_id,".$attribute_item_id.",".$product_attribute_id.",".$attribute_id.",$published)");
                        $db->query();
                        $att_items_updated++;
                    }
                }

                if($reorder)
                {
                    $model->getTable('productattributes')->reorder('attribute_id = '.$attribute_id.' And product_id = ' . $product_id);
                    $model->getTable('productattributeitems')->reorder('attribute_id = '.$attribute_id.' And product_attribute_id = '.$product_attribute_id.' And product_id = ' . $product_id);
                }
            }
        }

        return JText::_('Attributes Updated') . ': ' . $atts_updated . ', ' . JText::_('COM_BREEZINGCOMMERCE_ATTRIBUTE_ITEMS_UPDATED') . ': ' . $att_items_updated;
    }
}
