<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

if(!defined('DS')){
    define('DS','/');
}

require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'CrBcHelpers.php');

if(!class_exists('TCPDF')){
    require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'libraries'.DS.'tcpdf'.DS.'tcpdf.php');
}

class CrBcPDF extends TCPDF{
    
    public $order_cancelled = false;
    
    public function __construct($orientation='P', $unit='mm', $format='A4', $unicode=true, $encoding='UTF-8', $diskcache=false, $pdfa=false){
        
        parent::__construct($orientation, $unit, $format, $unicode, $encoding, $diskcache, $pdfa);
        
    }
    
    function Footer(){
        
        $pdf = $this;
        
        $invoice_folder = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'invoice';
        if(!JFolder::exists(JPATH_SITE . DS . 'media' . DS . 'breezingcommerce')) JFolder::create(JPATH_SITE . DS . 'media' . DS . 'breezingcommerce');
        if(!JFolder::exists($invoice_folder)) JFolder::create($invoice_folder);
        if(!JFolder::exists($invoice_folder.DS.'invoices')) JFolder::create($invoice_folder.DS.'invoices');
        $def = '';
        if(!JFile::exists($invoice_folder.DS.'invoices'.DS.'index.html')) JFile::write($invoice_folder.DS.'invoices'.DS.'index.html', $def);
        $def = 'deny from all';
        if(!JFile::exists($invoice_folder.DS.'invoices'.DS.'.htaccess')) JFile::write($invoice_folder.DS.'invoices'.DS.'.htaccess', $def);

        $templates_folder = $invoice_folder . DS . 'templates';
        if(!JFolder::exists($templates_folder)) JFolder::create($templates_folder);
        
        $active_found = CrBcCart::setCustomFont($pdf);
        
        if(!$active_found){
            $this->SetFont('courier','',9);
        }
        
        $this->SetY(-25);
        
        // PDF VERSION
        ob_start();
        
        if(JFile::exists($templates_folder . DS . 'invoice_tpl_footer_pdf.php')){
            require($templates_folder . DS . 'invoice_tpl_footer_pdf.php');
        }else{
            require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_tpl_footer_pdf.php');
        }
        
        $contents = ob_get_contents();
        ob_end_clean();
        
        if($contents == ''){
            $contents = '<div style="font-size: 8pt;"><table width="100%" border="0"><tr><td style="width: 80%;height:150mm;">&nbsp;</td><td style="width: 20%; height:150mm; text-align: right;padding-left: 25%;">'.str_repeat('&nbsp;<br/>', 4).$this->getAliasNumPage().' / '.$this->getAliasNbPages().'</td></tr></table></div>';
        }
        
        $this->writeHTML($contents, true, true, true, false, '');
        
    }
 }

class CrBcCart{

    static $BACKORDERS_DISALLOWED_VISIBLE = 'DISALLOWED_VISIBLE';
    static $BACKORDERS_DISALLOWED_HIDDEN = 'DISALLOWED_HIDDEN';

    public $cart = array();
    public $currency_id = 0;
    public $currency_symbol = '$';
    public $currency_code = 'USD';
    public $currency_conversion_rate = 1;
    
    // currency info may be unset by default
    // as it's only for displaying purposes
    public $currency_id_info = 0;
    public $currency_symbol_info = null;
    public $currency_code_info = null;
    public $currency_conversion_rate_info = null;
    
    public $group = null;
    
    static $max_age = '24:00:00';
    static $order_id = 0;
    // a list of product tax class ids that shouldn't be taken into account if taxes are supposed to be excluded
    static $exclude_from_no_tax_list = array();
    
    function  __construct($cart) {
        
        if(!is_array($cart)){
            
            $cart = array();
        }
        
        $db = JFactory::getDBO();
        $this->cart = $cart;
        
        // force the cart to be wiped if it's already checked out
        if(isset($this->cart['order_id']) && $this->cart['order_id']){
            
            self::$order_id = $this->cart['order_id'];
            
            $db->setQuery("Select checked_out From #__breezingcommerce_orders Where id = " . $db->quote(self::$order_id));
            
            if( $db->loadResult() == 1 ){
                
                $this->clear();
            }
        }
        
        // fix stale session cart
        // and general cart integrity
        if(isset($this->cart['order_id']) && $this->cart['order_id']){
            
            // changing the order's customer id  as well as the cart's one
            // if the user is logged in and there is a mismatch
            if(JFactory::getApplication()->isSite() && JFactory::getUser()->get('id',0) != 0){
                
               // is there a mismatch?
               $db->setQuery("Select userid, is_tmp From #__breezingcommerce_customers Where id = " . intval($this->cart['customer_id']) . " Limit 1");
               $customer_result = $db->loadObject();
               
               $customer_found = false;
               if( $customer_result !== null ){
                    $userid = $customer_result->userid;
                    $is_tmp = $customer_result->is_tmp;
                    $customer_found = true;
               }
               
               if($customer_found && $userid != JFactory::getUser()->get('id',0)){
                   
                    //mismatch found, now check if the user has a customer id yet
                    $db->setQuery("Select id From #__breezingcommerce_customers Where userid = " . intval(JFactory::getUser()->get('id',0)) . " Limit 1");
                    $customer_id = $db->loadResult();
                    
                    if($customer_id){
                        
                        // if it's not a temp customer, move the existing data from the mismatched session to the customer table
                        // (done by updating userid and customer id)
                        if(!$is_tmp){
                            // yes, he has one, delete the old customer row
                            $db->setQuery("Delete From #__breezingcommerce_customers Where id = " . intval($customer_id));
                            $db->execute();
                            // re-assign the customer and user id to latest entry made
                            $db->setQuery("Update #__breezingcommerce_customers Set userid = ".JFactory::getUser()->get('id',0).", id = ".intval($customer_id)." Where id = " . intval( $this->cart['customer_id'] ));
                            $db->execute();
                        
                        // if the customer is yet temporary but a user, delete the mismatched temporary customer
                        }else{
                            $db->setQuery("Delete From #__breezingcommerce_customers Where id = " . intval($this->cart['customer_id']));
                            $db->execute();
                        }
                        // and update the cart such that it belongs to the current user
                        $this->cart['customer_id'] = $customer_id;
                        JFactory::getSession()->set('crbc_cart', $this->cart);
                        $db->setQuery("Update #__breezingcommerce_orders Set customer_id = ".intval($customer_id)." Where id = " . intval( $this->cart['order_id'] ));
                        $db->execute();
                    }
               }
            }
            
            $query = ' Select count(id) From #__breezingcommerce_orders '.
                        '  Where id = '.intval($this->cart['order_id']);
            $db->setQuery( $query );
            $order_result = $db->loadResult();
            
            $db->setQuery("Select id From #__breezingcommerce_customers Where id = " . intval($this->cart['customer_id']) . " Limit 1");
            $customer_result = $db->loadResult();
            
            if(!$order_result || !$customer_result){
                $this->clear();
                $new_cart = new CrBcCart( JFactory::getSession()->get('crbc_cart', array()) );
                $this->cart = $new_cart->getArray();
            }
            
        }
        
        // having things done in the checkout array but no order would lead to a instable cart as well
        if(!isset($this->cart['order_id']) && isset($this->cart['checkout'])){
            $this->clear();
            $new_cart = new CrBcCart( JFactory::getSession()->get('crbc_cart', array()) );
            $this->cart = $new_cart->getArray();
        }
        
        // making sure the userid stored in the checkout vars will be the one from the logged on user.
        // this can for example happen if a user is in the checkout screen, registering but logging in with a different user
        if(JFactory::getUser()->get('id',0) > 0 && isset($this->cart['order_id']) && isset($this->cart['checkout']) && isset($this->cart['checkout']['userid'])){
            
            $this->cart['checkout']['userid'] = JFactory::getUser()->get('id',0);
            JFactory::getSession()->set('crbc_cart', $this->getArray());
        }
        
        $this->purgeExpiredCart();

        // find out if the current order id has been set to history and if yes, use its currency stuff
        // otherwise create the currency freshly
        // TODO: find out if it might be useful only in administration backend when checking orders
        $found = false;
        if(isset($this->cart['order_id']) && $this->cart['order_id']){
            
            $query = ' Select history_symbol, history_conversion_rate, history_currency_code From #__breezingcommerce_orders '.
                        '  Where history = 1 And id = '.intval($this->cart['order_id']);
            $db->setQuery( $query );
            $order_history = $db->loadAssoc();
            if(is_array($order_history)){
                $this->currency_symbol = $order_history['history_symbol'];
                $this->currency_conversion_rate = $order_history['history_conversion_rate'];
                $this->currency_code = $order_history['history_currency_code'];
                $found = true;
            }
        }

        if(!$found){
            
            $result = null;
            
            // force currency based on the current group
            $group = null;
            
            // is there a customer_id and a group attached to it already?
            if( isset( $this->cart['customer_id'] ) && $this->cart['customer_id'] ){
                
                $db->setQuery("
                    Select
                        groups.*
                    From 
                        #__breezingcommerce_customers As customers, 
                        #__breezingcommerce_customergroups As groups 
                    Where 
                        customers.id = " . intval($this->cart['customer_id']) . "
                    And 
                        customers.customergroup_id = groups.id 
                    And 
                        customers.published = 1 
                    And 
                        groups.published = 1");

                $group = $db->loadObject();
                
            } else if(intval(JFactory::getUser()->get('id',0)) != 0){
                
                $db->setQuery("Select Distinct groups.* From #__breezingcommerce_customergroups groups, #__breezingcommerce_customers customers Where groups.published = 1 And customers.published = 1 And customers.customergroup_id = groups.id And customers.userid = " . $db->quote(intval(JFactory::getUser()->get('id',0))) . " Limit 1");
                $group = $db->loadObject();
                
            } else {
                // let's see if there is one determinable by the user's location
                $group = CrBcHelpers::getBestCustomerGroupByLocation(true);
            }
            
            // if not, then let's see the other options
            if( $group === null ){
                
                $db->setQuery("Select *  From #__breezingcommerce_customergroups Where published = 1 And `default` = 1 Limit 1");
                $group = $db->loadObject();
                
            }
            
            $this->group = $group;
            
            $config = CrBcHelpers::getBcConfig();
            
            if( $config->get('allow_currency_selection', 0) && intval(JFactory::getSession()->get('com_breezingcommerce.currency', 0)) != 0 ){
                $db->setQuery("Select id, conversion_rate, symbol, code From #__breezingcommerce_currencies Where id = " . intval(JFactory::getSession()->get('com_breezingcommerce.currency', 0))  . " And published = 1");
                $result = $db->loadObject();
            }
            
            if($group && intval($group->currency_id) != 0){
                $db->setQuery("Select id, conversion_rate, symbol, code From #__breezingcommerce_currencies Where id = " . intval($group->currency_id)  . " And published = 1");
                $result = $db->loadObject();
            }
            
            if($result === null){
                $db->setQuery("Select id, conversion_rate, symbol, code From #__breezingcommerce_currencies Where `default` = 1 And published = 1 Limit 1");
                $result = $db->loadObject();
            }
            
            if($result){
                $this->currency_id = $result->id;
                $this->currency_code = $result->code;
                $this->currency_symbol = $result->symbol;
                $this->currency_conversion_rate = $result->conversion_rate;
            }
            
            // populate currency info if the currency has been forced to help displaying the info beide the main calculations
            if( $group && intval($group->currency_id) != 0 && $config->get('allow_currency_selection', 0) && intval(JFactory::getSession()->get('com_breezingcommerce.currency', 0)) != 0 ){
                $db->setQuery("Select conversion_rate, symbol, code, id From #__breezingcommerce_currencies Where id = " . intval(JFactory::getSession()->get('com_breezingcommerce.currency', 0))  . " And published = 1");
                $result = $db->loadObject();
                // only override if there actually is an info requested and the regular currency info differs from the requested one
                if($result && $result->code != $this->currency_code){
                    $this->currency_id_info = $result->id;
                    $this->currency_code_info = $result->code;
                    $this->currency_symbol_info = $result->symbol;
                    $this->currency_conversion_rate_info = $result->conversion_rate;
                }
            }
        }
        
        //self::purgeExpiredCarts();
    }
    
    public static function setCustomFont(CrBcPDF $pdf){
        
        $active_found = false;
        $font_loaded = false;
        
        if( JFolder::exists(JPATH_SITE.'/media/breezingcommerce/invoice/font/') ){
            
            $sourcePath = JPATH_SITE.'/media/breezingcommerce/invoice/font/';
            if (@file_exists($sourcePath) && @is_readable($sourcePath) && @is_dir($sourcePath) && $handle = @opendir($sourcePath)) {
                while (false !== ($file = @readdir($handle))) {
                    if($file!="." && $file!=".." && self::endsWith(strtolower($file), '.php')) {
                        $file_sep = explode('.', $file);
                        if(count($file_sep) > 1){
                            unset($file_sep[count($file_sep)-1]);
                            $pdf->AddFont(implode('_',$file_sep), '', $sourcePath.$file);
                            $font_loaded = true;
                        }
                    }
                    if($file!="." && $file!=".." && self::endsWith(strtolower($file), '.ttf')) {
                        $file_sep = explode('.', $file);
                        if(count($file_sep) > 1){
                            unset($file_sep[count($file_sep)-1]);
                            //$pdf->addTTFfont($sourcePath.$file, 'TrueTypeUnicode', '', 96);
                            TCPDF_FONTS::addTTFfont($sourcePath.$file, 'TrueTypeUnicode');
                            $font_loaded = true;
                        }
                    }
                    if(self::endsWith(strtolower($file), '_active')){
                        $active = explode('_', $file);
                        if(count($active) > 1){
                            unset($active[count($active)-1]);
                            $pdf->SetFont(implode('_',$active));
                            if($font_loaded){
                                $active_found = true;
                            }
                        }
                    }
                }
                @closedir($handle);
            }
        }
        
        return $active_found;
    }
    
    public static function endsWith($haystack, $needle){
        return $needle === "" || substr($haystack, -strlen($needle)) === $needle;
    }
    
    /**
     * Generates output data for cart item based on the given product into a stdClass destination object
     * 
     * The resulting object can be used for printing in invoices, carts and so on.
     * 
     * @param stdClass $cart_item_src
     * @param stdClass $product
     * @param stdClass $object_dest
     */
    public static function processItemOutputData(stdClass $cart_item_src, stdClass $product, stdClass $object_dest, $is_admin = false){
        
        $db = JFactory::getDbo();
        
        $cart_item = $cart_item_src;
        $prop = $object_dest;
        
        $prop->amount = $cart_item->amount;
        $prop->price_net = $cart_item->price_net < 0 ? 0 : $cart_item->price_net;
        $prop->price_gross = $cart_item->price_gross < 0 ? 0 : $cart_item->price_gross;
        $prop->off = $cart_item->off < 0 ? 0 : $cart_item->off;

        $prop->price_single_net = $cart_item->price_single_net < 0 ? 0 : $cart_item->price_single_net;
        $prop->price_single_gross = $cart_item->price_single_gross < 0 ? 0 : $cart_item->price_single_gross;
        $prop->single_off = $cart_item->single_off < 0 ? 0 : $cart_item->single_off;
        $prop->price_tax_list = $cart_item->price_tax_list;
        $prop->single_price_tax_list = $cart_item->price_single_tax_list;

        $taxes = 0;
        $applied_taxes = '';

        foreach ($cart_item->price_tax_list As $id => $info) {
            
            $translation = CrBcHelpers::loadTranslation($id, 'taxrate');
            
            if($translation !== null){
                $info['name'] = $translation->title;
                $cart_item->price_tax_list[$id]['name'] = $translation->title;
            }
            
            $applied_taxes .= ' - ' . $info['name'] . ' ('.CrBcCart::formatNumber($info['rate']).' %)';
            $taxes += $info['tax'] <= 0 ? 0 : self::round($info['tax']);
        }

        $prop->price_taxes = $taxes;
        $prop->applied_taxes = $applied_taxes . ' - ';

        $taxes = 0;
        $applied_taxes = '';

        foreach ($cart_item->price_single_tax_list As $id => $info) {
            
            $translation = CrBcHelpers::loadTranslation($id, 'taxrate');
            
            if($translation !== null){
                $info['name'] = $translation->title;
                $cart_item->price_single_tax_list[$id]['name'] = $translation->title;
            }
            
            $applied_taxes .= ' - ' . $info['name'] . ' ('.CrBcCart::formatNumber($info['rate']).' %)';
            $taxes += $info['tax'] <= 0 ? 0 : self::round($info['tax']);
        }

        $prop->price_single_taxes = $taxes;
        $prop->single_applied_taxes = $applied_taxes . ' - ';

        if (!$product->virtual_product) {
            $prop->stock = $product->stock;
        } else {
            $prop->stock = '<i>' . JText::_('Virtual Product') . '</i>';
        }

        if(is_array($cart_item->properties)){
            
            $pr = array();
            $pr_admin = array();
            $pr_admin_keys = array();
            
            foreach($cart_item->properties As $property){
                
                if($cart_item->is_combination){
                    
                    if(!$product->virtual_product){
                        $prop->stock = $property['stock'] ? $property['stock'] : 0;
                    }else{
                        $prop->stock = '<i>'.JText::_('Virtual Product').'</i>';
                    }
                }
                
                if(!isset($pr[$property['property_key']])){
                    
                    // keys translations
                    $translation = CrBcHelpers::loadTranslation($property['property_keys_id'], 'property_key_'.$property['property_id']);

                    $property_key_tmp = $property['property_key'];
                    
                    if(is_object($translation)){

                        $property['property_key'] = $translation->title;
                    }
                    
                    $pr[$property['property_key']] = array();
                    $pr_admin[$property['property_key']] = array();
                    $pr_admin_keys[$property['property_key']] = $property_key_tmp;
                }
                
                if( $property['custom_value'] == '' ){
                    
                    // values translations
                    $translation = CrBcHelpers::loadTranslation($property['property_values_id'], 'property_value_'.$property['property_values_key_id']);

                    $tmp_name = $property['value'];
                    
                    if(is_object($translation)){
                        $property['value'] = $translation->title;
                    }

                    $pr[$property['property_key']][] = $property['value'];
                    $pr_admin[$property['property_key']][] = $tmp_name;
                    
                }
                else
                {
                    $property['value'] = $property['custom_value'];
                    
                    // values translations
                    $translation = CrBcHelpers::loadTranslation($property['productpropertyvalue_id'], 'productpropertyvalue');

                    $tmp_name = $property['value'];
                    
                    if(is_object($translation)){
                        $property['value'] = $translation->body;
                    }

                    $pr[$property['property_key']][] = $property['value'];
                    $pr_admin[$property['property_key']][] = $tmp_name;
                }
            }

            foreach($pr As $key => $values){
                $prop->properties .= '<div class="crbc-property-processed"><strong>' . $key . ':</strong> ' . implode(',',$values) . '</div>' . "\n";
            }
            
            foreach($pr_admin As $key => $values){
                $prop->properties_admin .= '<div class="crbc-property-processed"><strong>' . $pr_admin_keys[$key] . ':</strong> ' . implode(',',$values) . '</div>' . "\n";
            }
        }

        if(is_array($cart_item->attributes)){
            
            $attributes_nostock_invoice = '';
            $attributes_nostock = '';
            $attributes_nostock_admin = '';
            $attributes_text = '';
            $attributes = '';
            $attributes_admin = '';
            $attrs = array();
            $attrs_admin = array();
            
            foreach($cart_item->attributes As $attribute){
               
                $tmp_key = $attribute['attribute_name'];
                
                if(!isset($attrs[$attribute['attribute_name']])){
                    
                    
                    $translation = CrBcHelpers::loadTranslation($attribute['attribute_id'], 'attribute');

                    if(is_object($translation)){

                        $attribute['attribute_name'] = $translation->title;
                    }
                    
                    $attrs[$attribute['attribute_name']] = array();
                }
                $attrs[$attribute['attribute_name']][] = $attribute;
                $attrs_admin[$attribute['attribute_name']] = $tmp_key;
            }

            if( count($attrs) ){
                
                foreach ($attrs As $key => $values) {

                    $attributes .= '<div class="crbc-attributes-processed"><strong>' . $key . '</strong><ul>'."\n";
                    $attributes_admin .= '<div class="crbc-attributes-processed"><strong>' . $attrs_admin[$key] . '</strong><ul>'."\n";
                    $attributes_nostock .= '<div class="crbc-attributes-processed"><strong>' . $key . '</strong><ul>'."\n";
                    $attributes_nostock_admin .= '<div class="crbc-attributes-processed"><strong>' . $attrs_admin[$key] . '</strong><ul>'."\n";
                    $attributes_nostock_invoice .= '<div class="crbc-attributes-processed"><strong>' . $key . '</strong><ul>'."\n";
                    $attributes_text .= '- ' . $key ."\n";

                    foreach ($values As $value) {
                        
                        $translation = CrBcHelpers::loadTranslation($value['id'], 'attribute_item_'.$value['attribute_id']);
            
                        $attributes_name_tmp = $value['name'];
                        
                        if(is_object($translation)){

                            $value['name'] = $translation->title;
                        }
                        
                        $attributes_admin .= '<li>' . $cart_item->attribute_amounts[$value['product_attribute_item_id']] . ' x ' . $attributes_name_tmp . '';
                        $attributes_nostock_admin .= '<li>' . $cart_item->attribute_amounts[$value['product_attribute_item_id']] . ' x ' . $attributes_name_tmp . '';
                        
                        $attributes .= '<li>' . $cart_item->attribute_amounts[$value['product_attribute_item_id']] . ' x ' . $value['name'] . '';
                        $attributes_nostock .= '<li>' . $cart_item->attribute_amounts[$value['product_attribute_item_id']] . ' x ' . $value['name'] . '';
                        $attributes_nostock_invoice .= '<li>' . $cart_item->attribute_amounts[$value['product_attribute_item_id']] . ' x ' . $value['name'] . '';
                        $attributes_text .= ' -- ' . $cart_item->attribute_amounts[$value['product_attribute_item_id']] . ' x ' . $value['name'] . ',';

                        $pid = 0;
                        if ($value['product_attribute_item_product_id']) {
                            $pid = $value['product_attribute_item_product_id'];
                        } else if ($value['item_product_id']) {
                            $pid = $value['product_id'];
                        }
                        
                        $prod_title_admin = '';
                        $prod_title_nostock_admin = '';

                        $prod_title = '';
                        $prod_title_nostock = '';
                        
                        if ($pid) {
                            $db->setQuery("Select `id`, `title`, `stock`,`alias` From #__breezingcommerce_products Where id = " . intval($pid));
                            $prod = $db->loadObject();
                            if ($prod) {
                                
                                $prod_title_admin = ' ' . "\n" . '<div class="crbc-attribute-link-admin"><a href="index.php?option=com_breezingcommerce&amp;controller=products&amp;task=edit&amp;cid[]=' . $prod->id . '">' . $prod->title . "</a></div><div class=\"crbc-attribute-stock-admin\">(".JText::_('COM_BREEZINGCOMMERCE_STOCK').": " . $prod->stock . ")</div>\n";
                                $prod_title_nostock_admin = ' ' . "\n" . '<div class="crbc-attribute-link-admin"><a href="index.php?option=com_breezingcommerce&amp;controller=products&amp;task=edit&amp;cid[]=' . $prod->id . '">' . $prod->title . "</a></div>{attr_stock".$value['product_attribute_item_id']."}\n";


                                $translation = CrBcHelpers::loadTranslation($prod->id, 'product');

                                if(is_object($translation)){

                                    $prod->title = $translation->title;
                                }

                                $prod_title = ' ' . "\n" . '<div class="crbc-attribute-link-site"><a href="'.JRoute::_('index.php?option=com_breezingcommerce&perm=1&alias='.$prod->alias.'&controller=product&product_id=' . $prod->id . '&Itemid='.CrBcHelpers::getDefaultMenuItemId()) . '">' . $prod->title . "</a></div>\n";
                                // same for site
                                $prod_title_nostock = $prod_title;
                                
                            }
                        }

                        $attributes = rtrim($attributes, ',') . ' ' . $prod_title . '</li>' . "\n";
                        $attributes_nostock = rtrim($attributes_nostock, ',') . ' ' . $prod_title_nostock . '</li>' . "\n";
                        $attributes_nostock_invoice = $attributes_nostock_invoice . '</li>' . "\n";
                        $attributes_text = rtrim($attributes_text, ',') . "\n";
                        
                        $attributes_admin = rtrim($attributes_admin, ',') . ' ' . $prod_title_admin . '</li>' . "\n";
                        $attributes_nostock_admin = rtrim($attributes_nostock_admin, ',') . ' ' . $prod_title_nostock_admin . '</li>' . "\n";
                        
                    }

                    $attributes_nostock_invoice .= '</ul></div>';
                    $attributes .= '</ul></div>';
                    $attributes_nostock .= '</ul></div>';

                    $attributes_admin .= '</ul></div>';
                    $attributes_nostock_admin .= '</ul></div>';
                    
                    $prop->attributes = $attributes;
                    
                    $prop->attributes_admin = $attributes_admin;
                    $prop->attributes_nostock_admin = $attributes_nostock_admin;
                    
                    $prop->attributes_nostock = $attributes_nostock;
                    $prop->attributes_nostock_invoice = $attributes_nostock_invoice;
                    // required for packing slip
                    $prop->attributes_invoice = $attributes_nostock_invoice;
                    //////
                    $prop->attributes_text = $attributes_text;
                }
            }
        }
    }

    public static function getData($order_id, array $cart_items, $limitstart = -1, $limit = -1, $is_admin = false)
    {
        $db = JFactory::getDbo();
        
        $query = "Select * From #__breezingcommerce_order_items Where order_id = " . intval( $order_id ) . " Order By `creation_date` Desc ";
        
        if(intval($limitstart) > 0 && intval($limit) > 0){
            $db->setQuery($query, $limitstart, $limit);
        } else {
            $db->setQuery($query);
        }
        
        $entries = $db->loadObjectList();
        $props = array();

        $cart_items_map = array();
        $cart_items_size = count($cart_items);
        for($i = 0; $i < $cart_items_size; $i++){
            $cart_items_map[$cart_items[$i]->order_item_id] = $i;
        }

        $i = 0;
        foreach( $entries As $entry ){

            if(!isset($props[$i]))
            {
                $props[$i] = new stdClass();
            }

            if(!$entry->history){

                $db->setQuery("Select `title`, `id`, `stock`, `virtual_product`, `attribs`, `weight`, `weight_type` From #__breezingcommerce_products Where id = " . $entry->product_id . " Limit 1");
                $product = $db->loadObject();

                $props[$i]->history = $entry->history;
                $props[$i]->id = $entry->id;
                $props[$i]->product_id = $entry->product_id;
                $props[$i]->order_item_id = $entry->id;
                $props[$i]->price_net = 0;
                $props[$i]->price_single_net = 0;
                $props[$i]->price_single_gross = 0;
                $props[$i]->price_gross = 0;
                $props[$i]->price_taxes = 0;
                $props[$i]->applied_taxes = '';
                $props[$i]->properties = '';
                $props[$i]->properties_admin = '';
                $props[$i]->attributes = '';
                $props[$i]->order_item_info = $entry->order_item_info != '' ? json_decode($entry->order_item_info) : array();
                $props[$i]->attributes_admin = '';
                $props[$i]->attributes_text = '';
                $props[$i]->attributes_nostock = '';
                $props[$i]->attributes_nostock_admin = '';
                $props[$i]->attributes_nostock_invoice = '';
                $props[$i]->amount = 0;
                $props[$i]->off = 0;
                $props[$i]->weight = $product ? $product->weight : 0;
                $props[$i]->weight_type = $product ? $product->weight_type : 0;
                $props[$i]->stock = $product ? $product->stock : 0;
                $props[$i]->price_tax_list = array();
                $props[$i]->single_price_tax_list = array();
                $props[$i]->attribs = $product->attribs;
                
                if(isset($cart_items_map[$entry->id])){

                    $cart_item = $cart_items[$cart_items_map[$entry->id]];
                    // processes stdClass cart item data (src) and applies them to a stdClass object (target)
                    self::processItemOutputData( $cart_item, $product, $props[$i], $is_admin );
                }

                $props[$i]->creation_date = $entry->creation_date;
                $props[$i]->published = $entry->published;
                $props[$i]->verified = $entry->verified;
                $props[$i]->ordering = $entry->ordering;
                $props[$i]->verification_expiration_date = $entry->verification_expiration_date;
                
                $props[$i]->title = '';

                if($product)
                {
                    $props[$i]->title_admin = $product->title;
                    
                    $translation = CrBcHelpers::loadTranslation($product->id, 'product');
            
                    if(is_object($translation)){

                        $product->title = $translation->title;
                    }
                    
                    $props[$i]->title = $product->title;
                }
                
            } else {

                $db->setQuery("Select `title`, `id`, `stock`, `virtual_product`, `attribs`, `weight`, `weight_type` From #__breezingcommerce_products Where id = " . $entry->product_id . " Limit 1");
                $product = $db->loadObject();

                $props[$i]->history = $entry->history;
                $props[$i]->id = $entry->id;
                $props[$i]->product_id = !$product ? 0 : $entry->product_id;
                $props[$i]->weight = $product ? $product->weight : 0;
                $props[$i]->weight_type = $product ? $product->weight_type : 0;
                $props[$i]->order_item_id = $entry->id;
                $props[$i]->title = $entry->history_title;
                $props[$i]->title_admin = $entry->history_title_admin;
                $props[$i]->price_net = $entry->history_net;
                $props[$i]->price_gross = $entry->history_gross;
                $props[$i]->price_taxes = $entry->history_taxes;
                $props[$i]->applied_taxes = $entry->history_applied_taxes;

                $props[$i]->price_single_net = $entry->history_single_net;
                $props[$i]->price_single_gross = $entry->history_single_gross;
                $props[$i]->price_single_taxes = $entry->history_single_taxes;
                $props[$i]->single_applied_taxes = $entry->history_single_applied_taxes;
                $props[$i]->price_tax_list = @unserialize($entry->history_price_tax_list);
                $props[$i]->single_price_tax_list = @unserialize($entry->history_price_single_tax_list);

                $props[$i]->properties = $entry->history_properties;
                $props[$i]->properties_admin = $entry->history_properties_admin;
                
                $attribute_sets = explode(',',$entry->attributes);
                foreach($attribute_sets As $attribute_set){
                    $attribute = explode('=', $attribute_set);
                    if(isset($attribute[0])){
                        $pstock = 0;
                        $db->setQuery("Select Distinct pai.item_product_id, ai.product_id From #__breezingcommerce_product_attribute_items As pai, #__breezingcommerce_attribute_items As ai Where ai.id = pai.attribute_item_id And pai.id = ".intval($attribute[0]));
                        $pai = $db->loadAssoc();
                        if(is_array($pai)){
                            $pid = 0;
                            if($pai['item_product_id']){
                                $pid = $pai['item_product_id'];
                            }else if($pai['product_id']){
                                $pid = $pai['product_id'];
                            }
                            $db->setQuery("Select stock From #__breezingcommerce_products Where id = ".intval($pid));
                            $pstock = $db->loadResult();
                            if(!$pstock){
                                $pstock = 0;
                            }
                        }
                        
                        $entry->history_attributes = str_replace('{attr_stock'.intval($attribute[0]).'}',' ('.JText::_('COM_BREEZINGCOMMERCE_STOCK').': '.$pstock.')',$entry->history_attributes);
                        $entry->history_attributes_admin = str_replace('{attr_stock'.intval($attribute[0]).'}',' ('.JText::_('COM_BREEZINGCOMMERCE_STOCK').': '.$pstock.')',$entry->history_attributes_admin);
                    }
                }
                $props[$i]->attributes = $entry->history_attributes;
                $props[$i]->order_item_info = $entry->order_item_info != '' ? json_decode($entry->order_item_info) : array();
                $props[$i]->attributes_admin = $entry->history_attributes_admin;
                $props[$i]->attributes_invoice = $entry->history_attributes_invoice;
                $props[$i]->attributes_text = $entry->history_attributes_text;

                $props[$i]->amount = $entry->history_amount;
                $props[$i]->off = $entry->history_discount;
                $props[$i]->single_off = $entry->history_single_discount;
                $props[$i]->stock = $product ? $product->stock : 0;

                $properties = explode(',', $entry->properties);

                if ($product && !$product->virtual_product) {
                    $props[$i]->stock = $product->stock;
                } else if($product) {
                    $props[$i]->stock = '<i>' . JText::_('Virtual Product') . '</i>';
                }

                foreach($properties As $property){
                    if ($entry->is_combination) {
                        if ($product && !$product->virtual_product) {
                            $db->setQuery("Select stock, weight, weight_type From #__breezingcommerce_product_property_combinations Where id = " . intval($property));
                            $combination = $db->loadObject();
                            $props[$i]->stock = $combination ? $combination->stock : 0;
                            $props[$i]->weight = $combination ? $combination->weight : 0;
                            $props[$i]->weight_type = $combination ? $combination->weight_type : 0;
                        } else if($product) {
                            $props[$i]->stock = '<i>' . JText::_('Virtual Product') . '</i>';
                            $props[$i]->weight = 0;
                            $props[$i]->weight_type = 0;
                        }
                        break;
                    }
                }

                $props[$i]->creation_date = $entry->creation_date;
                $props[$i]->published = $entry->published;
                $props[$i]->verified = $entry->verified;
                $props[$i]->ordering = $entry->ordering;
                $props[$i]->verification_expiration_date = $entry->verification_expiration_date;
                
                if($product && isset($product->attribs)){
                    $props[$i]->attribs = $product->attribs;
                }else{
                    $props[$i]->attribs = '{}';
                }
            }

            $db->setQuery("Select id From #__breezingcommerce_files Where product_id = " . intval($entry->product_id) . " And requires_verification = 1 Limit 1");
            $props[$i]->has_orderitemfiles = $db->loadResult() ? true : false;

            // images
            $db->setQuery("Select * From #__breezingcommerce_images Where product_id = " . intval($entry->product_id) . " And published = 1 Order By `ordering`");
            $props[$i]->images = $db->loadObjectList();
            
            $i++;
        }

        return $props;
    }
    
    public static function isVirtualOrder(array $cart_items){
        
        $db = JFactory::getDbo();
        
        $is_virtual_order = true;
        $test_products = array();

        foreach ($cart_items As $cart_item) {

            if (!$cart_item->virtual_product) {
                $is_virtual_order = false;
                break;
            }

            // trying attributes
            if(isset($cart_item->attributes) && is_array($cart_item->attributes)){
                foreach ($cart_item->attributes As $cart_item_attribute) {

                    $db->setQuery("Select Distinct pai.item_product_id, ai.product_id From #__breezingcommerce_product_attribute_items As pai, #__breezingcommerce_attribute_items As ai Where ai.id = pai.attribute_item_id And pai.id = " . intval($cart_item_attribute['product_attribute_item_id']));
                    $pai = $db->loadAssoc();

                    if (is_array($pai)) {
                        $pai_pid = 0;
                        if ($pai['item_product_id']) {
                            $pai_pid = $pai['item_product_id'];
                        } else if ($pai['product_id']) {
                            $pai_pid = $pai['product_id'];
                        }
                        if ($pai_pid && !in_array($pai_pid, $test_products)) {
                            $test_products[] = $pai_pid;
                        }
                    }
                }
            }
        }

        if (count($test_products)) {
            $db->setQuery("Select count(id) From #__breezingcommerce_products Where virtual_product = 0 And id In (" . implode(',', $test_products) . ")");
            if ($db->loadResult()) {
                $is_virtual_order = false;
            }
        }

        return $is_virtual_order;
    }
    
    public static function getOrder(
            $order_id, 
            CrBcCart $cart_instance, 
            array $cart_array, 
            array $cart_items, 
            $customer,
            array $items,
            array $plugins,
            array $payment_plugins
    )
    {
        $db = JFactory::getDbo();
        
        $query = ' Select * From #__breezingcommerce_orders '.
                    '  Where id = '.intval($order_id);
        $db->setQuery( $query );
        $data = $db->loadObject();
        
        if (!$data) {
            
            $data = new stdClass();
            $data->id = 0;
            $data->customer_id = null;
            $data->customer = null;
            $data->creation_date = null;
            $data->checked_out = null;
            $data->published = null;
            $data->send_notification = null;
            $data->customer_name = null;
            $data->cart_instance = new CrBcCart( array() );
            $data->shipping_options = array();
            $data->payment_options = array();
            $data->paid = 0;
            $data->price_net = 0;
            $data->price_gross = 0;
            $data->grand_total = 0;
            $data->price_tax_list = array();
            $data->off = 0;
            $data->taxes = 0;
            $data->shipping_costs = 0;
            $data->order_number = '';
            $data->order_display = '';
            $data->weight = array();

        } else {
            
            $data->cart_instance = $cart_instance;
            $data->payment_options = array();
            $data->shipping_options = array();
            $data->shipping_costs = 0;
            $data->order_display = '';

            if(!$data->checked_out){
                
                $data->customer_id = $cart_array['customer_id'];
                $data->customer_name = $cart_array['customer_name'];
                
                if($customer instanceof stdClass){
                    
                    $data->customer_name = '<a href="index.php?option=com_breezingcommerce&amp;controller=customers&amp;task=edit&amp;cid[]='.$customer->id.'&amp;return_order='.$order_id.'">'.$data->customer_name.'</a>';
                
                }else{
                    
                    $db->setQuery("Select * From #__breezingcommerce_customers Where id = " . intval($data->customer_id));
                    $customer = $db->loadObject();
                
                    if($customer instanceof stdClass){
                        $data->customer_name = '<a href="index.php?option=com_breezingcommerce&amp;controller=customers&amp;task=edit&amp;cid[]='.$customer->id.'&amp;return_order='.$order_id.'">'.$data->customer_name.'</a>';
                    }
                }
                
                if($customer instanceof stdClass){
                
                    $customer = self::prepareCustomerOutput($customer);
                    
                }
                
            }else{
                
                $data->customer_id = $data->history_customer_id;
                $data->customer_name = $data->history_customer_name;
                
                $customer = @unserialize($data->history_customer);
                
                $customer = self::prepareCustomerOutput($customer);
                
                // TODO: create links if customer still exists
                $db->setQuery("Select id From #__breezingcommerce_customers Where id = " . intval($customer->id));
                $check_customer = $db->loadResult();
                if($check_customer){
                    $data->customer_name = '<a href="index.php?option=com_breezingcommerce&amp;controller=customers&amp;task=edit&amp;cid[]='.$customer->id.'&amp;return_order='.$order_id.'">'.$data->customer_name.'</a>';
                }
            }

            $data->customer = $customer;

            $taxes = 0;
            $price_net = 0;
            $price_gross = 0;
            $data->price_net = 0;
            $data->price_gross = 0;
            $data->grand_total = 0;
            $data->price_tax_list = array();
            $data->off = 0;
            $data->weight = array();
            
            $cart_items_map = array();
            $cart_items_size = count($cart_items);
            for($i = 0; $i < $cart_items_size; $i++){
                $cart_items_map[$cart_items[$i]->order_item_id] = $i;
            }

            foreach($items As $item){
                
                if(!isset($data->weight[$item->weight_type])){
                    $data->weight[$item->weight_type] = 0;
                }
                
                $data->weight[$item->weight_type] += $item->weight;
                
                if(isset($item->history) && !$item->history){
                    
                    $cart_item = $cart_items[$cart_items_map[$item->order_item_id]];
                    $price_net += $cart_item->price_net <= 0 ? 0 : $cart_item->price_net;
                    $price_gross += $cart_item->price_gross <= 0 ? 0 : $cart_item->price_gross;
                    foreach ($cart_item->price_tax_list As $id => $info) {
                        if(!isset($data->price_tax_list[$id])){
                            $data->price_tax_list[$id]['tax'] = 0;
                            $data->price_tax_list[$id]['rate'] = $info['rate'];
                            $data->price_tax_list[$id]['name'] = $info['name'];
                        }
                        $taxes += $info['tax'] <= 0 ? 0 : self::round($info['tax']);
                        $data->price_tax_list[$id]['tax'] += $info['tax'] <= 0 ? 0 : $info['tax'];
                        $data->off = $cart_item->off;
                    }
                    
                } else if(isset($item->history)){
                    
                    $price_net += $item->price_net <= 0 ? 0 : $item->price_net;
                    $price_gross += $item->price_gross <= 0 ? 0 : $item->price_gross;
                    $taxes += $item->price_taxes <= 0 ? 0 : self::round($item->price_taxes);
                    foreach ($item->price_tax_list As $id => $info) {
                        if(!isset($data->price_tax_list[$id])){
                            $data->price_tax_list[$id]['tax'] = 0;
                            $data->price_tax_list[$id]['rate'] = $info['rate'];
                            $data->price_tax_list[$id]['name'] = $info['name'];
                        }
                        $data->price_tax_list[$id]['tax'] += $info['tax'] <= 0 ? 0 : $info['tax'];
                        $data->off = $item->off;
                    }
                }
            }
           
            $data->taxes = $taxes;
            $data->price_net = $price_net;
            $data->price_gross = $price_gross;

            if(!$data->invoice_created){
                $is_virtual_order = self::isVirtualOrder($cart_items);
                if(!$is_virtual_order && is_array($plugins)){
                    $i = 0;
                    foreach($plugins As $plugin){
                        $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], JFactory::getApplication()->isAdmin() ? 'admin' : 'site');
                        if($plugin_instance instanceof CrBcShippingPlugin){
                            $plugin_instance->init($cart_instance);
                            $plugin_instance->determineBestShippingOptions();
                            if($plugin['id'] == $data->plugin_id){
                                $producttaxclass_id = $plugin_instance->getProductTaxclassId($data->selected_shipping_id);
                                self::$exclude_from_no_tax_list[] = $producttaxclass_id;
                                $price = $plugin_instance->getPrice($data->selected_shipping_id);
                                $price_calc = CrBcCart::getPrice($price, 1, $producttaxclass_id, 1, $data->customer_id);
                                $data->shipping_costs = $price_calc['gross'];
                            }
                            $options = $plugin_instance->onDisplayOptions($plugin['id'] == $data->plugin_id ? $data->selected_shipping_id : null);
                            if($options){
                                $data->shipping_options[$i]['options'] = $options;
                                $data->shipping_options[$i]['title'] = $plugin_instance->getPluginDisplayName();
                                $data->shipping_options[$i]['id'] = $plugin['id'];
                                $i++;
                            }
                        }
                    }
                }

                if(is_array($payment_plugins)){
                    $i = 0;
                    foreach($payment_plugins As $payment_plugin){
                        $plugin_instance = CrBcPlugin::getPluginInstance($payment_plugin['name'], $payment_plugin['type'], JFactory::getApplication()->isAdmin() ? 'admin' : 'site');
                        
                        if($plugin_instance instanceof CrBcPaymentPlugin){
                            if( method_exists($plugin_instance, 'isPaymentSuitable') && $plugin_instance->isPaymentSuitable() === false ){
                                // skip, not accepted
                            } else {
                                $data->payment_options[$i]['title'] = $plugin_instance->getPluginDisplayName();
                                $data->payment_options[$i]['description'] = $plugin_instance->getPluginDescription();
                                $data->payment_options[$i]['icon'] = $plugin_instance->getPluginIcon();
                                if( JFactory::getApplication()->isAdmin() ){
                                    $data->payment_options[$i]['order_display'] = $plugin_instance->handleOrderDisplay($order_id);
                                }
                                $data->payment_options[$i]['id'] = $payment_plugin['id'];
                                $i++;
                            }
                        }
                    }
                }

            }else{
                $data->shipping_costs = $data->history_shipping_costs;
                $data->shipping_options = $data->history_shipping_plugin . ': ' . $data->history_shipping_name . ' ('.$cart_instance->formatPrice($data->shipping_costs).')';
                $data->payment_options = $data->history_payment_plugin;
                
                foreach($payment_plugins As $payment_plugin){
                    
                    $plugin_instance = CrBcPlugin::getPluginInstance($payment_plugin['name'], $payment_plugin['type'], JFactory::getApplication()->isAdmin() ? 'admin' : 'site');

                    if($plugin_instance instanceof CrBcPaymentPlugin){
                        if( method_exists($plugin_instance, 'isPaymentSuitable') && $plugin_instance->isPaymentSuitable() === false ){
                            // skip, not accepted
                        } else {
                            
                            if( $data->payment_plugin_id == $payment_plugin['id'] && JFactory::getApplication()->isAdmin() ){
                                $data->order_display = $plugin_instance->handleOrderDisplay($order_id);
                                break;
                            }
                        }
                    }
                }
            }

            
            // determine tax rounding errors
            
            $difftax = 0;
            
            foreach($data->price_tax_list As $key => $value){
                    
                $difftax += self::round($data->price_tax_list[$key]['tax']);
            }
            
            // find out if there are rounding errors against the glbally rounded tax
            $rounderror = self::round($data->price_gross - $data->price_net) - self::round($difftax);
           
            // if there is a rounding error, add the error to the very next item from the tax list
            if($rounderror != 0){
                
                foreach($data->price_tax_list As $key => $value){
                    $data->price_tax_list[$key]['tax'] += self::round($rounderror);
                    break;
                }
                
            }
            
            $data->taxes = $data->price_gross - $data->price_net;
            
            $data->grand_total = $data->price_gross + $data->shipping_costs;

            $db->setQuery("Select * From #__breezingcommerce_order_status Where published = 1 Order By ordering");
            $data->order_status_list = $db->loadObjectList();
        }

        return $data;
    }
    
    public static function updateCustomer($customer_data){
        
        $db = JFactory::getDbo();
        
        $table_path = JPATH_SITE . DS . 'administrator' . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'tables' . DS . 'customer.php';
        
        if(JFile::exists($table_path)) {
            
            require_once($table_path);
            
            if(class_exists('TableCustomer')) {
                
                $table = new TableCustomer($db);
                
                if (!$table->bind($customer_data)) {
                    throw new CrBcCartException('Customer bind failed!');
                }

                if (!$table->check()) {
                    throw new CrBcCartException('Customer check failed!');
                }

                if(!$table->store()){
                    throw new CrBcCartException('Customer storage failed!');
                }
                
                if( intval($customer_data['id']) == 0 )
                {
                    $customer_data['id'] = $db->insertid();
                }
                
                return $customer_data;
            }
        }
        
        return null;
    }

    function getSelectedAttributeItemIds(){
        $attributes = array();
        if(isset($this->cart['products'])){
            foreach($this->cart['products'] As $product){
                if(isset($product['attribute_amounts'])){
                    foreach($product['attribute_amounts'] As $key => $value){
                        if(!isset($attributes[$key])){
                            $attributes[$key] = $value;
                        }
                    }
                }
            }
        }
        
        return $attributes;
    }

    function getSelectedPropertyValueIds(){
        $properties = array();
        if(isset($this->cart['products'])){
            foreach($this->cart['products'] As $product){
                if(isset($product['properties'])){
                    foreach($product['properties'] As $property){
                        if(!in_array($property, $properties)){
                            $properties[] = $property;
                        }
                    }
                }
            }
        }
        
        return $properties;
    }
    
    function updateAmount($product_id, $order_item_id, $amount){
        
        if($product_id < 1 || !is_numeric($product_id)){
            throw new CrBcCartException('Invalid product id!');
        }
        
        if(!isset($this->cart['products']) || !is_array($this->cart['products'])){
            return;
        }
        
        $db = JFactory::getDBO();
        
        $size = count($this->cart['products']);
        for($i = 0; $i < $size; $i++){
            
            if($order_item_id == $this->cart['products'][$i]['order_item_id']){
                
                $db->setQuery("Select attribs From #__breezingcommerce_products Where id = " . intval($product_id));
                $attribs = $db->loadResult();
                
                $product_form = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
                $form = $product_form->loadString($attribs);
                $minimum_amount = intval($form->get('minimum_amount', 1));
                $maximum_amount = intval($form->get('maximum_amount', 99999));
                
                if(intval($amount) < $minimum_amount){
                    $amount = $minimum_amount;
                }
                
                if(intval($amount) > $maximum_amount){
                    $amount = $maximum_amount;
                }
                
                $this->cart['products'][$i]['amount'] = intval($amount);
                $db->setQuery("Update #__breezingcommerce_order_items Set amount = ".$db->quote(intval($amount))." Where id = ".intval($order_item_id)." And order_id = ".$db->quote($this->cart['order_id'])."");
                $db->query();
                JFactory::getSession()->set('crbc_cart', $this->getArray());
                break;
            }
        }
    }
    
    public static function prepareCustomerOutput($customer){
        
        $db = JFactory::getDbo();
        
        if($customer instanceof stdClass){
                
            // customer's country

            $db->setQuery("Select `name`,`id` From #__breezingcommerce_countries Where id = " . intval($customer->country_id));
            $res = $db->loadObject();

            if(isset($res->id)){

                $trans = CrBcHelpers::populateTranslation($res, 'country', array('title' => 'name'));
                $customer->country = isset($trans->name) ? $trans->name : '';
            }

             // customer's shipping country

            $db->setQuery("Select `name`,`id` From #__breezingcommerce_countries Where id = " . intval($customer->shipping_country_id));
            $res = $db->loadObject();

            if(isset($res->id)){

                $trans = CrBcHelpers::populateTranslation($res, 'country', array('title' => 'name'));
                $customer->shipping_country = isset($trans->name) ? $trans->name : '';
            }

            // customer's region

            $db->setQuery("Select `name`,`id` From #__breezingcommerce_country_regions Where id = " . intval($customer->region_id));
            $res = $db->loadObject();

            if(isset($res->id)){

                $trans = CrBcHelpers::populateTranslation($res, 'country_region_' . $customer->country_id, array('title' => 'name'));
                $customer->region = isset($trans->name) ? $trans->name : '';
            }

            // customer's shipping region

            $db->setQuery("Select `name`,`id` From #__breezingcommerce_country_regions Where id = " . intval($customer->shipping_region_id));
            $res = $db->loadObject();

            if(isset($res->id)){

                $trans = CrBcHelpers::populateTranslation($res, 'country_region_' . $customer->shipping_country_id, array('title' => 'name'));
                $customer->shipping_region = isset($trans->name) ? $trans->name : '';
            }
        }
        
        return $customer;
    }
    
    public static function hasAttributes($product_id)
    {
        $db = JFactory::getDBO();
        $db->setQuery("Select count(id) From #__breezingcommerce_product_attributes Where product_id = " . intval($product_id) . " And published = 1 Limit 1");
        return $db->loadResult() != 0 ? true : false;
    }
    
    public static function hasProperties($product_id)
    {
        $db = JFactory::getDBO();
        $db->setQuery("Select count(id) From #__breezingcommerce_product_property_keys Where product_id = " . intval($product_id) . " And published = 1 Limit 1");
        return $db->loadResult() != 0 ? true : false;
    }
    
    function getOrderedPriceGroup($price_array, $currency_symbol = null, $currency_code = null){
        
        if(!is_array($price_array) || count($price_array) == 0){
            return array();
        }
        
        if(!isset($price_array['gross']) || !isset($price_array['net']) || !isset($price_array['tax_list'])){
            return array();
        }
        
        // determine tax rounding errors
            
        $difftax = 0;

        foreach($price_array['tax_list'] As $key => $value){

            $difftax += self::round($price_array['tax_list'][$key]['tax']);
        }

        // find out if there are rounding errors against the glbally rounded tax
        $rounderror = self::round($price_array['gross'] - $price_array['net']) - self::round($difftax);

        // if there is a rounding error, add the error to the very next item from the tax list
        if($rounderror != 0){

            foreach($price_array['tax_list'] As $key => $value){
                $price_array['tax_list'][$key]['tax'] += self::round($rounderror);
                break;
            }

        }
        
        $taxes = 0;
        $applied_taxes = '';

        foreach ($price_array['tax_list'] As $id => $info) {
            
            $translation = CrBcHelpers::loadTranslation($id, 'taxrate');
            
            if($translation !== null){
                $info['name'] = $translation->title;
            }
            
            $applied_taxes .= JText::_('COM_BREEZINGCOMMERCE_PRICE_RATE_INCL').' '.$info['name'] . ': '.CrBcCart::formatNumber($info['rate']).' %<div class="crbc-sub-price-br"></div>';
            $taxes += $info['tax'] <= 0 ? 0 : self::round($info['tax']);
        }
        
        $config = CrBcHelpers::getBcConfig();
        
        $order = $config->get('price_display', 'gross');
        
        switch( $order ){
            case 'gross,net,taxes,rates':
                return array(
                    $this->formatPrice($price_array['gross'], $currency_symbol, $currency_code),
                    JText::_('COM_BREEZINGCOMMERCE_PRICE_NET').': '.$this->formatPrice($price_array['net'], $currency_symbol, $currency_code),
                    JText::_('COM_BREEZINGCOMMERCE_PRICE_TAXES').': '.$this->formatPrice($taxes, $currency_symbol, $currency_code),
                    $applied_taxes
                );
                break;
            case 'net,gross,taxes,rates':
                return array(
                    $this->formatPrice($price_array['net'], $currency_symbol, $currency_code),
                    JText::_('COM_BREEZINGCOMMERCE_PRICE_GROSS').': '.$this->formatPrice($price_array['gross'], $currency_symbol, $currency_code),
                    JText::_('COM_BREEZINGCOMMERCE_PRICE_TAXES').': '.$this->formatPrice($taxes, $currency_symbol, $currency_code),
                    $applied_taxes
                );
                break;
            case 'net,taxes,gross,rates':
                return array(
                    $this->formatPrice($price_array['net'], $currency_symbol, $currency_code),
                    JText::_('COM_BREEZINGCOMMERCE_PRICE_TAXES').' '.$this->formatPrice($taxes, $currency_symbol, $currency_code),
                    JText::_('COM_BREEZINGCOMMERCE_PRICE_GROSS').' '.$this->formatPrice($price_array['gross'], $currency_symbol, $currency_code),
                    $applied_taxes
                );
                break;
            case 'gross':
                return array(
                    $this->formatPrice($price_array['gross'], $currency_symbol, $currency_code)
                );
                break;
            case 'net':
                return array(
                    $this->formatPrice($price_array['net'], $currency_symbol, $currency_code)
                );
                break;
        }
        
        return array();
    }

    function add($product_id, $amount, $properties = null, $attributes = null, $attribute_amounts = null, $customer_id = null){

        if($product_id < 1 || !is_numeric($product_id)){
            throw new CrBcCartException('Invalid product id!');
        }
        
        $db = JFactory::getDBO();
        $attr_amounts = null;
        $attr_amounts_db = '';
        $attrs_flat = array();

        ///////
        // CONVERT ATTRIBUTE AND PROPERY SELECTIONS

        // cleaning up attributes with a zero amount value...
        if(is_array($attributes) && is_array($attribute_amounts)){
            
            $copy_attributes = $attributes;
            $copy_attribute_amounts = $attribute_amounts;
            
            foreach($attributes As $attribute_id => $items){
                if( intval($attribute_amounts[intval($attribute_id)]) <= 0 ){
                    unset($copy_attributes[intval($attribute_id)]);
                    unset($copy_attribute_amounts[intval($attribute_id)]);
                }
            }
            
            $attributes = $copy_attributes;
            $attribute_amounts = $copy_attribute_amounts;
        }
        
        // ...and add the rest to the cart
        if(is_array($attributes) && is_array($attribute_amounts)){
            $attr_amounts = array();
            foreach($attributes As $attribute_id => $items){
                foreach($items As $item){
                    $item = intval($item);
                    $attrs_flat[] = $item;
                    $attr_amounts[$item] = intval($attribute_amounts[intval($attribute_id)]);
                    if($item != 0 && $attr_amounts[$item] > 0){
                        $attr_amounts_db .= $item.'='.intval($attribute_amounts[$attribute_id]).',';
                    }
                }
            }
            $attr_amounts_db = rtrim($attr_amounts_db, ',');
        }

        if(is_array($properties)){
            JArrayHelper::toInteger($properties);
            $props = array();
            $size = count($properties);
            for($i = 0; $i < $size; $i++){
                if($properties[$i] != 0){
                    $props[] = $properties[$i];
                }
            }
            $properties = $props;
        }else{
            $properties = array();
        }
        
        // retrieve additional product info
        $db->setQuery("Select use_combinations, attribs, stock, virtual_product, producttaxclass_id From #__breezingcommerce_products Where id = " . intval($product_id));
        $product = $db->loadAssoc();
        
        if($product === null){
            throw new CrBcCartException('Invalid product!');
        }
        
        $is_combination = $product['use_combinations'];

        // used for product level attributes
        $pform = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
        $pform->loadString($product['attribs']);

        $min_amount = $pform->get('minimum_amount', 1);
        $max_amount = $pform->get('maximum_amount', 1);
        
        if($amount < $min_amount){
            $amount = $min_amount;
        }
        
        if($amount > $max_amount){
            $amount = $max_amount;
        }
        
        // used for global override
        $form = CrBcHelpers::getBcConfig();

        $published = 0;
        $backordered = false;

        // TODO: Simplify the product stock checks below!

        ///////////
        // PROPERTY COMBINATION STOCK CHECK

        if(
             !$product['virtual_product']
             &&
             $is_combination
             &&
             count($properties)
             &&
             $pform->get('backorders','') != ''
          ){
            $db->setQuery("Select Distinct combinations.stock From #__breezingcommerce_product_property_combinations As combinations Where combinations.id In (".implode(',',$properties).")");
            $stocks = $db->loadColumn();

            foreach($stocks As $cstock){
                if(($cstock - $amount) < 0){
                    if(
                       $pform->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN
                       ||
                       $pform->get('backorders','') == self::$BACKORDERS_DISALLOWED_VISIBLE
                      ){
                        throw new CrBcOutOfStockException('Backorders are not allowed for this product combination');
                      } else {
                          $backordered = true;
                          break;
                      }
                }
            }
            
        }
        else
        if(
             !$product['virtual_product']
             &&
             $is_combination
             &&
             count($properties)
             &&
             $pform->get('backorders','') == ''
          ){
            $db->setQuery("Select Distinct combinations.stock From #__breezingcommerce_product_property_combinations As combinations Where combinations.id In (".implode(',',$properties).")");
            $stocks = $db->loadColumn();

            foreach($stocks As $cstock){
                if(($cstock - $amount) < 0){
                    if(
                       $form->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN
                       ||
                       $form->get('backorders','') == self::$BACKORDERS_DISALLOWED_VISIBLE
                      ){
                          throw new CrBcOutOfStockException('Backorders are not allowed for this product combination');
                      }else{
                          $backordered = true;
                          break;
                      }
                }
            }
        }

        ///////////
        // ATTRIBUTES STOCK CHECK

        if(
             is_array($attributes)
             &&
             count($attributes)
           ){

            $aproducts = array();
            $aproducts_amounts = array();

            $db->setQuery("Select pai.id, pai.item_product_id, pai.plus_price, ai.product_id, ai.plus_price As original_plus_price From #__breezingcommerce_product_attribute_items As pai, #__breezingcommerce_attribute_items As ai Where ai.id = pai.attribute_item_id And pai.id In (".implode(',',$attrs_flat).") And pai.published = 1");
            $attribs = $db->loadAssocList();

            foreach($attribs As $attrib){
                
                $aproduct_id = 0;

                if($attrib['item_product_id']){
                    $aproduct_id = $attrib['item_product_id'];
                }else if($attrib['product_id']){
                    $aproduct_id = $attrib['product_id'];
                }

                if($aproduct_id){
                    $aproducts[] = intval($aproduct_id);
                    $aproducts_amounts[$aproduct_id] = $attr_amounts[$attrib['id']];
                }
            }

            if(count($aproducts)){
                
                $db->setQuery("Select id, stock, virtual_product, attribs From #__breezingcommerce_products Where id In (".implode(',',$aproducts).")");
                $aproducts_list = $db->loadAssocList();

                $aform = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'models' . DS . 'product.xml');

                foreach($aproducts_list As $aproduct){

                    $aform->loadString($aproduct['attribs']);

                    ///////
                    // CASE OF PRODUCT OVERRIDES GLOBAL
                    if (
                            !$aproduct['virtual_product']
                            &&
                            $aform->get('backorders','') != ''
                    ) {
                        if (($aproduct['stock'] - $aproducts_amounts[$aproduct['id']]) < 0) {
                            if(
                                $aform->get('backorders', '') == self::$BACKORDERS_DISALLOWED_HIDDEN
                                ||
                                $aform->get('backorders', '') == self::$BACKORDERS_DISALLOWED_VISIBLE
                                )
                            {
                                throw new CrBcAttributesOutOfStockException('Backorders are not allowed for this product attribute');
                            }else{
                                $backordered = true;
                                break;
                            }
                        }
                    }
                    else
                    //////
                    // GLOBAL CHECK
                    if (
                            !$aproduct['virtual_product']
                            &&
                            $aform->get('backorders','') == ''
                    ) {
                        if (($aproduct['stock'] - $aproducts_amounts[$aproduct['id']]) < 0) {
                            if(
                                $form->get('backorders', '') == self::$BACKORDERS_DISALLOWED_HIDDEN
                                ||
                                $form->get('backorders', '') == self::$BACKORDERS_DISALLOWED_VISIBLE
                            ){
                                throw new CrBcAttributesOutOfStockException('Backorders are not allowed for this product attribute');
                            }else{
                                $backordered = true;
                                break;
                            }
                        }
                    }
                }
            }
        }

        ///////////
        // REGULAR STOCK CHECK

        $stock = $product['stock'];

        if(   !$product['virtual_product']
              &&
              !$is_combination
              &&
              ($stock - $amount < 0)
              &&
              $pform->get('backorders','') != ''
           ){
            if(
               $pform->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN
               ||
               $pform->get('backorders','') == self::$BACKORDERS_DISALLOWED_VISIBLE
              ){
                throw new CrBcOutOfStockException('Backorders are not allowed for this product');
              }else{
                $backordered = true;
              }
        }
        else
        if(   !$product['virtual_product']
              &&
              !$is_combination
              &&
              ($stock - $amount < 0)
              &&
              $pform->get('backorders','') == ''
           ){
            if(
               $form->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN
               ||
               $form->get('backorders','') == self::$BACKORDERS_DISALLOWED_VISIBLE
              ){
                throw new CrBcOutOfStockException('Backorders are not allowed for this product');
              }else{
                  $backordered = true;
                  
              }
        }

        if(!$backordered){
            $published = 1;
        }

        /////////////
        // MAIN CART ADD

        // TODO: find out how we can offer updated product add to cart
        // if there are attributes or properties selected, we cannot replace existing products in the cart but treat them as separate product
        //if( ( is_array($attributes) && count($attributes) ) || ( is_array($properties) && count($properties) ) ){
            
            $db->setQuery("Insert Into #__breezingcommerce_order_items (`producttaxclass_id`, `history`,`published`,`attributes`, `properties`, order_id, product_id, amount, `creation_date`, `is_combination`) Values ('".$product['producttaxclass_id']."', 0, $published,'".$attr_amounts_db."','".trim(implode(',',$properties))."','".intval($this->cart['order_id'])."','".intval($product_id)."','".intval($amount)."','".JFactory::getDate()->toSql()."', ".($is_combination ? 1 : 0).")");
            $return = $db->query();
            if($return !== false){
                
                $new_item = array('order_item_id' => $db->insertid(), 'producttaxclass_id' => $product['producttaxclass_id'], 'id' => intval($product_id), 'amount'=>intval($amount), 'properties' => $properties, 'attributes' => $attributes, 'attribute_amounts' => $attr_amounts, 'is_combination' => $is_combination ? true : false);
                $this->cart['products'][] = $new_item;
                $this->cart['products_list'][] = intval($product_id);

                $db->setQuery("Update #__breezingcommerce_orders Set checked_out = 0 Where id = " . intval($this->cart['order_id']));
                $db->query();
                return $new_item;
            }
        /*   
        }else{

            if($customer_id){
                $db->setQuery("Update #__breezingcommerce_orders Set checked_out = 0, customer_id = '".intval($customer_id)."', `creation_date` = '".JFactory::getDate()->toSql()."' Where id = " . intval($this->cart['order_id']));
                $db->query();
            }

            $size = count($this->cart['products']);
            for($i = 0; $i < $size; $i++){
                if($product_id == $this->cart['products'][$i]['id']){
                    $db->setQuery("Update #__breezingcommerce_order_items Set `producttaxclass_id` = '".$product['producttaxclass_id']."', `history` = 0, `published` = $published, `attributes` = '".$attr_amounts_db."', `is_combination` = ".($is_combination ? 1 : 0).", `properties` = '".trim(implode(',',$properties))."', product_id = '".intval($product_id)."', amount = '".intval($amount)."', `creation_date` = '".JFactory::getDate()->toSql()."' Where product_id = ".intval($product_id)." And order_id = '".$this->cart['order_id']."'");
                    $return = $db->query();
                    if($return !== false){
                        $this->cart['products'][$i]['amount'] = intval($amount);
                        $this->cart['products'][$i]['properties'] = $properties;
                        $this->cart['products'][$i]['attributes'] = $attributes;
                        $this->cart['products'][$i]['attribute_amounts'] = $attr_amounts;
                        $this->cart['products'][$i]['producttaxclass_id'] = $product['producttaxclass_id'];

                        $db->setQuery("Update #__breezingcommerce_orders Set checked_out = 0 Where id = " . intval($this->cart['order_id']));
                        $db->query();
                    }
                    break;
                }
            }
        }*/
    }

    function remove($order_item_id, $db_delete = true){
        
        $db = JFactory::getDbo();
        $db->setQuery('Select product_id From #__breezingcommerce_order_items Where id = '. $db->quote($order_item_id));
        $product_id = $db->loadResult();
        
        if(is_array($this->cart) && isset($this->cart['products_list']) && in_array($product_id, $this->cart['products_list'])){
            $size = count($this->cart['products']);
            for($i = 0; $i < $size; $i++){
                if($order_item_id == $this->cart['products'][$i]['order_item_id']){

                    if($db_delete){
                        $db = JFactory::getDBO();
                        $db->setQuery("Delete From #__breezingcommerce_order_items Where history = 0 And id = ".intval($order_item_id)." And order_id = '".intval($this->cart['order_id'])."'");
                        $db->query();
                    }

                    unset($this->cart['products'][$i]);
                    $this->cart['products'] = array_merge(array(), $this->cart['products']);
                    
                    //unset($this->cart['products_list'][$i]);
                    //$this->cart['products_list'] = array_merge(array(), $this->cart['products_list']);

                    $session_cart = JFactory::getSession()->get('crbc_cart', array());
                    
                    if(isset($session_cart['products'])){
                    
                        $session_size = count($session_cart['products']);

                        for($j = 0; $j < $session_size; $j++){
                            if($order_item_id == $session_cart['products'][$j]['order_item_id']){
                                unset($session_cart['products'][$j]);
                                $session_cart['products'] = array_merge(array(), $session_cart['products']);
                                
                                //unset($session_cart['products_list'][$j]);
                                //$session_cart['products_list'] = array_merge(array(), $session_cart['products_list']);
                                
                                JFactory::getSession()->set('crbc_cart', $session_cart);
                                break;
                            }
                        }
                    }
                    
                    if(!isset($this->cart['products']) || count($this->cart['products']) == 0){
                        $this->clear();
                    }
                    
                    break;
                }
            }
        }
    }

    function clear(){
        
        if(isset($this->cart['products'] )){
            foreach($this->cart['products'] As $product){
                $this->remove($product['id']);
            }
        }
        
        if(isset($this->cart['order_id'] )){
            $db = JFactory::getDBO();
            $db->setQuery("Delete From #__breezingcommerce_orders Where checked_out = 0 And history = 0 And id = '".intval($this->cart['order_id'])."'");
            $db->query();
        }
        
        $this->cart = array();
        
        self::$order_id = 0;
        self::$exclude_from_no_tax_list = array();
        
        JFactory::getSession()->clear('crbc_cart');
    }
    
    public static function getInstance(){
        
        static $_cart = null;
        
        if( !($_cart instanceof CrBcCart) ){
        
            $_session_cart = JFactory::getSession()->get('crbc_cart', array());
            $_cart = new CrBcCart( $_session_cart !== null ? $_session_cart : array() );
        
        }

        return $_cart;
    }
    
    public static function applyProductPricePlugins($product_id, $amount = 1){
        
        $db = JFactory::getDBO();
        
        $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'productprice' Order By `ordering`");
        $plugins = $db->loadObjectList();
        
        $where = '';
        if(JFactory::getApplication()->isSite()){
            $where = ' published = 1 And ';
        }

        $db->setQuery("Select title, price, sale_price, id, producttaxclass_id, use_combinations, virtual_product From #__breezingcommerce_products Where ".$where." id = ".intval($product_id)."");
        $row = $db->loadObject();
        
        if( is_object($row) ){
            
            $row->item_prices = array();
            
            $new_price = $row->price;
            foreach($plugins As $plugin){
                $class = CrBcPlugin::getPluginClass($plugin->name, $plugin->type);
                $class_instance = new $class();
                $new_price = $class::getPrice($row->id, 0, $amount, $new_price);
                if($new_price !== null){
                    $row->price = $new_price;
                }
                /* cannot be used here as we need the order item id on every call
                 * in fact this method here should only be used to display the main product price
                $item_price = $class::getItemPrice($row->id, 0, $amount, $row->price, $needs_order_item_id);
                if($item_price !== null){
                    $row->item_prices[] = $item_price;
                }*/
            }
            
            return $row;
        }
        
        return null;
    }

    function getItems($retrieve_property_info = false){
        
        $items = array();
        if(isset($this->cart['products'])){
            $db = JFactory::getDBO();
            $amounts = array();
            $properties = array();
            $attributes = array();
            $products = array();
            $combination = array();
            
            foreach($this->cart['products'] As $product){
                $products[] = $product['order_item_id'];
                $amounts[$product['order_item_id']] = isset($product['amount']) ? intval($product['amount']) : 0;
                $properties[$product['order_item_id']] = isset($product['properties']) && is_array($product['properties']) ? $product['properties'] : array();
                $attributes[$product['order_item_id']] = isset($product['attribute_amounts']) && is_array($product['attribute_amounts']) ? $product['attribute_amounts'] : array();
            }
            
            JArrayHelper::toInteger($products);
            JArrayHelper::toInteger($amounts);
            
            $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'productprice' Order By `ordering`");
            $plugins = $db->loadObjectList();
            
            if(count($products)){
                
                $where = '';
                if(JFactory::getApplication()->isSite()){
                    $where = ' p.published = 1 And ';
                }
                
                $db->setQuery("Select oi.id As order_item_id, p.title, p.price, p.weight, p.weight_type, p.id, p.producttaxclass_id, p.use_combinations, p.virtual_product, p.item_number From #__breezingcommerce_order_items As oi, #__breezingcommerce_products As p Where p.id = oi.product_id And ".$where." oi.id In (".implode(',',$products).")");
                $rows = $db->loadAssocList();
                
                foreach($rows As $row){

                    $item_prices_single = array();
                    $item_prices = array();
                    $new_price = $row['price'];
                    
                    foreach($plugins As $plugin){
                        
                        $class = CrBcPlugin::getPluginClass($plugin->name, $plugin->type);
                        $class_instance = new $class();
                        
                        $combination_id = 0;
                        if($row['use_combinations']){
                            foreach($properties[$row['order_item_id']] As $property){
                                $combination_id = $property;
                                break;
                            }
                        }
                        
                        $new_price = $class::getPrice($row['id'], $combination_id, $amounts[$row['order_item_id']], $new_price);
                        if($new_price !== null){
                            $row['price'] = $new_price;
                        }
                        
                        $_item_price_single = $class::getItemPrice($row['id'], $combination_id, 1, $row['price'], $row['order_item_id']);
                        if($_item_price_single !== null){
                            $item_prices_single[] = $_item_price_single;
                        }
                        
                        $item_price = $class::getItemPrice($row['id'], $combination_id, $amounts[$row['order_item_id']], $row['price'], $row['order_item_id']);
                        if($item_price !== null){
                            $item_prices[] = $item_price;
                        }
                    }
                    

                    $item = new stdClass();
                    $item->id = $row['id'];
                    $item->item_number = $row['item_number'];
                    $item->title = $row['title'];
                    $item->virtual_product = $row['virtual_product'];
                    $item->amount = $amounts[$row['order_item_id']];
                    $item->order_item_id = $row['order_item_id'];
                    $item->weight = $row['weight'];
                    $item->weight_type = $row['weight_type'];
                    
                    $price = self::getItemPrice( $row['price'], $this->currency_conversion_rate, $row['producttaxclass_id'], $properties[$item->order_item_id], $attributes[$item->order_item_id], $item->amount, $this->cart['customer_id'], $row['use_combinations'] == 1 ? true :  false);
                    $price_single = self::getItemPrice( $row['price'], $this->currency_conversion_rate, $row['producttaxclass_id'], $properties[$item->order_item_id], $attributes[$item->order_item_id], 1, $this->cart['customer_id'], $row['use_combinations'] == 1 ? true :  false);

                    $item_price_single_net = 0;
                    $item_price_single_gross = 0;
                    
                    foreach( $item_prices_single As $item_price_single ){
                        
                        $item_price_single_net += $item_price_single['net'];
                        $item_price_single_gross += $item_price_single['gross'];
                        
                        foreach( $item_price_single['tax_list'] As $item_price_producttaxclass_id => $item_price_tax_list ){
                            
                            if( !isset( $price_single['tax_list'][$item_price_producttaxclass_id] ) ){
                                
                                $price_single['tax_list'][$item_price_producttaxclass_id] = $item_price_tax_list;
                            }
                            else
                            {
                                $price_single['tax_list'][$item_price_producttaxclass_id]['tax'] += $item_price_tax_list['tax'];
                            }
                        }
                    }
                    
                    $item->price_single_net = $price_single['net'] + $item_price_single_net;
                    $item->price_single_gross = $price_single['gross'] + $item_price_single_gross;
                    
                    // determine tax rounding errors
            
                    $difftax = 0;

                    foreach($price_single['tax_list'] As $key => $value){

                        $difftax += self::round($price_single['tax_list'][$key]['tax']);
                    }

                    // find out if there are rounding errors against the glbally rounded tax
                    $rounderror = self::round($item->price_single_gross - $item->price_single_net) - self::round($difftax);

                    // if there is a rounding error, add the error to the very next item from the tax list
                    if($rounderror != 0){

                        foreach($price_single['tax_list'] As $key => $value){
                            $price_single['tax_list'][$key]['tax'] += self::round($rounderror);
                            break;
                        }

                    }
                    
                    $item->price_single_tax_list = $price_single['tax_list'];
                    $item->single_off = $price_single['off'];

                    // TODO: merging $item_prices from product price plugins to allow plugins treating a price as single item, with own producttaxclass id
                    // goes here...
                    $item_price_net = 0;
                    $item_price_gross = 0;
                    $item_price_off = 0;
                    
                    foreach( $item_prices As $item_price ){
                        $item_price_net += $item_price['net'];
                        $item_price_gross += $item_price['gross'];
                        $item_price_off += $item_price['off'];
                        
                        foreach( $item_price['tax_list'] As $item_price_producttaxclass_id => $item_price_tax_list ){
                            
                            if( !isset( $price['tax_list'][$item_price_producttaxclass_id] ) ){
                                
                                $price['tax_list'][$item_price_producttaxclass_id] = $item_price_tax_list;
                            }
                            else
                            {
                                $price['tax_list'][$item_price_producttaxclass_id]['tax'] += $item_price_tax_list['tax'];
                            }
                        }
                    }
                    
                    $item->price_net = $price['net'] + $item_price_net;
                    $item->price_gross = ( $price_single['gross'] + $item_price_single_gross ) * $item->amount;
                    $item->off = $price['off'] + $item_price_off;
                    
                    // determine tax rounding errors
            
                    $difftax = 0;

                    foreach($price['tax_list'] As $key => $value){

                        $difftax += self::round($price['tax_list'][$key]['tax']);
                    }

                    // find out if there are rounding errors against the glbally rounded tax
                    $rounderror = self::round($item->price_gross - $item->price_net) - self::round($difftax);

                    // if there is a rounding error, add the error to the very next item from the tax list
                    if($rounderror != 0){

                        foreach($price['tax_list'] As $key => $value){
                            $price['tax_list'][$key]['tax'] += self::round($rounderror);
                            break;
                        }

                    }
                    
                    //$item->price_tax = 0;
                    
                    //foreach($price['tax_list'] As $key => $value){
                    //    $item->price_tax += $price['tax_list'][$key]['tax'];
                    //}
                    
                    $item->price_tax_list = $price['tax_list'];
                    
                    $item->is_combination = $row['use_combinations'] == 1 ? true :  false;
                    
                    $item->properties = null;
                    $item->attributes = null;
                    $item->attribute_amounts = $attributes[$item->order_item_id];

                    if($retrieve_property_info){
                    
                        JArrayHelper::toInteger($properties[$item->order_item_id]);
                        $props = implode(',',$properties[$item->order_item_id]);

                        if($props != '' && !$item->is_combination){

                            $db->setQuery("
                                Select Distinct
                                    ppv.*,
                                    ppk.id As ppk_id,
                                    ppk.required As required,
                                    pv.`value`,
                                    ppv.id As productpropertyvalue_id,
                                    pv.id As property_values_id,
                                    pv.key_id As property_values_key_id,
                                    k.key As property_key,
                                    k.property_id As property_keys_property_id,
                                    k.id As property_keys_id,

                                    k.key As property_key,
                                    p.producttaxclass_id
                                From
                                    #__breezingcommerce_product_property_values As ppv,
                                    #__breezingcommerce_product_property_keys As ppk,
                                    #__breezingcommerce_property_values As pv,
                                    #__breezingcommerce_property_keys As k,
                                    #__breezingcommerce_products As p
                                Where
                                    ppv.property_key_id = k.id
                                And
                                    ppv.id In (".$props.")
                                And
                                    ppk.property_key_id = k.id
                                And
                                    ppv.published = 1
                                And
                                    ppk.published = 1
                                And
                                    ppv.property_key_id = k.id
                                And
                                    ppk.property_key_id = k.id
                                And
                                    ppv.product_id = " . intval($item->id) . "
                                And
                                    ppk.product_id = " . intval($item->id) . "
                                And
                                    p.id = " . intval($item->id) . "
                                And
                                    pv.id = ppv.property_value_id
                                And
                                    k.id = ppk.property_key_id
                                Order By
                                    ppk.`ordering`,
                                    ppv.`ordering`
                            ");
                            
                            $item->properties = $db->loadAssocList();
                        }
                        else if($props != '' && $item->is_combination){

                            $db->setQuery("
                                Select Distinct
                                    ppc.*,
                                    ppc.weight As weight,
                                    ppc.weight_type As weight_type,
                                    ppi.id As ppi_id,
                                    pv.`value`,
                                    ppv.`custom_value`,
                                    ppv.id As productpropertyvalue_id,
                                    pv.id As property_values_id,
                                    pv.key_id As property_values_key_id,
                                    k.key As property_key,
                                    k.property_id As property_keys_property_id,
                                    k.id As property_keys_id,

                                    k.key As property_key,
                                    p.producttaxclass_id
                                From
                                    #__breezingcommerce_product_property_combinations As ppc,
                                    #__breezingcommerce_product_property_combination_items As ppi,
                                    #__breezingcommerce_product_property_values As ppv,
                                    #__breezingcommerce_product_property_keys As ppk,
                                    #__breezingcommerce_property_values As pv,
                                    #__breezingcommerce_property_keys As k,
                                    #__breezingcommerce_products As p
                                Where
                                        ppc.id In (".$props.")
                                    And
                                        ppi.product_id = " . intval($item->id) . "
                                    And
                                        ppc.id = ppi.combination_id
                                    And
                                        ppi.property_key_id = k.id
                                    And
                                        ppv.property_value_id = ppi.property_value_id
                                    And
                                        ppk.property_key_id = ppi.property_key_id
                                    And
                                        ppv.product_id = " . intval($item->id) . "
                                    And
                                        ppk.product_id = " . intval($item->id) . "
                                    And
                                        pv.id = ppi.property_value_id
                                    And
                                        p.id = " . intval($item->id) . "
                                    And
                                        ppc.published = 1
                                    And
                                        ppv.published = 1
                                    And
                                        ppk.published = 1
                                    Order By
                                        ppc.`ordering`,
                                        ppk.`ordering`,
                                        ppv.`ordering`
                            ");

                            $item->properties = $db->loadAssocList();
                        }

                        $attrs = array();
                        foreach($attributes[$item->order_item_id] As $id => $amount){
                            $attrs[] = intval($id);
                        }
                        $attrs = implode(',',$attrs);

                        if($attrs != ''){

                            $db->setQuery("
                                Select
                                    items.*,
                                    items.product_id As item_product_id,
                                    attributes.`name` As attribute_name,
                                    attributes.`min_amount`,
                                    attributes.`allow_amount`,
                                    attributes.display_as As attribute_display_as,
                                    attributes.required As attribute_required,
                                    product_attribute_items.`name` As product_attribute_item_name,
                                    product_attribute_items.`id` As product_attribute_item_id,
                                    product_attribute_items.plus_price As product_attribute_item_plus_price,
                                    product_attribute_items.item_product_id As product_attribute_item_product_id,
                                    product_attribute_items.selected As product_attribute_item_selected,
                                    p.producttaxclass_id
                                From
                                    #__breezingcommerce_attributes As attributes,
                                    #__breezingcommerce_attribute_items As items,
                                    #__breezingcommerce_product_attributes As product_attributes,
                                    #__breezingcommerce_product_attribute_items As product_attribute_items,
                                    #__breezingcommerce_products As p
                                Where
                                    product_attribute_items.id In (".$attrs.")
                                And
                                    product_attributes.product_id = ".intval($item->id)."
                                And
                                    product_attribute_items.product_attribute_id = product_attributes.id
                                And
                                    attributes.id = product_attributes.attribute_id
                                And
                                    items.id = product_attribute_items.attribute_item_id
                                And
                                    attributes.id = items.attribute_id
                                And
                                    p.id = product_attributes.product_id
                                And
                                    product_attributes.published = 1
                                And
                                    product_attribute_items.published = 1
                                Order By
                                    product_attributes.`ordering`,
                                    product_attribute_items.`ordering`
                            ");

                            $item->attributes = $db->loadAssocList();
                        }
                    }

                    $items[] = $item;
                }
            }
        }
        
        return $items;
    }

    function getArray(){
        return $this->cart;
    }
    
    function hasLimitedAddressData(){
        
        $complete = false;
        $customer = $this->getCustomer();
        if($customer){
            $complete = $customer->address_complete == 0 ? false : true;
        }
        
        if(!$this->isVirtualOrder($this->getItems(true)) || $complete || CrBcHelpers::getBcConfig()->get('checkout_limited_address_data', 0) == 0){
            
            return false;
        }
        
        return true;
    }
    
    function getCustomer(){
        
        if(isset( $this->cart['customer_id'] )){
            
            $db = JFactory::getDbo();
            $db->setQuery("Select * From #__breezingcommerce_customers Where id = " . intval($this->cart['customer_id']) . " Limit 1");
            return $db->loadObject();
        }
        
        return null;
    }

    function purgeExpiredCart(){
       $max_age = self::$max_age;
       $db = JFactory::getDBO();
       if(is_array($this->cart) && isset($this->cart['order_id']) && is_numeric($this->cart['order_id'])){
          $db->setQuery("Select id From #__breezingcommerce_orders Where id = '".intval($this->cart['order_id'])."'");
          $rows = $db->loadAssocList();
          if(count($rows) == 1){
                $db->setQuery("Select id From #__breezingcommerce_orders Where checked_out = 0 And history = 0 And id = '" . intval($this->cart['order_id']) . "' And Timediff('" . JFactory::getDate()->toSql() . "', `creation_date`) > '$max_age' Limit 1");
                $rows = $db->loadAssocList();
                foreach ($rows As $row) {
                    $db->setQuery("Delete From #__breezingcommerce_order_items Where order_id = '" . $row['id'] . "'");
                    $db->query();
                    //$this->cart = self::buildCart($this->cart['customer_id'], $this->cart['customer_name']);
                }
                $db->setQuery("Delete From #__breezingcommerce_orders Where checked_out = 0 And history = 0 And id = '" . intval($this->cart['order_id']) . "' And Timediff('" . JFactory::getDate()->toSql() . "', `creation_date`) > '$max_age' Limit 1");
                $db->query();
          }else{
              //$this->cart = self::buildCart($this->cart['customer_id'], $this->cart['customer_name']);
          }
       }
    }
    
    /**
     * Returns a cart's order info.
     * 
     * If given oder_id, it method will try to restore the cart from the database by the id given.
     * This cart restore will only work with users that are allowed to load.
     * For example the buyer itself (hashes links to access the cart later), the user_id associated with id (if given) and adminstrators
     * 
     * @param int $order_id
     * @return array
     * @throws CrBcCartException
     */
    
    public static function getOrderInfoFromCart($order_id = 0){
        
        $_cart = null;
        $_session_cart = null;
        
        // trying to restore from session if no order id is given
        if($order_id == 0){
            
            $_session_cart = JFactory::getSession()->get('crbc_cart', null);
            
            if($_session_cart !== null){
                $_cart = new CrBcCart( $_session_cart );
            }
            
        } else {
            
            // new empty cart and loading below
            $_cart = new CrBcCart( array() );
            
            // doing 2 things: returning the session_cart array + internally updating the cart state
            $_session_cart = $_cart->restoreCart($order_id); 
            
        }
        
        if($_session_cart === null || $_cart === null){
            throw new CrBcCartException('Invalid cart. CrBcCart::getOrderInfoFromCart() cannot be executed.');
        }
        
        $customer = null;
        if($_cart instanceof CrBcCart && isset($_cart->cart['customer_id'])){
            JFactory::getDbo()->setQuery("Select * From #__breezingcommerce_customers Where id = " . intval($_cart->cart['customer_id']));
            $customer = JFactory::getDbo()->loadObject();
        }

        $_cart_items = $_cart->getItems(true);

        $_order_info = array();
        $data = array();
        
        if( isset($_session_cart['order_id']) ){
        
            $data = self::getData($_session_cart['order_id'], $_cart_items, -1, -1);

            $_order_info = self::getOrder(
                                        $_session_cart['order_id'], 
                                        $_cart, 
                                        $_session_cart, 
                                        $_cart_items, 
                                        $customer,
                                        $data,
                                        array(),
                                        array()
                                );
        }
        
        return array('info' => $_order_info, 'items' => $_cart_items, 'data' => $data);
    }

    function restoreCart($order_id){
        
        // TODO: recreate in frontend only if valid session and/or logged on user matches the cart to restore
        
        $order_id = intval($order_id);
        
        self::$order_id = $order_id;
        
        $cart = array();
        $db = JFactory::getDBO();
        $db->setQuery("Select o.*, concat(c.firstname, ' ', concat(c.lastname), If(c.company <> c.company, '', concat(', ', c.company)), ' (', c.id, ')') As customer_name From #__breezingcommerce_orders As o, #__breezingcommerce_customers As c Where o.id = " . intval($order_id) . " And c.id = o.customer_id And o.published = 1");
        $result = $db->loadAssoc();

        $query = ' Select customer_id, history_symbol, history_conversion_rate, history_currency_code, invoice_created From #__breezingcommerce_orders '.
                    '  Where history = 1 And id = '.intval($order_id);
        $db->setQuery( $query );
        $order_history = $db->loadAssoc();
        if(is_array($order_history)){
            //if($order_history['invoice_created']){
            //    $this->cart = array('customer_id' => 0, 'customer_name' => '', 'order_id' => 0, 'products' => array(), 'products_list' => array(), 'properties' => array(), 'attributes' => array(), 'attribute_amounts' => array() );
            //    return $this->cart;
            //}
            $this->currency_symbol = $order_history['history_symbol'];
            $this->currency_conversion_rate = $order_history['history_conversion_rate'];
            $this->currency_code = $order_history['history_currency_code'];
            
        } else {
            
            $form = CrBcHelpers::getBcConfig();
            
            if(  intval($result['customer_id']) > 0 && $form->get('location_based_tax_rules', 0) == 1 ){
            
                $db->setQuery("Select Distinct groups.* From #__breezingcommerce_customergroups groups, #__breezingcommerce_customers customers Where customers.customergroup_id = groups.id And customers.id = " . $db->quote(intval($result['customer_id'])) . " Limit 1");
                
                $group = $db->loadObject();

                if($group && intval($group->currency_id) != 0){
                
                    $db->setQuery("Select conversion_rate, symbol, code From #__breezingcommerce_currencies Where id = " . intval($group->currency_id)  . " And published = 1");
                    $currency = $db->loadAssoc();
                    $this->currency_symbol = $currency['symbol'];
                    $this->currency_conversion_rate = $currency['conversion_rate'];
                    $this->currency_code = $currency['code'];
                }
            }
        }
        
        if(is_array($result)){
            $products = array();
            $products_list = array();

            $db->setQuery("Select oi.* From #__breezingcommerce_order_items As oi, #__breezingcommerce_products As p Where p.id = oi.product_id And oi.order_id = " . intval($order_id) . " Order By oi.`creation_date`");
            $items = $db->loadAssocList();

            foreach($items As $item){
                $attr_amounts = array();
                $attribs = array();
                $attributes = explode(',',$item['attributes']);
                foreach($attributes As $attribute){
                    $pair = explode('=', $attribute);
                    if(isset($pair[1])){
                        $attr_amounts[$pair[0]] = intval($pair[1]);
                        $db->setQuery("Select attribute_id From #__breezingcommerce_product_attribute_items Where id = " . intval($pair[0]) . " And published = 1 Limit 1");
                        $attr = $db->loadResult();
                        if($attr){
                            if(!isset($attribs[$attr])){
                                $attribs[$attr] = array();
                            }
                            $attribs[$attr][] = intval($pair[0]);
                        }
                    }
                }
                $properties = explode(',', $item['properties']);
                JArrayHelper::toInteger($properties);
                $products[] = array('order_item_id' => $item['id'], 'producttaxclass_id'=> $item['producttaxclass_id'],'id'=>intval($item['product_id']), 'amount'=>intval($item['amount']), 'properties' => $properties, 'attributes' => $attribs, 'attribute_amounts' => $attr_amounts);
                $products_list[] = intval($item['product_id']);
            }

            $cart = array(
                'customer_id' => intval($result['customer_id']),
                'customer_name' => $result['customer_name'],
                'order_id' => intval($order_id),
                'products' => $products,
                'products_list' => $products_list
            );
        }
        
        $this->cart = $cart;
        return $cart;
    }

    public static function getPrice($base_price, $conversion_rate, $producttaxclass_id, $amount, $customer_id){
        return self::getItemPrice($base_price, $conversion_rate, $producttaxclass_id, array(), array(), $amount, $customer_id, false, false);
    }

    public static function getItemPrice($base_price, $conversion_rate, $producttaxclass_id, $properties, $attributes, $amount, $customer_id, $is_combination = false, $singleton_taxes = true, $group_id = 0){

        $base_price = $base_price * $conversion_rate;
        
        //static $rows;
        //static $group;

        JArrayHelper::toInteger($properties);  

        $shipping = '';
        $off = 0;
        $gross = 0;
        $tax_list = array();
        $db = JFactory::getDBO();

        // TODO: CHECK IF THAT PROPERTY BELONGS TO THE PRODUCT!!!! (SQL BASED)
        // ALSO ADD REQUIRED TO PROPERTIES ADMIN
        if(is_array($properties) && count($properties)){

            $plugins = array();
            if(!$is_combination){
                $db->setQuery("Select price, is_plus_price From #__breezingcommerce_product_property_values Where price <> -1 And id In (".implode(',',$properties).") And published = 1");
            }else{
                $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'productprice' Order By `ordering`");
                $plugins = $db->loadObjectList();
                $db->setQuery("Select Distinct combinations.id, combinations.price, combinations.product_id From #__breezingcommerce_product_property_combinations As combinations, #__breezingcommerce_product_property_combination_items As items Where items.combination_id = combinations.id And combinations.published = 1 And combinations.id In (".implode(',',$properties).")");
            }
            
            $props = $db->loadAssocList();
            
            $plus_prices = 0;
            $replace_prices = 0;
            $cnt_plugins = count($plugins);

            foreach($props As $prop){
                
                if ($is_combination) {
                    $new_price = $prop['price'];
                    foreach ($plugins As $plugin) {
                        $class = CrBcPlugin::getPluginClass($plugin->name, $plugin->type);
                        $class_instance = new $class();
                        $new_price = $class::getPrice($prop['product_id'], $prop['id'], $amount, $new_price < 0 ? 0 : $new_price);
                        if ($new_price !== null) {
                            $prop['price'] = $new_price;
                        }
                    }
                }
                
                if($prop['price'] != -1){
                    if((isset($prop['is_plus_price']) && $prop['is_plus_price'])){
                        $plus_prices += $prop['price'];
                    }else{
                        $replace_prices += $prop['price'];
                    }
                }
            }

            if($replace_prices != 0){
                $base_price = ( $replace_prices + $plus_prices ) * $conversion_rate;
            }else{
                $base_price += $plus_prices * $conversion_rate;
            }
            
        }

        if(is_array($attributes) && count($attributes)){

            $attribute_ids = array();

            foreach($attributes As $attribute_id => $attribute_amount){
                $attribute_ids[] = intval($attribute_id);
            }

            JArrayHelper::toInteger($attributes);

            $db->setQuery("Select pai.id, pai.item_product_id, pai.plus_price, ai.product_id, ai.plus_price As original_plus_price From #__breezingcommerce_product_attribute_items As pai, #__breezingcommerce_attribute_items As ai Where ai.id = pai.attribute_item_id And pai.id In (".implode(',',$attribute_ids).") And pai.published = 1");
            $attribs = $db->loadAssocList();

            $plus_price = 0;
            foreach($attribs As $attrib){

                $plus_price2 = 0;

                if($attrib['product_id'] || $attrib['item_product_id']){

                    $product_id = 0;
                    if($attrib['item_product_id']){
                       $product_id = $attrib['item_product_id'];
                    } else {
                       $product_id = $attrib['product_id'];
                    }

                    if($product_id){
                        $db->setQuery("Select price From #__breezingcommerce_products Where id = ".intval($product_id)." And published = 1");
                        $product_price = $db->loadResult();;
                        if($product_price){
                            $plus_price2 = $product_price * $attributes[$attrib['id']];
                        }
                    }
                }
                
                if($attrib['original_plus_price'] != 0 && !$attrib['item_product_id']){

                    $plus_price2 = $attrib['original_plus_price'] * $attributes[$attrib['id']];

                }

                if($attrib['plus_price'] != 0){

                    $plus_price2 = $attrib['plus_price'] * $attributes[$attrib['id']];

                }

                $plus_price += $plus_price2;
            }

            $base_price += $plus_price * $conversion_rate;
            
        }

        // groups
        //if(!is_array($group)){
            if($group_id == 0){
                
                $db->setQuery("
                    Select
                        groups.*, 
                        customers.use_shipping_address, 
                        customers.region_id, 
                        customers.country_id, 
                        customers.zip, 
                        customers.shipping_region_id, 
                        customers.shipping_country_id, 
                        customers.shipping_zip
                    From 
                        #__breezingcommerce_customers As customers, 
                        #__breezingcommerce_customergroups As groups 
                    Where 
                        customers.id = " . intval($customer_id) . "
                    And 
                        customers.customergroup_id = groups.id 
                    And 
                        customers.published = 1 
                    And 
                        groups.published = 1
                    Limit 1");
                
            } else {
                
                $db->setQuery("
                    Select
                        *, 
                        0 As use_shipping_address, 
                        default_region_id As region_id, 
                        default_country_id As country_id, 
                        '' As zip, 
                        default_region_id As shipping_region_id, 
                        default_country_id As shipping_country_id, 
                        '' As shipping_zip
                    From 
                        #__breezingcommerce_customergroups
                    Where 
                        id = ".intval($group_id)."
                    And 
                        published = 1
                    Limit 1");
                
            }
            
            $group = $db->loadAssoc();
            $group_original_copy = $group;

            $form = CrBcHelpers::getBcConfig();
            
            // determing what taxes to pull later on based on the config
            if( $group && $form->get('apply_taxes_on', 'customergroup') == 'customergroup' ){
                
                $group['country_id'] = $group['default_country_id'];
                $group['region_id'] = $group['default_region_id'];
                $group['shipping_region_id'] = $group['default_region_id'];
                $group['shipping_country_id'] = $group['default_country_id'];
                
            }
            else if( $group && $form->get('apply_taxes_on', 'customergroup') == 'billing_address' ){
                
                $group['use_shipping_address'] = 0;
            }
            else if( $group && $form->get('apply_taxes_on', 'customergroup') == 'shipping_address' ){
                
                $group['use_shipping_address'] = 1;
            }
            
            // no group for the customer in db yet?
            if(!$group){
                
                // ok, then let's see if there is one determinable by the user's location
                $group = CrBcHelpers::getBestCustomerGroupByLocation();
                
                // still not there yet? then let's get the default customergroup
                if(!$group){
                    
                    $db->setQuery("Select *  From #__breezingcommerce_customergroups Where `default` = 1 Limit 1");
                    $group = $db->loadAssoc();
                }

                if($group){
                    // TODO: retrieve default use_shipping_address, zip, country and region ids
                    $group['country_id'] = $group['default_country_id'];
                    $group['region_id'] = $group['default_region_id'];
                    $group['zip'] = '*'; // no zip on default
                    $group['shipping_region_id'] = $group['default_region_id'];
                    $group['shipping_country_id'] = $group['default_country_id'];
                    $group['shipping_zip'] = '*'; // no zip on default
                    $group['use_shipping_address'] = 0;
                    
                }
                
            }
        //}

        if(is_array($group)){
            
            if($group['off_percent'] > 0 && $group['off_percent'] <= 100){
                $off = ( ( $base_price / 100 ) * $group['off_percent'] );
                $base_price = $base_price - self::round($off);
            }
            
        }
            
        $tax_disabled = false;
        
        if( self::$order_id > 0 && !in_array($producttaxclass_id, self::$exclude_from_no_tax_list) ){
            
            $db->setQuery("Select tax_disabled From #__breezingcommerce_orders Where id = " . $db->quote(self::$order_id));
            $tax_disabled = $db->loadResult() == 1 ? true : false;
            
        }
        
        $deduct_but_dont_list = false;
        
        if( $tax_disabled && $group['tax_on_base_price'] == 0 ){
            
            $deduct_but_dont_list = true;
            $tax_disabled = false;
        }
        
        // taxes
        if(!$tax_disabled && is_array($group)){

            if($group['use_shipping_address']){
                $shipping = 'shipping_';
            }

            $customer_taxclass_id = $group['tax_class_id'];
            
            // TODO: off plugins

            $rows = array();
            if(!isset($rows[$producttaxclass_id]) || !$singleton_taxes){
                
                $with_customer_tax_class = true;
                if( $group && $form->get('apply_taxes_on', 'customergroup') == 'billing_address' || $group && $form->get('apply_taxes_on', 'customergroup') == 'shipping_address' || $group && $form->get('apply_taxes_on', 'customergroup') == 'billing_or_shipping_address' ){
                    $with_customer_tax_class = false;
                }
                
                $db->setQuery("
                    Select Distinct txrates.*, ptxclass.taxrule_id, txrules.priority From
                        #__breezingcommerce_producttaxclasses As ptxclasses,
                        ".($with_customer_tax_class ? "#__breezingcommerce_customertaxclasses As ctxclasses," : "")."
                        #__breezingcommerce_taxrules_producttaxclasses As ptxclass,
                        ".($with_customer_tax_class ? "#__breezingcommerce_taxrules_customertaxclasses As ctxclass," : "")."
                        #__breezingcommerce_taxrules_taxrates As txrulesrates,
                        #__breezingcommerce_taxrules txrules,
                        #__breezingcommerce_taxrates As txrates
                    Where
                       ptxclasses.id = " . intval($producttaxclass_id) . "
                       ".($with_customer_tax_class ? " And ctxclasses.id = " . intval($customer_taxclass_id) : "")."
                    And
                       ptxclass.producttaxclass_id = ptxclasses.id
                        ".($with_customer_tax_class ? " And ctxclass.customertaxclass_id = ctxclasses.id" : "")."
                        ".($with_customer_tax_class ? " And ctxclass.taxrule_id = ptxclass.taxrule_id" : "")."
                    And
                       ptxclass.taxrule_id = txrulesrates.taxrule_id
                    And
                       txrulesrates.taxrate_id = txrates.id
                    And
                       txrules.id = txrulesrates.taxrule_id
                    And
                        (
                            txrates.country_id = ".$group[$shipping . 'country_id']."
                           Or
                            txrates.country_id = 0
                        )
                    And
                        (
                            txrates.region_id = ".$group[$shipping . 'region_id']."
                           Or
                            txrates.region_id = 0
                        )
                    And
                       txrates.published = 1
                    And
                       txrules.published = 1
                    And
                       ptxclasses.published = 1
                       
                    ".($with_customer_tax_class ? " And ctxclasses.published = 1" : "")."
                        
                    Order By
                       txrules.priority,
                       txrates.ordering
                ");

                $rows[$producttaxclass_id] = $db->loadAssocList();
                
            }
            
            $base_price2 = $base_price;
            
            if($group['tax_on_base_price'] == 1){
                $base_price2 *= $amount;
            }
            
            $curr_prio = null;
            $compound_list = array();
            $processed_rates = array();
            
            foreach ($rows[$producttaxclass_id] As $row) {
                
                if(!in_array($row['id'], $processed_rates)){
                    
                    $match = false;
                    
                    if( $row['is_range'] && $row['range_from'] <= $group[$shipping . 'zip'] && $row['range_to'] <= $group[$shipping . 'zip'] ){
                        $match = true;
                    }
                    else
                    if(!$row['is_range'] && trim($row['zip_code']) == '*'){
                        $match = true;
                    }
                    else
                    if(!$row['is_range'] && preg_match("/^".str_replace("*",".",trim($row['zip_code']))."$/", trim($group[$shipping . 'zip']))){
                        $match = true;
                    }

                    if($match){

                        if(!isset($compound_list[$row['priority']])){
                            $compound_list[$row['priority']] = 0;
                        }

                        if($curr_prio !== null && $curr_prio != $row['priority']){
                            $base_price2 = $base_price2 + $compound_list[$curr_prio];
                        }

                        if(!isset($tax_list[$row['id']])){
                            $tax_list[$row['id']] = array('name' => $row['name'], 'tax' => 0, 'rate' => $row['rate_percent']);
                        }

                        // reversal
                        // orig
                        if($group['tax_on_base_price'] == 1){
                            $tax = ($base_price2 / 100) * $row['rate_percent'];
                        } else {
                            $tax = ( ( $base_price2 / ( ( $row['rate_percent'] / 100 ) + 1 ) ) / 100 ) * $row['rate_percent'];
                        }
                        
                        $tax_list[$row['id']]['tax'] += $tax;
                        
                        // reversal
                        // orig:
                        if($group['tax_on_base_price'] == 1){
                            $compound_list[$row['priority']] += $tax;
                        }else{
                            $compound_list[$row['priority']] -= $tax;
                        }
                        
                        $processed_rates[] = $row['id'];
                        $curr_prio = $row['priority'];
                    }
                }
            }

            foreach($compound_list As $result){
                $gross += $result;
            }
            
            if($group['tax_on_base_price'] != 1){
                $gross = self::round($gross) * $amount;
                foreach($tax_list As $key => $value){
                    $tax_list[$key]['tax'] = $tax_list[$key]['tax']*$amount;
                }
            }
        }
        
        // amount
        $base_price = self::round($base_price) * $amount;
        
        $gross = self::round($base_price + $gross);
        
        // TODO: price plugins
        
        // reversal
        if(is_array($group) && $group['tax_on_base_price'] == 0){
            
            if($deduct_but_dont_list){
                
                return array('net' => $gross, 'gross' => $gross, 'tax_list' => array(), 'off' => $off);
            }
            else
            {
                return array('net' => $gross, 'gross' => $base_price, 'tax_list' => $tax_list, 'off' => $off);
            }
        }
        // orig:
        return array('net' => $base_price, 'gross' => $gross, 'tax_list' => $tax_list, 'off' => $off);
    }

    function formatPrice($price, $currency_symbol = null, $currency_code = null, $dec_point = null, $th_sep = null){
        
        $config = CrBcHelpers::getBcConfig();
        $places = $config->get('decimal_places', 2);
        if($places < 2){
            $places = 2;
        }else if($places > 4){
            $places = 4;
        }
        
        $minus = '';
        if($price < 0){
            $price = $price * -1;
            $minus = '-';
        }
        
        if($dec_point === null){
            $dec_point = $config->get('decimal_point', '.');
        }
        
        if($th_sep === null){
            $th_sep = $config->get('thousands_separator', ',');
        }
        
        $left = '';
        $right = '';
        $code = $this->currency_code;
        $symbol = $this->currency_symbol;
        
        if($currency_symbol !== null && $currency_code !== null){
            $symbol = $currency_symbol;
            $code = $currency_code;
        }
        
        switch($config->get('currency_display', 'symbol,left,code,right')){
            case 'none':
                break;
            case 'symbol,right,code,left':
                $right = $symbol;
                $left = $code . ' ';
                break;
            case 'symbol,only,left':
                $left = $symbol;
                break;
            case 'symbol,only,right':
                $right = $symbol;
                break;
            case 'code,only,left':
                $left = $code . ' ';
                break;
            case 'code,only,right':
                $right = ' ' . $code;
                break;
            default:
                $left = $symbol;
                $right = ' ' . $code;
        }
        
        return $minus . $left . number_format( round( $price, $places), $places, $dec_point, $th_sep ) . $right;
        //return $minus . $this->currency_symbol . $price;
    }

    public static function formatNumber($price, $dec_point = null, $th_sep = null){
        
        $config = CrBcHelpers::getBcConfig();
        $places = $config->get('decimal_places', 2);
        if($places < 2){
            $places = 2;
        }else if($places > 4){
            $places = 4;
        }
        
        if($dec_point === null){
            $dec_point = $config->get('decimal_point', '.');
        }
        
        if($th_sep === null){
            $th_sep = $config->get('thousands_separator', ',');
        }
        
        return number_format( round( $price, $places), $places, $dec_point, $th_sep );
        //return $price;
    }
    
    public static function round($price){
        
        $config = CrBcHelpers::getBcConfig();
        $places = $config->get('decimal_places', 2);
        if($places < 2){
            $places = 2;
        }else if($places > 4){
            $places = 4;
        }
        
        return round( $price,$places);
        //return $price;
    }

    public static function buildCart($customer_id, $customer_name){
        $cart = array();
        $db = JFactory::getDBO();
        $db->setQuery("Insert Into #__breezingcommerce_orders (customer_id, `creation_date`, history_customer_id, history_customer_name) Values ('".intval($customer_id)."', '".JFactory::getDate()->toSql()."', '".intval($customer_id)."',".$db->Quote($customer_name).")");
        $result = $db->query();
        
        if($result !== false){
            $cart = array('customer_id' => intval($customer_id), 'customer_name' => $customer_name, 'order_id' => $db->insertid(), 'products' => array(), 'products_list' => array() );
        }
        
        return $cart;
    }

    public static function purgeExpiredCarts(){
        $max_age = self::$max_age;
        $db = JFactory::getDBO();
        $db->setQuery("Select id From #__breezingcommerce_orders Where checked_out = 0 And history = 0 And Timediff('".JFactory::getDate()->toSql()."', `creation_date`) > '$max_age'");
        $rows = $db->loadAssocList();
        foreach($rows As $row){
            $db->setQuery("Delete From #__breezingcommerce_order_items Where order_id = '".$row['id']."'");
            $db->query();
        }
        $db->setQuery("Delete From #__breezingcommerce_orders Where checked_out = 0 And history = 0 And Timediff('".JFactory::getDate()->toSql()."', `creation_date`) > '$max_age'");
        $db->query();
        return $rows;
    }
    
    // ORDER FUNCTIONS
    
    public static function log($type, $message, $order_id, $identifier = ''){
        
        $db = JFactory::getDbo();
        
        $db->setQuery("Insert Into #__breezingcommerce_order_log (`order_id`,`type`,`message`,`user`,`creation_date`,`identifier`) Values (".$db->quote($order_id).", ".$db->quote($type).",".$db->quote(strip_tags($message)).",".$db->quote(JFactory::getUser()->get('username','unknown') . ' ('.JFactory::getUser()->get('id',0).')').", '".JFactory::getDate()->toSql()."', ".$db->quote($identifier).")");
        $db->query();
    }
    
    public static function createOrderNumber(){
        
        $config = CrBcHelpers::getBcConfig();
        
        if(trim($config->get('custom_order_numbers_algorithm','')) != ''){
            
            $return = '';
            
            eval( trim($config->get('custom_order_numbers_algorithm','')) );
            
            return $return;
            
        } else {
        
            $str = uniqid();
            $str .= (string)time();
            $checksum = crc32($str);
            $date = JHTML::_('date', date('Y-m-d H:i:s'), 'Y-m-d', false);
            return substr($date . ($checksum < 0 ? '' : '-' ) . $checksum, 0, 19);
        }
    }
    
    public static function sendStatusMail($order, $email, $result, $result2, $customer_result, $order_id){
        
        if($email){
            
            $subject = '';
            
            if(trim($result['name']) != '' && $result['name'] != $result2['name']){
                
                $subject = JText::_('COM_BREEZINGCOMMERCE_STATUS_MAIL_SUBJECT') . ' ' . $result['name'] . ' ' . JText::_('to') . ' ' . $result2['name'];
            
            }else{
                
                $subject = JText::_('COM_BREEZINGCOMMERCE_STATUS_MAIL_SUBJECT_ALTERNATE') . ' ' . $result2['name'];
            }
            
            $config = CrBcHelpers::getBcConfig();
            
            $body = str_replace('{firstname}', $customer_result['firstname'], $result2['email_template']);
            $body = str_replace('{lastname}', $customer_result['lastname'], $body);
            $body = str_replace('{order_number}', $order->order_number, $body);
            $body = str_replace('{business_name}', $config->get('business_name'), $body);
            
            JFactory::getMailer()->sendMail(
                    JFactory::getApplication()->getCfg('mailfrom'), 
                    JFactory::getApplication()->getCfg('fromname'), 
                    $email, 
                    $subject, 
                    $body
            );
            
            self::log('general', 'Status mail sent: ' . $result2['name'], $order_id);
        }
    }
    
    public static function getCustomerResults($order, $order_status_id, $customer){
        
        $db = JFactory::getDbo();
        
        $return = array('result'=>null,'result2'=>null,'customer_result'=>null,'email'=>'');
        $db->setQuery("Select * From #__breezingcommerce_order_status Where published = 1 And id = " . intval($order->order_status_id));
        $return['result'] = $db->loadAssoc();

        $db->setQuery("Select * From #__breezingcommerce_order_status Where published = 1 And id = " . intval($order_status_id) . " And trigger_email = 1");
        $return['result2'] = $db->loadAssoc();

        if (is_array($return['result2'])) {

            $db->setQuery("Select c.* From #__breezingcommerce_customers As c, #__users As u Where (u.id = c.userid Or c.userid = 0) And c.id = " . intval($order->customer_id));
            $return['customer_result'] = $db->loadAssoc();

            if (is_array($return['customer_result'])) {
                if ($return['customer_result']['userid'] == 0) {
                    $return['email'] = $return['customer_result']['email'];
                } else {
                    $db->setQuery("Select email From #__users Where id = " . intval($return['customer_result']['userid']));
                    $return['email'] = $db->loadResult();
                }
            } else {
                $return['email'] = $customer->email;
            }
        }
        return $return;
    }
    
    public static function checkout($order_id, $_cart_instance, $_cart_items, $_cart_array, $shipping_info = null, $payment_info = null, $limitstart = -1 , $limit = -1){
        
        $db = JFactory::getDbo();
        
        require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/order.php');
        
        $orow = new TableOrder($db);

        if ($orow->load($order_id) && $orow->history == 0) {
            
            $orow->checked_out = 1;
            $orow->checkout_date = JFactory::getDate()->toSql();
            $orow->history = 1;
            
            $order_number = self::createOrderNumber();
            
            $orow->order_number = $order_number;
            $orow->history_customer_id = $_cart_array['customer_id'];
            $orow->history_customer_name = $_cart_array['customer_name'];
            
            if($shipping_info !== null && is_array($shipping_info)){
                $shipping_plugin = $shipping_info['shipping_plugin'];
                $shipping_name = $shipping_info['shipping_name'];
                $shipping_costs = $shipping_info['shipping_costs'];
                $orow->history_shipping_plugin = $shipping_plugin;
                $orow->history_shipping_name = $shipping_name;
                $orow->history_shipping_costs = $shipping_costs;
            }
            
            if($payment_info !== null && is_array($payment_info)){
                $payment_plugin = $payment_info['payment_plugin'];
                $payment_plugin_info = $payment_info['payment_plugin_info'];
                $orow->history_payment_plugin = $payment_plugin;
                $orow->history_payment_plugin_info = $payment_plugin_info;
            }
            
            $db->setQuery("Select * From #__breezingcommerce_customers Where id = " . intval($_cart_array['customer_id']));
            $customer = $db->loadObject();
            if($customer instanceof stdClass){
                $db->setQuery("Select `name` From #__breezingcommerce_countries Where id = " . intval($customer->country_id));
                $res = $db->loadObject();
                $customer->country = isset($res->name) ? $res->name : '';

                $db->setQuery("Select `name` From #__breezingcommerce_countries Where id = " . intval($customer->shipping_country_id));
                $res = $db->loadObject();
                $customer->shipping_country = isset($res->name) ? $res->name : '';

                $db->setQuery("Select `name` From #__breezingcommerce_country_regions Where id = " . intval($customer->region_id));
                $res = $db->loadObject();
                $customer->region = isset($res->name) ? $res->name : '';

                $db->setQuery("Select `name` From #__breezingcommerce_country_regions Where id = " . intval($customer->shipping_region_id));
                $res = $db->loadObject();
                $customer->shipping_region = isset($res->name) ? $res->name : '';

                $orow->history_customer = @serialize($customer);
            }
            
            // if these special fields are empty, use the ones from the cart
            // because in the frontend checkout, they will be pre-filled
            if($orow->history_currency_code == ''){
                $orow->history_symbol = $_cart_instance->currency_symbol;
                $orow->history_conversion_rate = $_cart_instance->currency_conversion_rate;
                $orow->history_currency_code = $_cart_instance->currency_code;
            }
            
            $config = CrBcHelpers::getBcConfig();
            $data['order_status_id'] = $config->get('notifications_default_status', 0);
            
            $results = self::getCustomerResults($orow, $data, $customer);
            
            $email = $results['email'];
            $result = $results['result'];
            $result2 = $results['result2'];
            $customer_result = $results['customer_result'];
            
            $orow->store();
            
            // Inform admins
            /* Causing a bug ($cart and $_cart_items empty when ordering in frontend
            
            $_cart = new CrBcCart( array() );
            $_cart->restoreCart(intval($order_id));

            $_cart_items = $_cart->getItems(true);

            if(count($_cart_items) > 0){

                $_cart_array = $_cart->getArray();

                $data = CrBcCart::getData(intval($order_id), $_cart_items, -1, -1);

                $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1");
                $shipping_plugins = $db->loadAssocList();
                
                $_order_info = CrBcCart::getOrder(
                                    intval($order_id), 
                                    $_cart, 
                                    $_cart_array, 
                                    $_cart_items, 
                                    $_cart_array['customer_id'],
                                    $data,
                                    $shipping_plugins,
                                    array()
                            );


                CrBcCart::sendInvoiceAdmins(
                        $_order_info->customer, 
                        $orow, 
                        $_order_info, 
                        $_cart, 
                        $_cart_items
                );
            }
             
             */
        }

        require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/orderitems.php');
        
        $irow = new TableOrderitems($db);
        
        $items = CrBcCart::getData($order_id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        
        foreach($items As $item){
            if ($irow->load($item->id) && $orow) {
                if($item->history == 0){
                    $irow->history_net = $item->price_net;
                    $irow->history_gross = $item->price_gross;
                    $irow->history_taxes = $item->price_taxes;
                    $irow->history_applied_taxes = $item->applied_taxes;
                    $irow->history_single_net = $item->price_single_net;
                    
                    $irow->history_single_gross = $item->price_single_gross;
                    $irow->history_single_taxes = $item->price_single_taxes;
                    $irow->history_single_applied_taxes = $item->single_applied_taxes;
                    
                    $irow->history_properties = $item->properties;
                    $irow->history_properties_admin = $item->properties_admin;
                    $irow->history_attributes = $item->attributes_nostock;
                    $irow->history_attributes_admin = $item->attributes_nostock_admin;
                    $irow->history_attributes_text = $item->attributes_text;
                    $irow->history_attributes_invoice = $item->attributes_nostock_invoice;
                    $irow->history_amount = $item->amount;
                    $irow->history_title = $item->title;
                    $irow->history_title_admin = $item->title_admin;
                    $irow->history_discount = $item->off;
                    $irow->history_single_discount = $item->single_off;
                    $irow->history_price_tax_list = @serialize($item->price_tax_list);
                    $irow->history_price_single_tax_list = @serialize($item->single_price_tax_list);
                    $irow->history = 1;
                    $irow->store();
                }
            }
        }

        //// BACKORDERS CHECK

        $unpublished_items = array();
        
        $items = CrBcCart::getData($order_id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        
        foreach($items As $item){
            if($item->published == 0){
                $unpublished_items[] = $item->id;
            }
        }

        $cnt_unpublished_items = count($unpublished_items);
        $cnt_items = count($items);

        if($cnt_unpublished_items && $cnt_unpublished_items != $cnt_items){

            require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'CrBcHelpers.php');
            $form = CrBcHelpers::getBcConfig();
            $pform = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
            
            require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/product.php');
            $prow = new TableProduct($db);

            $copied_items = array();
            $insert_id = 0;
            
            $irow = new TableOrderitems($db);

            foreach($unpublished_items As $unpublished_item){
                if ($irow->load($unpublished_item)) {
                    
                    $irow->order_id = 0;
                    //unset($irow->id);
                    $prow->load($irow->product_id);
                    $pform->loadString($prow->attribs);
                    if($pform->get('backorders','') == 'ALLOWED_SPLIT' || $form->get('backorders','') == 'ALLOWED_SPLIT'){
                        $copied_items[] = $unpublished_item;
                        self::log('general', 'Splitting item ' . $unpublished_item . ' into new order', $order_id);
                    }
                }
            }
            
            if(count($copied_items)){
                
                self::copy($copied_items, $order_id, $_cart_instance, $_cart_items, $_cart_array, $customer);
                
                self::log('general', 'Moving splitted items ' . implode(',',$copied_items) . ' into new order', $order_id);
            }
        }
        
        self::log('checkout', 'Checked out', $order_id);
        
    }
    
    function copy($copied_items = null, $order_id, $_cart_instance, $_cart_items, $_cart_array, $customer)
    {
        $db = JFactory::getDbo();
        
        $insertid = 0;
        $insertIds = array();
        $items = CrBcCart::getData($order_id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());

        require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/orderitems.php');
        
        foreach($items As $item){
            if($copied_items === null || (is_array($copied_items) && in_array($item->id, $copied_items))){
                
                $irow = new TableOrderitems($db);
                
                $db->setQuery("Select id From #__breezingcommerce_products Where id = " . $item->product_id);
                if($db->loadResult() && $irow->load($item->id)){
                    unset($irow->id);
                    $irow->history = 0;
                    $irow->creation_date = JFactory::getDate()->toSql();
                    $irow->verified = 0;
                    $db->insertObject('#__breezingcommerce_order_items', $irow);
                    $_insertid = $db->insertid();
                    $insertIds[] = $_insertid;
                    if(is_array($copied_items) && in_array($item->id, $copied_items)){
                        $irow->delete($item->id);
                        self::log('general', 'Moved splitted item ' . $item->id . ' into new order. New item id: ' . $_insertid, $order_id);
                    }
                }
            }
        }

        if(count($insertIds)){
            
            $this->_db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'shipping' Order By `ordering`");
            $plugins = $this->_db->loadAssocList();
            
            $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'payment' Order By `ordering`");
            $payment_plugins = $db->loadAssocList();
            
            $order = CrBcCart::getOrder(
                                    $order_id, 
                                    $_cart_instance,
                                    $_cart_array, 
                                    $_cart_items, 
                                    $customer,
                                    $items,
                                    $plugins,
                                    $payment_plugins
                            );

            $db->setQuery("Select id From #__breezingcommerce_customers Where id = " . $order->customer_id);
            if(!$db->loadResult()){
                $db->setQuery("Delete From #__breezingcommerce_order_items Where id In (".implode(',',$insertIds).")");
                $db->query();
                return 0;
            }
            
            require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/order.php');
        
            $orow = new TableOrder($db);
            
            if($orow->load($order_id)){
                $old_id = $orow->id;
                unset($orow->id);
                unset($orow->order_number);
                $orow->history = 0;
                $orow->checked_out = 0;
                $orow->invoice_created = 0;
                $orow->cancelled = 0;
                $orow->order_status_id = 0;
                $orow->send_notification = 1;
                $orow->plugin_id = 0;
                $orow->selected_shipping_id = 0;
                $orow->creation_date = JFactory::getDate()->toSql();
                $orow->paid = 0;
                $orow->tax_disabled = 0;
                $orow->payment_date = '0000-00-00 00:00:00';
                $db->insertObject('#__breezingcommerce_orders', $orow);
                $insertid = $db->insertid();
                if($insertid){
                    $db->setQuery("Update #__breezingcommerce_order_items Set order_id = " . $insertid . " Where id In (".implode(',',$insertIds).")");
                    $db->query();
                    self::log('general', 'Copied from orderid ' . $old_id, $insertid, $order_id);
                    self::log('general', 'Copy created, new orderid ' . $insertid, $order_id);
                }
            }
        }

        return $insertid;
    }
    
    // TODO: remove redundance to createAndSendInvoice
    public static function sendInvoiceAdmins($customer, $order, $order_info, $_cart_instance, $_cart_items){
        
        $recipients = array();
        
        $config = CrBcHelpers::getBcConfig();
        $order_notification_emails = $config->get('order_notification_emails', '');
        
        if($order_notification_emails != ''){
            
            $order_notification_emails = str_replace("\r", '', $order_notification_emails);
            $_recipients = explode("\n", $order_notification_emails);
            
            foreach($_recipients As $recipient){
                
                $recipient = trim($recipient);
                
                if(CrBcHelpers::isEmail($recipient)){
                    $recipients[] = $recipient;
                }
            }
        }
        
        if(!is_object($order) || count($_cart_items) == 0 || !is_array($recipients) || count($recipients) == 0){
            
            return;
        }
        
        
        require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/order.php');
        
        $db = JFactory::getDbo();
        
        $invoice_folder = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'invoice';
        if(!JFolder::exists(JPATH_SITE . DS . 'media' . DS . 'breezingcommerce')) JFolder::create(JPATH_SITE . DS . 'media' . DS . 'breezingcommerce');
        if(!JFolder::exists($invoice_folder)) JFolder::create($invoice_folder);
        if(!JFolder::exists($invoice_folder.DS.'invoices')) JFolder::create($invoice_folder.DS.'invoices');
        $def = '';
        if(!JFile::exists($invoice_folder.DS.'invoices'.DS.'index.html')) JFile::write($invoice_folder.DS.'invoices'.DS.'index.html', $def);
        $def = 'deny from all';
        if(!JFile::exists($invoice_folder.DS.'invoices'.DS.'.htaccess')) JFile::write($invoice_folder.DS.'invoices'.DS.'.htaccess', $def);

        $templates_folder = $invoice_folder . DS . 'templates';
        if(!JFolder::exists($templates_folder)) JFolder::create($templates_folder);
        
        $invoice_number = JText::_('*** Not yet generated ***');
        
        if( $order->invoice_number != '' ){
            $invoice_number = $order->invoice_number;
        }
        
        $items = CrBcCart::getData($order->id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        
        // applying productdisplay order item handlers
        // productdetails PLUGINS INIT - BEGIN
        
        $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'productdetails' Order By `ordering`");
        $productdetails_plugins = $db->loadAssocList();
        
        $cnt = count($items);
        for($i = 0; $i < $cnt; $i++){
            
            $productdetails_plugin_instances = array();
            foreach( $productdetails_plugins As $plugin ){

                $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
                if($plugin_instance instanceof CrBcProductdetailsAdminPlugin){
                    
                    $plugin_instance->init($_cart_instance);
                    
                    $info = $plugin_instance->handleOrderItemDisplay($items[$i]->order_item_info, 'text_invoice');
                    $items[$i]->order_item_info = $info;
                }
            }
            // PRODUCTDETAILS PLUGINS INIT - END
        }
        
        $db->setQuery("Select * From #__breezingcommerce_order_log Where order_id = " . intval($order->id) . " And `type` = 'plugin_invoice' Order By `creation_date` Desc");
        $logs = $db->loadObjectList();
        
        // EMAIL NOTIFICATION
        ob_start();
        
        if(JFile::exists($templates_folder . DS . 'invoice_tpl_text.php')){
            require($templates_folder . DS . 'invoice_tpl_text.php');
        }else{
            require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_tpl_text.php');
        }
           
        $contents = ob_get_contents();
        ob_end_clean();
    
        $subject = '';

        $subject = JText::_('COM_BREEZINGCOMMERCE_NOTIFICATION_ABOUT_ORDER') . ' ' . $order->order_number;
        
        $body = JText::_('COM_BREEZINGCOMMERCE_NOTIFICATION_ABOUT_ORDER_DESC')."\n\n==============\n\n".$contents;

        $attachment = array();

        $result = JFactory::getMailer()->sendMail(
                JFactory::getApplication()->getCfg('mailfrom'), 
                JFactory::getApplication()->getCfg('fromname'), 
                $recipients, 
                $subject, 
                $body,
                false,
                null,
                null,
                $attachment
        );
        
        //////////////

        if( $result ){
            self::log('invoice', 'Admin Notification sent for order id: "'.$order->id . '" to the addresses ' . implode(', ', $recipients), $order->id);
        } else {
            self::log('invoice', 'Error while sending admin notification for order id: "'.$order->id. '". Email addresses involved ' . implode(', ', $recipients), $order->id);
        }
        
    }
    
    public static function createAndSendInvoice($customer, $order, $order_info, $_cart_instance, $_cart_items, $cancelled = false){
        
        if(!is_object($order) || count($_cart_items) == 0){
            return;
        }
        
        $config = CrBcHelpers::getBcConfig();
        
        require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/tables/order.php');
        
        $db = JFactory::getDbo();
        
        $invoice_folder = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'invoice';
        if(!JFolder::exists(JPATH_SITE . DS . 'media' . DS . 'breezingcommerce')) JFolder::create(JPATH_SITE . DS . 'media' . DS . 'breezingcommerce');
        if(!JFolder::exists($invoice_folder)) JFolder::create($invoice_folder);
        if(!JFolder::exists($invoice_folder.DS.'invoices')) JFolder::create($invoice_folder.DS.'invoices');
        $def = '';
        if(!JFile::exists($invoice_folder.DS.'invoices'.DS.'index.html')) JFile::write($invoice_folder.DS.'invoices'.DS.'index.html', $def);
        $def = 'deny from all';
        if(!JFile::exists($invoice_folder.DS.'invoices'.DS.'.htaccess')) JFile::write($invoice_folder.DS.'invoices'.DS.'.htaccess', $def);

        $templates_folder = $invoice_folder . DS . 'templates';
        if(!JFolder::exists($templates_folder)) JFolder::create($templates_folder);
        
        $zeros = intval($config->get('zeros',7));
        $prefix = !$cancelled ? $config->get('invoice_prefix','') : $config->get('invoice_return_prefix','');
        $suffix = !$cancelled ? $config->get('invoice_suffix','B') : $config->get('invoice_return_suffix','R');
        $invoice_number = '';
        
        // don't re-create the invoice id if it alreaddy exists
        if($order->order_number != ''){
            
            $db->setQuery("Select id From #__breezingcommerce_invoice_numbers Where order_number = " . $db->quote($order->order_number));
            $invoice_id = $db->loadResult();
            
            if(!$invoice_id){
                $db->setQuery("Insert Into #__breezingcommerce_invoice_numbers (order_number) Values (".$db->Quote($order->order_number).")");
                $db->query();
                $invoice_id = ''.$db->insertid();
            }else{
                $invoice_id = ''.$invoice_id;
            }
            
        }else{
            $db->setQuery("Insert Into #__breezingcommerce_invoice_numbers (order_number) Values (".$db->Quote($order->order_number).")");
            $db->query();
            $invoice_id = ''.$db->insertid();
        }
        
        if(trim($config->get('custom_invoice_numbers_algorithm','')) == ''){
        
            if($invoice_id){
                if($zeros > strlen($invoice_id)){
                    for($i = 0; $i < $zeros-strlen($invoice_id); $i++){
                        $invoice_number .= '0';
                    }
                }
                $invoice_number = $prefix.$invoice_number.$invoice_id.$suffix;
            }else{
               $invoice_number = '0000000B';
            }
        
        } else {
            
            $return = '';
            
            eval( trim($config->get('custom_invoice_numbers_algorithm','')) );
            
            $invoice_number = $return;
        }
        
        
        $items = CrBcCart::getData($order->id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        $items_backup = CrBcCart::getData($order->id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        
        // applying productdisplay order item handlers
        // productdetails PLUGINS INIT - BEGIN
        
        $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'productdetails' Order By `ordering`");
        $productdetails_plugins = $db->loadAssocList();
        
        $cnt = count($items);
        for($i = 0; $i < $cnt; $i++){
            
            $productdetails_plugin_instances = array();
            foreach( $productdetails_plugins As $plugin ){

                $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
                if($plugin_instance instanceof CrBcProductdetailsAdminPlugin){
                    
                    $plugin_instance->init($_cart_instance);
                    
                    $info = $plugin_instance->handleOrderItemDisplay($items[$i]->order_item_info, 'html_invoice');
                    $items[$i]->order_item_info = $info;
                    
                    $info = $plugin_instance->handleOrderItemDisplay($items_backup[$i]->order_item_info, 'text_invoice');
                    $items_backup[$i]->order_item_info = $info;
                }
            }
            // PRODUCTDETAILS PLUGINS INIT - END
        }
        
        if(!class_exists('TCPDF')){
            require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'libraries'.DS.'tcpdf'.DS.'tcpdf.php');
        }
        //$pdf = new TCPDF('P', 'mm', 'A4', true, 'UTF-8');
        $pdf = new CrBcPDF();
        $pdf->order_cancelled = $cancelled;
        $pdf->SetAutoPageBreak(true, 40);
        $pdf->setPrintHeader(false);
        
        $active_found = self::setCustomFont($pdf);
        
        if(!$active_found){
            $pdf->SetFont('courier','',9);
        }
        
        //$pdf->SetFont('helvetica','',10);
        //$pdf->SetFont('stsongstdlight'); // chinese 1
        //$pdf->SetFont('msungstdlight');  // chinese 2
        
        // PDF VERSION
        
        $logo = JPATH_SITE . DS . 'components' . DS . 'com_breezingcommerce'.DS.'images'.DS.'templates'.DS.'logo.png';
        $logo_html = Juri::root().'components/com_breezingcommerce/images/templates/logo.png';
        
        $alt_logo_folder = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce'.DS.'invoice'.DS.'templates'.DS.'logo.png';
        $alt_logo_web = Juri::root().'media/breezingcommerce/invoice/templates/logo.png';
        
        if(JFile::exists($alt_logo_folder)){
            
            $logo = $alt_logo_folder;
            $logo_html = $alt_logo_web;
        
        }
        
        if(!JFile::exists($logo)){
            
            $logo = '';
            $logo_info = array(0,0,0,0,'mime' => '');
            
        } else {
            
            $logo_info = getimagesize($logo);
        }
        
        $db->setQuery("Select * From #__breezingcommerce_order_log Where order_id = " . intval($order->id) . " And `type` = 'plugin_invoice' Order By `creation_date` Desc");
        $logs = $db->loadObjectList();
        
        ob_start();
        if(!$cancelled){
            if(JFile::exists($templates_folder . DS . 'invoice_tpl_pdf.php')){
                require($templates_folder . DS . 'invoice_tpl_pdf.php');
            }else{
                require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_tpl_pdf.php');
            }
        }else{
            if(JFile::exists($templates_folder . DS . 'invoice_return_tpl_pdf.php')){
                require($templates_folder . DS . 'invoice_return_tpl_pdf.php');
            }else{
                require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_return_tpl_pdf.php');
            }
        }
        $contents = ob_get_contents();
        ob_end_clean();
        
        $pdf->AddPage();
        $pdf->writeHTML($contents);
        $pdfname = $invoice_folder.DS.'invoices'.DS.$invoice_number.'_'.$order->order_number.'.pdf';
        $pdf->lastPage();
        
        $pdf->Output($pdfname, "F");
        
        // HTML VERSION
        ob_start();
        if(!$cancelled){
            if(JFile::exists($templates_folder . DS . 'invoice_tpl_html.php')){
                require($templates_folder . DS . 'invoice_tpl_html.php');
            }else{
                require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_tpl_html.php');
            }
        }else{
            if(JFile::exists($templates_folder . DS . 'invoice_return_tpl_html.php')){
                require($templates_folder . DS . 'invoice_return_tpl_html.php');
            }else{
                require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_return_tpl_html.php');
            }
        }
        $contents = ob_get_contents();
        ob_end_clean();
        $html = $contents;
        JFile::write($invoice_folder.DS.'invoices'.DS.$invoice_number.'_'.$order->order_number.'.html',$html);
        
        // EMAIL NOTIFICATION
        
        $items = $items_backup;
        
        ob_start();
        if(!$cancelled){
            if(JFile::exists($templates_folder . DS . 'invoice_tpl_text.php')){
                require($templates_folder . DS . 'invoice_tpl_text.php');
            }else{
                require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_tpl_text.php');
            }
        }else{
            if(JFile::exists($templates_folder . DS . 'invoice_return_tpl_text.php')){
                require($templates_folder . DS . 'invoice_return_tpl_text.php');
            }else{
                require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'assets'.DS.'templates'.DS.'invoice'.DS.'invoice_return_tpl_text.php');
            }
        }
        $contents = ob_get_contents();        
        ob_end_clean();

        $db->setQuery("Select c.* From #__breezingcommerce_customers As c, #__users As u Where (u.id = c.userid Or c.userid = 0) And c.id = " . intval($customer->id));
        $customer_result = $db->loadAssoc();

        $email = '';
        if (is_array($customer_result)) {
            if ($customer_result['userid'] == 0) {
                $email = $customer_result['email'];
            } else {
                $db->setQuery("Select email From #__users Where id = " . intval($customer_result['userid']));
                $email = $db->loadResult();
                // deleted in the meanwhile?
                if(!$email){
                    $email = $customer_result['email'];
                }
            }
        }else{
            $email = $customer->email;
        }

        if($email){
            
            $subject = '';
            
            if(!$cancelled){
                $subject = JText::_('COM_BREEZINGCOMMERCE_RECEIPT_FOR_YOUR_ORDER') . ' ' . $order->order_number;
            }else{
                $subject = JText::_('COM_BREEZINGCOMMERCE_CANCELLATION_RECEIPT_FOR_YOUR_ORDER') . ' ' . $order->order_number;
            }
            
            $body = $contents;
           
            $attachment = array();
            
            if($config->get('invoice_attach_pdf',0)){
                $attachment[] = $pdfname;
            }
            
            $is_html = false;
            if($config->get('invoice_html_email',0)){
                $body = $html;
                $is_html = true;
            }
            
            JFactory::getMailer()->sendMail(
                    JFactory::getApplication()->getCfg('mailfrom'), 
                    JFactory::getApplication()->getCfg('fromname'), 
                    $email, 
                    $subject, 
                    $body,
                    $is_html,
                    null,
                    null,
                    $attachment
            );
            
        }
        //////////////

        if(!$cancelled){
            $order->invoice_number = $invoice_number;
            $order->invoice_name = $invoice_number.'_'.$order->order_number;
            $order->invoice_date = JFactory::getDate()->toSql();
            self::log('invoice', 'Created invoice #'.$invoice_number, $order->id);
        }else{
            $order->cancelled_number = $invoice_number;
            $order->cancelled_name = $invoice_number.'_'.$order->order_number;
            $order->cancelled_date = JFactory::getDate()->toSql();
            self::log('invoice', 'Created cancellation #'.$invoice_number, $order->id);
        }
        
        return $order;
    }
    
    public static function verifyItemsGlobally(){
        
        jimport( 'joomla.user.helper' );
        $product_form = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
        
        $db = JFactory::getDbo();
        
        $db->setQuery("
            Select
                a.id,
                c.userid, 
                d.attribs,
                b.payment_date
            From 
                #__breezingcommerce_order_items As a, 
                #__breezingcommerce_orders As b, 
                #__breezingcommerce_customers As c,
                #__breezingcommerce_products As d 
            Where 
                a.verification_expiration_date > " . $db->quote( JFactory::getDate('now', 'UTC')->toSql() ) . "
            And 
                a.verification_expiration_date <> '0000-00-00 00:00:00' 
            And 
                a.verified = 0 
            And 
                a.order_id = b.id 
            And 
                b.customer_id = c.id
            And
                d.id = a.product_id
            ");
        
        $rows = $db->loadObjectList();
        
        foreach($rows As $row){
            
            $product_form->loadString($row->attribs);
            
            if($product_form->get('verification_days', 0) > 0){
            
                $verification_days = $product_form->get('verification_days', 0);

                $valid = JFactory::getDate($row->payment_date . ' +'.intval($verification_days).' days', 'UTC')->toSql();

                $db->setQuery("Update #__breezingcommerce_order_items Set verification_expiration_date = ".$db->quote($valid)." Where id = " . intval($row->id));
                $db->query();
                
                $groups = $product_form->get('verification_groups', array());

                if( is_array($groups) && $row->userid ){

                    foreach( $groups As $group ){
                        JUserHelper::addUserToGroup($row->userid, $group);
                    }
                }
            }
        }
        
        $db->setQuery("Update #__breezingcommerce_order_items Set verified = 1 Where verified = 0 And verification_expiration_date > " . $db->quote( JFactory::getDate('now', 'UTC')->toSql() ) . "  ");
        $db->query();
    }
    
    public static function unverifyItemsGlobally(){
        
        jimport( 'joomla.user.helper' );
        $product_form = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
        
        $db = JFactory::getDbo();
        
        $db->setQuery("
            Select 
                a.id,
                c.userid, 
                d.attribs 
            From 
                #__breezingcommerce_order_items As a, 
                #__breezingcommerce_orders As b, 
                #__breezingcommerce_customers As c,
                #__breezingcommerce_products As d 
            Where 
                a.verification_expiration_date <= " . $db->quote( JFactory::getDate('now', 'UTC')->toSql() ) . "
            And 
                a.verification_expiration_date <> '0000-00-00 00:00:00' 
            And 
                a.verified = 1 
            And 
                a.order_id = b.id 
            And 
                b.customer_id = c.id
            And
                d.id = a.product_id
            ");
        
        $rows = $db->loadObjectList();
        
        foreach($rows As $row){
            
            $product_form->loadString($row->attribs);
            
            if($product_form->get('verification_days', 0) > 0){
            
                $groups = $product_form->get('verification_groups', array());

                if( is_array($groups) && $row->userid ){

                    foreach( $groups As $group ){
                        JUserHelper::removeUserFromGroup($row->userid, $group);
                    }
                }
            }
        }
        
        $db->setQuery("Update #__breezingcommerce_order_items Set verification_expiration_date = '0000-00-00 00:00:00', verified = 0 Where verified = 1 And verification_expiration_date <> '0000-00-00 00:00:00' And verification_expiration_date <= " . $db->quote( JFactory::getDate('now', 'UTC')->toSql() ) . "  ");
        $db->query();
    }
    
    public static function verifyItem($item_id, $by_payment_date = true, $custom_date = ''){
        
        jimport( 'joomla.user.helper' );
        $product_form = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
        
        $db = JFactory::getDbo();
        
        // ...and verify all files of all items (=products) within this order
        $db->setQuery("Select * From #__breezingcommerce_order_items Where id = " . $db->quote($item_id) . " And verified <> 1");
        $order_items = $db->loadObjectList();
        
        foreach($order_items As $order_item){
            
            if($custom_date != ''){
                
                $payment_date = $custom_date;
                
            }else
            if( $by_payment_date ){
            
                $db->setQuery("Select payment_date From #__breezingcommerce_orders Where id = " . intval($order_item->order_id));
                $payment_date = $db->loadResult();
            
            } else {
                
                $db->setQuery("Select creation_date From #__breezingcommerce_orders Where id = " . intval($order_item->order_id));
                $payment_date = $db->loadResult();
                
            }
            
            $db->setQuery("Select * From #__breezingcommerce_products Where id = " . intval($order_item->product_id));
            $product = $db->loadObject();
            
            if($product && isset($product->attribs)){
                $product_form->loadString($product->attribs);

                if($product_form && $product_form->get('verification_days', 0) > 0){

                    $verification_days = $product_form->get('verification_days', 0);

                    $valid = JFactory::getDate($payment_date.' +'.intval($verification_days).' days', 'UTC')->toSql();

                    $db->setQuery("Update #__breezingcommerce_order_items Set verification_expiration_date = ".$db->quote($valid)." Where id = " . intval($order_item->id));
                    $db->query();

                    // find the customer, then user, will only work if there is a joomla user attached to the customer
                    $groups = $product_form->get('verification_groups', array());

                    $db->setQuery("Select a.userid From #__breezingcommerce_customers As a, #__breezingcommerce_orders As b Where b.id = " . intval($order_item->order_id) . " And b.customer_id = a.id Limit 1");
                    $user_id = $db->loadResult();

                    if( is_array($groups) && $user_id ){

                        foreach( $groups As $group ){
                            JUserHelper::addUserToGroup($user_id, $group);
                        }
                    }
                }
            }
        }
        
        $db->setQuery("Update #__breezingcommerce_order_items Set verified = 1 Where id = " . $db->quote($item_id) );
        $db->query();
    }
    
    public static function verifyItems($order_id, $by_payment_date = true){
        
        jimport( 'joomla.user.helper' );
        $product_form = new JRegistry(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');
        
        $db = JFactory::getDbo();
        
        // ...and verify all files of all items (=products) within this order
        $db->setQuery("Select * From #__breezingcommerce_order_items Where verified <> 1 And order_id = " . $db->quote($order_id));
        $order_items = $db->loadObjectList();
        
        foreach($order_items As $order_item){
            
            if( $by_payment_date ){
            
                $db->setQuery("Select payment_date From #__breezingcommerce_orders Where id = " . intval($order_item->order_id));
                $payment_date = $db->loadResult();
            
            } else {
                
                $db->setQuery("Select creation_date From #__breezingcommerce_orders Where id = " . intval($order_item->order_id));
                $payment_date = $db->loadResult();
            }
            
            $db->setQuery("Select * From #__breezingcommerce_products Where id = " . intval($order_item->product_id));
            $product = $db->loadObject();
            
            $product_form->loadString($product->attribs);
            
            if($product_form->get('verification_days', 0) > 0){
            
                $verification_days = $product_form->get('verification_days', 0);

                $valid = JFactory::getDate($payment_date . ' +'.intval($verification_days).' days', 'UTC')->toSql();

                $db->setQuery("Update #__breezingcommerce_order_items Set verification_expiration_date = ".$db->quote($valid)." Where id = " . intval($order_item->id));
                $db->query();

                // find the customer, then user, will only work if there is a joomla user attached to the customer
                $groups = $product_form->get('verification_groups', array());
                
                $db->setQuery("Select a.userid From #__breezingcommerce_customers As a, #__breezingcommerce_orders As b Where b.id = " . intval($order_item->order_id) . " And b.customer_id = a.id Limit 1");
                $user_id = $db->loadResult();
                
                if( is_array($groups) && $user_id ){
                    
                    foreach( $groups As $group ){
                        JUserHelper::addUserToGroup($user_id, $group);
                    }
                }
            }
        }
        
        $db->setQuery("Update #__breezingcommerce_order_items Set verified = 1 Where order_id = " . $order_id);
        $db->query();
    }
    
    public static function synchFileVerifications($customer_id, $override_date = ''){
        
        // TODO: horribly nested sql, needs to be fixed asap
        
        $db = JFactory::getDbo();
        
        $db->setQuery("Select id, payment_date From #__breezingcommerce_orders Where paid = 1 And payment_date <> '0000-00-00 00:00:00' And customer_id = " . $db->quote($customer_id));
        $orders = $db->loadObjectList();
        
        foreach($orders As $order){
            
            $db->setQuery("Select * From #__breezingcommerce_order_items Where published = 1 And order_id = " . $db->quote($order->id));
            $order_items = $db->loadObjectList();
            
            foreach($order_items As $order_item){
            
                $db->setQuery("Select * From #__breezingcommerce_files Where published = 1 And requires_verification = 1 And product_id = " . $db->quote($order_item->product_id));
                $files = $db->loadObjectList();

                foreach($files As $file){

                    $db->setQuery("Select id From #__breezingcommerce_orderitemfiles Where order_id = " . $db->quote($order->id) . " And order_item_id = ".$db->quote($order_item->id)." And file_id = ".$db->quote($file->id)." And product_id = ".$db->quote($order_item->product_id));
                    $exists = $db->loadResult();

                    if(!$exists){

                        $valid = '';
                        
                        $forever = 0;
                        
                        if($override_date == ''){
                    
                            if(intval($file->verification_days) > 0){

                                $valid = JFactory::getDate($order->payment_date . ' +'.intval($file->verification_days).' days', 'UTC')->toSql();
                            } else {
                                // zero days means practically forever
                                $valid = JFactory::getDate('now +100 years', 'UTC')->toSql();
                                
                                $forever = 1;
                            }

                        } else {

                            $valid = JFactory::getDate($override_date, 'UTC')->toSql();
                        }
                        
                        $db->setQuery("
                            Insert 
                                Into  
                            #__breezingcommerce_orderitemfiles 
                            (
                                order_id, 
                                order_item_id, 
                                product_id, 
                                file_id,
                                valid,
                                forever
                            ) 
                            Values 
                            (
                                ".$db->quote($order->id).",
                                ".$db->quote($order_item->id).",
                                ".$db->quote($order_item->product_id).",
                                ".$db->quote($file->id).",
                                ".$db->quote($valid).",
                                ".$db->quote($forever)."
                            )
                        ");
                        
                        $db->execute();
                    }
                }
            }
        }
    }
    
    public static function verifyFiles($order_id, $override_date = ''){
        
        $db = JFactory::getDbo();
        
        // ...purge all existing order item files for this order and re-create them later on...
        $db->setQuery("Delete From #__breezingcommerce_orderitemfiles Where order_id = " . $db->quote($order_id));
        $db->query();
        
        // ...and verify all files of all items (=products) within this order
        $db->setQuery("Select * From #__breezingcommerce_order_items Where order_id = " . $db->quote($order_id));
        $order_items = $db->loadObjectList();
        
        foreach($order_items As $order_item){
            
            $db->setQuery("Select * From #__breezingcommerce_files Where product_id = " . $db->quote($order_item->product_id) . " And published = 1 And requires_verification = 1");
            $files = $db->loadObjectList();
        
            foreach($files As $file){
                
                $forever = 0;
                
                if($override_date == ''){
                    
                    if(intval($file->verification_days) > 0){
                        
                        $valid = JFactory::getDate('now +'.intval($file->verification_days).' days', 'UTC')->toSql();
                    } else {
                        // zero days means practically forever
                        $valid = JFactory::getDate('now +100 years', 'UTC')->toSql();
                        
                        $forever = 1;
                    }
                    
                } else {
                    
                    $valid = JFactory::getDate($override_date, 'UTC')->toSql();
                }
                
                $db->setQuery("
                            Insert Into 
                                #__breezingcommerce_orderitemfiles
                                (
                                    order_id,
                                    order_item_id,
                                    product_id,
                                    file_id,
                                    valid,
                                    forever
                                )
                                Values
                                (
                                    ".$db->quote($order_id).",
                                    ".$db->quote($order_item->id).",
                                    ".$db->quote($order_item->product_id).",
                                    ".$db->quote($file->id).",
                                    ".$db->quote($valid).",
                                    ".$db->quote($forever)."
                                )
                ");
                
                $db->query();
                
            }
        }
    }
    
    public static function unverifyFiles($order_id){
        
        $db = JFactory::getDbo();
        
        // ...purge all existing order item files for this order and re-create them later on...
        $db->setQuery("Delete From #__breezingcommerce_orderitemfiles Where order_id = " . $db->quote($order_id));
        $db->query();
    }
    
    public static function unverifyFilesByOrderItemId($item_id){
        
        $db = JFactory::getDbo();
        
        // ...purge all existing order item files for this order and re-create them later on...
        $db->setQuery("Delete From #__breezingcommerce_orderitemfiles Where order_item_id = " . $db->quote($item_id));
        $db->query();
    }
    
    public static function verifyFilesByOrderItemId($item_id, $override_date = ''){
        
        $db = JFactory::getDbo();
        
        // ...purge all existing order item files for this order and re-create them later on...
        $db->setQuery("Delete From #__breezingcommerce_orderitemfiles Where order_item_id = " . $db->quote($item_id));
        $db->query();
        
        // ...and verify all files of all items (=products) within this order
        $db->setQuery("Select * From #__breezingcommerce_order_items Where id = " . $db->quote($item_id));
        $order_items = $db->loadObjectList();
        
        // will be only of length = 1
        foreach($order_items As $order_item){
            
            $db->setQuery("Select * From #__breezingcommerce_files Where product_id = " . $db->quote($order_item->product_id) . " And published = 1 And requires_verification = 1");
            $files = $db->loadObjectList();
        
            foreach($files As $file){
                
                $forever = 0;
                
                if($override_date == ''){
                    
                    if(intval($file->verification_days) > 0){
                        
                        $valid = JFactory::getDate('now +'.intval($file->verification_days).' days', 'UTC')->toSql();
                    } else {
                        // zero days means practically forever
                        $valid = JFactory::getDate('now +100 years', 'UTC')->toSql();
                        
                        $forever = 1;
                    }
                    
                } else {
                    
                    $valid = JFactory::getDate($override_date, 'UTC')->toSql();
                }
                
                $db->setQuery("
                            Insert Into 
                                #__breezingcommerce_orderitemfiles
                                (
                                    order_id,
                                    order_item_id,
                                    product_id,
                                    file_id,
                                    valid,
                                    forever
                                )
                                Values
                                (
                                    ".$db->quote($order_item->order_id).",
                                    ".$db->quote($order_item->id).",
                                    ".$db->quote($order_item->product_id).",
                                    ".$db->quote($file->id).",
                                    ".$db->quote($valid).",
                                    ".$db->quote($forever)."
                                )
                ");
                
                $db->query();
            }
        }
    }
    
    public static function stockUpdate($order_id, $_cart_items, $order_item_id = false, $sign = '-'){
        $db = JFactory::getDbo();
         // adding products back to stock
        $putback = array();
        $items = CrBcCart::getData($order_id, $_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        foreach ($items As $item) {
            if ($order_item_id === false || $item->id == $order_item_id) {
                $db->setQuery("Select properties, attributes, is_combination From #__breezingcommerce_order_items Where id = " . intval($item->id));
                $result = $db->loadAssoc();
                if (is_array($result)) {

                    if ($result['properties'] && $result['is_combination']) {
                        $db->setQuery("Update #__breezingcommerce_product_property_combinations As ppc, #__breezingcommerce_products As p Set ppc.stock = ppc.stock ".$sign." " . intval($item->amount) . " Where p.virtual_product = 0 And p.use_combinations = 1 And ppc.product_id = p.id And ppc.id In (" . $result['properties'] . ")");
                        $db->query();
                    }else{
                        if(!isset($putback[intval($item->product_id)])){
                            $putback[intval($item->product_id)] = intval($item->amount);
                        }else{
                            $putback[intval($item->product_id)] += intval($item->amount);
                        }
                    }

                    if ($result['attributes']) {
                        $attributes_set = explode(',', $result['attributes']);
                        foreach ($attributes_set As $attribute_amounts) {
                            $attribute = explode('=', $attribute_amounts);
                            
                            if (isset($attribute[0]) && isset($attribute[1])) {
                                $db->setQuery("Select Distinct pai.item_product_id, ai.product_id From #__breezingcommerce_product_attribute_items As pai, #__breezingcommerce_attribute_items As ai Where ai.id = pai.attribute_item_id And pai.id = " . intval($attribute[0]));
                                $pai = $db->loadAssoc();
                                
                                if (is_array($pai)) {
                                    $pai_pid = 0;
                                    if ($pai['item_product_id']) {
                                        $pai_pid = $pai['item_product_id'];
                                    } else if ($pai['product_id']) {
                                        $pai_pid = $pai['product_id'];
                                    }
                                    if(!isset($putback[$pai_pid])){
                                        $putback[$pai_pid] = intval($attribute[1]);
                                    }else{
                                        $putback[$pai_pid] += intval($attribute[1]);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        foreach($putback As $id => $amount){
            $db->setQuery("Update #__breezingcommerce_products As p Set p.stock = p.stock ".$sign." " . intval($amount) . " Where p.virtual_product = 0 And p.id = " . $id);
            $db->query();
        }

        self::log('general', 'Stock update called ' . $sign, $order_id);
    }
    
    public static function addOrderInfo($order_item_id, $title, $text, $key, $id){
        
        $db = JFactory::getDbo();
        
        $db->setQuery("Select order_item_info From #__breezingcommerce_order_items Where id = " . intval($order_item_id));
        $info = $db->loadResult();
        
        if(!$info){
            
            $info = '[]';
        }
        
        $info = json_decode($info);
        $info[] = array( 
                        'title' => $title,
                        'key' => $key,
                        'id' => $id,
                        'text' => $text,
                        'order_item_id' => $order_item_id
                  );
        $info = json_encode($info);
        
        $db->setQuery("Update #__breezingcommerce_order_items Set order_item_info = ".$db->quote($info)." Where id = " . intval($order_item_id));
        $db->execute();
        
    }
}