<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access
defined('_JEXEC') or die('Restricted access');

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcShippingPlugin.php');

interface CrBcShippingSitePlugin extends CrBcShippingPlugin{
    
    /**
     * inits the shipping, hands over a mixed parameter (or none).
     * Can be used to prepare things.
     * Subject is usually of type CrBcCart
     * @param mixed $subject
     */
    function init($subject = NULL);
    
    /**
     * 
     * 
     * @param int $option
     * @param int $order_id
     * @return empty string for no errors and a string != '' with the error message for that selected option
     */
    function validateSelectedOption($option, $order_id);
    
    /**
     * Determines the best available options for this shipping plugin.
     * This is called right after init(CrBcCart $cart) and gives the plugin the chance
     * to gather available shipping options before onDisplayOptions() is called.
     *
     * May also be used with external services like Canada Post.
     */
    function determineBestShippingOptions();
     /**
     * Returns a string that renders the available shipping options.
     *
     * The string will be rendered in the Order Details view in the backend.
     *
     * Leave the return value an empty string if no option is available for your shipping plugin.
     *
     * The $selected_key will be stored in the order and returned to this method
     *
     * The selection is mandatory, so it could help when there is a default element with the value "0" (zero)
     * to provide the validator.
     *
     * Only use integers for the option ids!
     *
     * @param mixed $selected_key
     * @return string
     */
    function onDisplayOptions($selected_key);
    /**
     * Called if this shipping plugin was choosen.
     *
     * @param int $option
     * @param int $order_id
     */
    function onOptionUpdate($option, $order_id);
    /**
     * Returns the selected product taxclass id for the selected shipping option.
     * The shipping options screen should provide a select list to gather the id.
     *
     * Available product taxclass ids can be retrieved using:
     *
     * $db->setQuery("Select * From #__breezingcommerce_producttaxclasses Order By `name`");
     * $taxclasses = $this->_db->loadObjectList();
     *
     * It is up to the plugin developer to render the taxclass selection!
     *
     * @param int $option
     */
    function getProductTaxclassId($option);
    /**
     * Returns the price as float/double based on the option given
     *
     * Should NOT return the formatted price as it may be used for further calculations.
     *
     * The price should be returned for the default price.
     * So if you are retrieving the price from a service like canada post, convert it to the shop's default currency!
     *
     * @param int $option
     */
    function getPrice($option);
     /**
     * Returns the name of the selected shipping rate
     *
     * @param int $option
     * @return string
     */
    function getName($option);
    /**
     * Returns the name of the option selection form element you are using
     * in onDisplayOptions()
     *
     * @return string
     */
    function getOptionElementName();
}