<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class BreezingcommerceModelCustomers extends JModelLegacy
{
    /**
     * Items total
     * @var integer
     */
    private $_total = null;

    /**
     * Pagination object
     * @var object
     */
    private $_pagination = null;

    function  __construct($config) {
        parent::__construct($config);

        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $region_id = $mainframe->getUserStateFromRequest("$option.customers_region_id", 'region_id', '0', 'int');
        $this->setState('customers_region_id', $region_id);

        $country_id = $mainframe->getUserStateFromRequest("$option.customers_country_id", 'country_id', '0', 'int');
        $this->setState('customers_country_id', $country_id);

        $search = $mainframe->getUserStateFromRequest("$option.customer_search", 'customer_search', '', 'string');
        $this->setState('customer_search', $search);

        $search_shipping = $mainframe->getUserStateFromRequest("$option.customer_search_shipping", 'customer_search_shipping', '0', 'int');
        $this->setState('customer_search_shipping', $search_shipping);

        $search = $mainframe->getUserStateFromRequest("$option.customer_search_by", 'customer_search_by', '', 'string');
        $this->setState('customer_search_by', $search);

        // Get pagination request variables
        $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);

        $filter_order     = $mainframe->getUserStateFromRequest(  $option.'customers_filter_order', 'filter_order', 'ordering', 'cmd' );
        $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'customers_filter_order_Dir', 'filter_order_Dir', 'asc', 'word' );

        $this->setState('customers_filter_order', $filter_order);
        $this->setState('customers_filter_order_Dir', $filter_order_Dir);

        $filter_state = $mainframe->getUserStateFromRequest( $option.'customers_filter_state', 'filter_state', '', 'word' );
        $this->setState('customers_filter_state', $filter_state);

    }

    function setPublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_customers '.
                '  Set published = 1 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();

    }

    function setUnpublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_customers '.
                '  Set published = 0 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();
    }

    function getCountries(){
        $query = ' Select * From #__breezingcommerce_countries ';
        $this->_db->setQuery($query);
        $countries = $this->_db->loadObjectList();
        foreach ($countries As $country) {
            $country->regions = array();
            $query = ' Select * From #__breezingcommerce_country_regions Where country_id = ' . $country->id;
            $this->_db->setQuery($query);
            $regions = $this->_db->loadObjectList();
            foreach ($regions As $region) {
                $country->regions[] = $region;
            }
        }
        return $countries;
    }

    private function buildOrderBy() {
        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $orderby = '';
        $filter_order     = $this->getState('customers_filter_order');
        $filter_order_Dir = $this->getState('customers_filter_order_Dir');

        /* Error handling is never a bad thing*/
        if(!empty($filter_order) && !empty($filter_order_Dir) ) {
            $orderby = ' ORDER BY '.$filter_order.' '.$filter_order_Dir;
        }

        return $orderby;
    }

    /*
     *
     * MAIN LIST AREA
     * 
     */

    /**
     * @return string The query
     */
    private function _buildQuery(){

        $where = '';
        $match = '';
        $filter_state = '';
        $country = '';
        $country_table = '';
        $region_table = '';
        $shipping = '';

        if( $this->getState('customer_search_shipping') ){
            $shipping = 'shipping_';
        }

        if($this->getState('customer_search_by') == 'id' && $this->getState('customer_search') != '')
        {
            $match = " c.id = " . $this->_db->Quote( $this->getState('customer_search' ) ) . " ";
        }
        else if($this->getState('customer_search_by') == 'userid' && $this->getState('customer_search') != '')
        {
            $match = " c.userid = " . $this->_db->Quote( $this->getState('customer_search' ) ) . " ";
        }
        else if($this->getState('customer_search_by') == 'lastname' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."lastname Like " . $this->_db->Quote( $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'firstname' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."firstname Like " . $this->_db->Quote( $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'fullname' && $this->getState('customer_search') != '')
        {
            $match = " Concat(c.".$shipping."firstname, c.".$shipping."lastname) Like " . $this->_db->Quote( '%' . str_replace(' ','',$this->getState('customer_search' )) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'company' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."company Like " . $this->_db->Quote( $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'email' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."email Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'address' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."address Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'address2' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."address2 Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'city' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."city Like " . $this->_db->Quote( $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'zip' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."zip Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'phone' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."phone Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'mobile' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."mobile Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }
        else if($this->getState('customer_search_by') == 'fax' && $this->getState('customer_search') != '')
        {
            $match = " c.".$shipping."fax Like " . $this->_db->Quote( '%' . $this->getState('customer_search' ) . '%' ) . " ";
        }

        if($this->getState('customers_country_id') > 0)
        {
            $and = '';
            if($match != '') {
                $and = ' And ';
            }

            $country_table = ' ,#__breezingcommerce_countries As country ';
            $country .= $and . ' country.id = ' . $this->_db->Quote($this->getState('customers_country_id')) . ' And c.'.$shipping.'country_id = country.id ';

            if($this->getState('customers_region_id') > 0)
            {
                $country_table = '';
                $country = '';
                $region_table = ' ,#__breezingcommerce_country_regions As region ';
                $country .= $and . ' region.id = ' . $this->_db->Quote($this->getState('customers_region_id')) . ' And c.'.$shipping.'region_id = region.id ';
            }
        }

        if($this->getState('customers_filter_state') == 'P' || $this->getState('customers_filter_state') == 'U')
        {
            $published = 0;
            if($this->getState('customers_filter_state') == 'P')
            {
                $published = 1;
            }

            if($match != '' || $country != '') {
                $filter_state = ' And ';
            }

            $filter_state .= ' c.published = ' . $published;
        }

        $where = ' Where is_tmp = 0 ';
        
        if($filter_state != '' || $match != '' || $country != '')
        {
            $where .= ' And ';
        }
        return "Select SQL_CALC_FOUND_ROWS
                    c.*,
                        (
                            Select count(id) From #__breezingcommerce_orders Where
                                published = 1
                            And
                                (customer_id = c.id)
                         )
                   As orders,
                        (
                            Select count(id) From #__breezingcommerce_orders Where
                                published = 1
                            And
                                (history_customer_id = c.id)
                            And
                                history_customer_name <> ''
                         )
                   As orders_history
                 From
                    #__breezingcommerce_customers As c " .
                    $country_table . $region_table . $where . $match . $country . $filter_state . $this->buildOrderBy();
    }

    /**
    * Gets the customers
    * @return array List of customers
    */
    function getData()
    {
        // Lets load the data if it doesn't already exist
        if (empty( $this->_data ))
        {
            $query = $this->_buildQuery();
            $this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
            
            $this->_db->setQuery("SELECT FOUND_ROWS()");
            $this->_total = $this->_db->loadResult();
            echo $this->_db->getErrorMsg();
        }
        //echo $this->_db->getErrorMsg();
        return $this->_data;
    }

    function getTotal() {
        // Load the content if it doesn't already exist
        if (empty($this->_total)) {
            $query = $this->_buildQuery();
            $this->_total = $this->_getListCount($query);
        }
        return $this->_total;
    }

    function getPagination() {
        // Load the content if it doesn't already exist
        if (empty($this->_pagination)) {
            jimport('joomla.html.pagination');
            //$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
            $this->_pagination = new JPagination($this->_total, $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }


    

}
