<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class BreezingcommerceModelManufacturers extends JModelLegacy
{
    /**
     * Items total
     * @var integer
     */
    private $_total = null;

    /**
     * Pagination object
     * @var object
     */
    private $_pagination = null;

    function  __construct($config) {
        parent::__construct($config);

        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $search = $mainframe->getUserStateFromRequest("$option.manufacturer_search", 'manufacturer_search', '', 'string');
        $this->setState('manufacturer_search', $search);

        $search = $mainframe->getUserStateFromRequest("$option.manufacturer_search_by", 'manufacturer_search_by', '', 'string');
        $this->setState('manufacturer_search_by', $search);

        // Get pagination request variables
        $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);

        $filter_order     = $mainframe->getUserStateFromRequest(  $option.'manufacturers_filter_order', 'filter_order', 'ordering', 'cmd' );
        $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'manufacturers_filter_order_Dir', 'filter_order_Dir', 'asc', 'word' );

        $this->setState('manufacturers_filter_order', $filter_order);
        $this->setState('manufacturers_filter_order_Dir', $filter_order_Dir);

        $filter_state = $mainframe->getUserStateFromRequest( $option.'manufacturers_filter_state', 'filter_state', '', 'word' );
        $this->setState('manufacturers_filter_state', $filter_state);

    }

    function setPublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        
        $this->_db->setQuery( ' Update #__breezingcommerce_manufacturers '.
                '  Set published = 1 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();

    }

    function setUnpublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_manufacturers '.
                '  Set published = 0 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();
    }

    /*
     *
     * MAIN LIST AREA
     * 
     */

    /**
     * @return string The query
     */
    private function _buildQuery(){
        $where = '';
        $filter_state = '';
        $match = '';
        if($this->getState('manufacturers_filter_state') == 'P' || $this->getState('manufacturers_filter_state') == 'U')
        {
            $published = 0;
            if($this->getState('manufacturers_filter_state') == 'P')
            {
                $published = 1;
            }

            $filter_state .= ' published = ' . $published;
        }
        
        if($this->getState('manufacturer_search_by') == 'name' && $this->getState('manufacturer_search') != '')
        {
            $match = " `name` Like ".$this->_db->Quote('%'.$this->getState('manufacturer_search').'%')." ";
        }
        else if($this->getState('manufacturer_search_by') == 'tag' && $this->getState('manufacturer_search') != '')
        {
            $match = " `tag` = ".$this->_db->Quote($this->getState('manufacturer_search'))." ";
        }

        if($match != '' || $filter_state != '')
        {
            $where = ' Where ';
        }

        $and = '';
        if($match != '' && $filter_state != ''){
            $and = ' And ';
        }

        return 'Select SQL_CALC_FOUND_ROWS * From #__breezingcommerce_manufacturers ' . $where . $match . ' '.$and.' ' . $filter_state . ' Order By `ordering`';
    }

    /**
    * Gets the currencies
    * @return array List of currencies
    */
    function getData()
    {
        // Lets load the data if it doesn't already exist
        if (empty( $this->_data ))
        {
            $query = $this->_buildQuery();
            $this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
        }

        return $this->_data;
    }

    function getTotal() {
        // Load the content if it doesn't already exist
        if (empty($this->_total)) {
            $query = $this->_buildQuery();
            $this->_total = $this->_getListCount($query);
        }
        return $this->_total;
    }

    function getPagination() {
        // Load the content if it doesn't already exist
        if (empty($this->_pagination)) {
            jimport('joomla.html.pagination');
            $this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }


    function saveOrder()
    {
        $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);

        $total		= count( $items );
        $row		= $this->getTable('manufacturer');
        $groupings	= array();

        $order		= JRequest::getVar( 'order', array(), 'post', 'array' );
        JArrayHelper::toInteger($order);

        // update ordering values
        for( $i=0; $i < $total; $i++ ) {
            $row->load( $items[$i] );
            if ($row->ordering != $order[$i]) {
                $row->ordering = $order[$i];
                if (!$row->store()) {
                    $this->setError($row->getError());
                    return false;
                }
            } // if
        } // for
        
        $row->reorder();
    }

}
