<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/
defined('_JEXEC') or die('Restricted access');

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcAPaymentSitePlugin.php');
require_once($libpath . 'CrBcPaymentSitePlugin.php');

class CrBc_Plugins_Payment_Paypal_Site extends CrBcAPaymentSitePlugin implements CrBcPaymentSitePlugin
{
    private $tx = '';
    
    function __construct() {
        // always call the parent constructor and always call it _first_
        parent::__construct();
        // define the default table for built-in list/details view
        $this->table = '#__breezingcommerce_plugin_payment_paypal';
    }
    
    /**
     * Will return the tx id and is called right after a successfull and verified payment
     * through $this->verifyPayment()
     * 
     * @return mixed
     */
    function getPaymentTransactionId(){
        
        return $this->tx;
    }
    
    function verifyIpn($paypal, $_cart, $_order_info){
     
        $return = false;
        
        $auth_token = $paypal->token;
        
        $url = 'https://www.paypal.com';
        if($paypal->test_account){
            $url = 'https://www.sandbox.paypal.com';
        }

        $req = 'cmd=_notify-validate';

        $tx_token = JRequest::getVar('txn_id', 0 );
        
        $this->tx = $tx_token;
        
        $keyarray = array();
        $postback = $_REQUEST;
        
        foreach ($postback as $key => $value) {
            $value = urlencode(stripslashes($value));
            $req .= "&$key=$value";
        }

        $header = "POST /cgi-bin/webscr HTTP/1.0\r\n";
        $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $header .= "Content-Length: " . strlen($req) . "\r\n\r\n";

        $pointer = null;
        $res = '';

        $ch = curl_init();
        $pointer = $ch;
        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);
        curl_setopt($ch,CURLOPT_URL, $url.'/cgi-bin/webscr');
        curl_setopt($ch,CURLOPT_POST,1);
        curl_setopt($ch,CURLOPT_POSTFIELDS,$req);
        curl_setopt($ch, CURLOPT_SSLVERSION, 6); //6 is for TLSV1.2

        ob_start();
        curl_exec($ch);
        $res=ob_get_contents();

        $lines = explode("\n", $res);

        if (strcmp ($lines[0], "VERIFIED") == 0 ) {
            
            foreach($postback As $key => $value){
                $keyarray[$key] = $value;
            }
            
            //echo $keyarray['mc_gross'] ."!=". ( round( $_order_info->price_gross, 2 ) + round( $_order_info->shipping_costs, 2 ) ) ." || ".  $keyarray['mc_currency'] ."!=". strtoupper($_cart->currency_code);
            //exit;
            
            if ( doubleval($keyarray['mc_gross']) != doubleval(round( $_order_info->price_gross, 2 ) + round( $_order_info->shipping_costs, 2 )) || $keyarray['mc_currency'] != strtoupper($_cart->currency_code) ) {
            
                throw new Exception('Stale payment data', 500);

            }else{

                // should be kept open until sending the status headers
                if (function_exists('curl_init')) {
                    curl_close($pointer);
                    ob_end_clean();
                }
                else
                {
                    fclose ($pointer);
                }
                
                return true;
            }
            
            //return true;
        }
       
        // should be kept open until sending the status headers
        if (function_exists('curl_init')) {
            curl_close($pointer);
            ob_end_clean();
        }
        else
        {
            fclose ($pointer);
        }
        
        throw new Exception('General payment error', 500);
        
        return $return;
    }
   
    function verifyPayment(CrBcCart $_cart, stdClass $order){
        
        $db = JFactory::getDBO();
        
        $db->setQuery("Select * From " . $this->table . " Order By `".$this->identity_column."` Desc Limit 1");
        $paypal = $db->loadObject();

        if(!($paypal instanceof stdClass)){
            throw new Exception('No paypal payment setup found, please create one first in Admin => BreezingCommerce => Plugins => PayPal');
        }
        
        if($order === null || $_cart === null){
            throw new Exception('Invalid order object');
        }
        
        $_cart_items = $_cart->getItems(true);
        
        if(count($_cart_items) == 0){
            throw new Exception('Empty cart');
        }
        
        $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'shipping' Order By `ordering`");
        $shipping_plugins = $db->loadAssocList();
        
        $data = CrBcCart::getData($order->id, $_cart_items, -1, -1);
        
        $_order_info = CrBcCart::getOrder(
                                    $order->id, 
                                    $_cart, 
                                    $_cart->getArray(), 
                                    $_cart_items, 
                                    $order->customer_id,
                                    $data,
                                    $shipping_plugins,
                                    array()
                            );
        
        if($paypal->use_ipn && JRequest::getVar('paypal_ipn', 'false') == 'true'){
            
            return $this->verifyIpn($paypal, $_cart, $_order_info);
        }
        
        $auth_token = $paypal->token;
        
        $url = 'https://www.paypal.com';
        if($paypal->test_account){
            $url = 'https://www.sandbox.paypal.com';
        }

        $req = 'cmd=_notify-synch';

        $tx_token = JRequest::getVar('tx', 0 );
        
        $this->tx = $tx_token;
        
        $req .= "&tx=".urlencode($tx_token)."&at=".urlencode($auth_token);

        $header = "POST /cgi-bin/webscr HTTP/1.0\r\n";
        $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $header .= "Content-Length: " . strlen($req) . "\r\n\r\n";

        $ch = curl_init();

        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);
        curl_setopt($ch,CURLOPT_URL, $url.'/cgi-bin/webscr');
        curl_setopt($ch,CURLOPT_POST,1);
        curl_setopt($ch,CURLOPT_POSTFIELDS,$req);
        curl_setopt($ch, CURLOPT_SSLVERSION, 6); //6 is for TLSV1.2

        ob_start();
        curl_exec($ch);
        $res=ob_get_contents();
        curl_close($ch);
        ob_end_clean();
             
        $lines = explode("\n", $res);
       
        $keyarray = array();

        if (strcmp ($lines[0], "SUCCESS") == 0) {
            
            for ($i=1; $i<count($lines);$i++){
                if ($lines[$i] != "") {
                        list($key,$val) = explode("=", $lines[$i]);
                        $keyarray[urldecode($key)] = urldecode($val);
                }
            }
            
            //echo doubleval($keyarray['mc_gross']) ."!=". round( $_order_info->price_gross, 2 ) ."+". round( $_order_info->shipping_costs, 2 );
            //exit;
            
            if ( doubleval($keyarray['mc_gross']) != doubleval(round( $_order_info->price_gross, 2 ) + round( $_order_info->shipping_costs, 2 )) || $keyarray['mc_currency'] != strtoupper($_cart->currency_code) ) {
                
                return false;

            }else{

                return true;
            }
            
            //return true;
        }
        
        return false;
    }
    
    function getInitOutput(){
        
        $db = JFactory::getDBO();
        
        require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'CrBcCart.php');
        
        $_session_cart = JFactory::getSession()->get('crbc_cart', array());
        
        if(!isset($_session_cart['checkout']) || !isset($_session_cart['checkout']['payment_plugin_id'])){
            throw new Exception('User checkout not performed yet');
        }
        
        $payment_plugin_id = intval($_session_cart['checkout']['payment_plugin_id']);
        
        $_cart = new CrBcCart( $_session_cart );
        $_cart_items = $_cart->getItems(true);
        
        if(count($_cart_items) == 0){
            
            throw new Exception('Trying to pay an empty cart');
        }
        
        $db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'shipping' Order By `ordering`");
        $shipping_plugins = $db->loadAssocList();
        
        $data = CrBcCart::getData($_session_cart['order_id'], $_cart_items, -1, -1);
        
        $_order_info = CrBcCart::getOrder(
                                    $_session_cart['order_id'], 
                                    $_cart, 
                                    $_session_cart, 
                                    $_cart_items, 
                                    $_session_cart['customer_id'],
                                    $data,
                                    $shipping_plugins,
                                    array()
                            );
        
        if($_order_info->grand_total <= 0){
            
            throw new Exception('Trying to use paypal while the total is zero.');
        }
        
        $db->setQuery("Select * From " . $this->table . " Order By `".$this->identity_column."` Desc Limit 1");
        $paypal = $db->loadObject();

        if(!($paypal instanceof stdClass)){
            throw new Exception('No paypal payment setup found, please create one first in Admin => BreezingCommerce => Plugins => PayPal');
        }
        
        $paypal->url = 'https://www.paypal.com';
        
        if($paypal->test_account == 1){
            $paypal->url = 'https://www.sandbox.paypal.com';
        }
        
        //$paypal->item_number = 0;
        //$business_name = CrBcHelpers::getBcConfig()->get('business_name', 'Default Shop Name');
        //$paypal->item_name = JText::_('Your order at') . ' ' . $business_name;
        //$paypal->amount = $_order_info->price_net;
        
        $paypal->items = $_cart_items;
        
        $paypal->tax = $_order_info->taxes;
        $paypal->locale = JFactory::getApplication()->getLanguage()->getTag();
        $paypal->locale = explode('-',$paypal->locale);
        $paypal->locale = $paypal->locale[1];
        
        if( $paypal->force_locale != '' ){
            
            $paypal->locale = $paypal->force_locale;
        }
        
        $paypal->currency = $_cart->currency_code;
        
        if( $paypal->force_currency != '' ){
            
            $paypal->currency = $paypal->force_currency;
        }
        
        $paypal->return_url = JUri::getInstance()->toString();
        
        if($paypal->cancel_url == ''){
           $paypal->cancel_url = JUri::root(); 
        }
        
        $paypal->no_shipping = $_cart->isVirtualOrder($_cart_items) ? 1 : 0;
        
        $paypal->shipping = $_order_info->shipping_costs;
        
        $paypal->order_id = $_session_cart['order_id'];
        
        $paypal->payment_plugin_id = $payment_plugin_id;
        
        ob_start();
        require_once( JPATH_SITE . '/media/breezingcommerce/plugins/payment/paypal/site/tmpl/payment.php' );
        $content = ob_get_contents();
        ob_end_clean();
        
        return $content;
    }
    
    /**
     * Optional method to prevent this payment from being used if it's not suitable.
     * For example determing if user's location is actually suitable for the payment option.
     * If it returns false, the option won't be displayed upon checkout and also not being processed.
     * 
     * @return boolean
     */
    function isPaymentSuitable(){
        return true;
    }
    
    function getAfterPaymentInfo(){
        return JText::_('COM_BREEZINGCOMMERCE_PAYPAL_INFO_PAID');
    }
    
    public function getPluginDisplayName(){
        return JText::_('COM_BREEZINGCOMMERCE_PAYPAL');
    }
    
    public function getPluginIcon(){
        return '<i class="crbc-fa crbc-fa-paypal"></i>';
    }
    
    public function getPluginDescription(){
        return JText::_('COM_BREEZINGCOMMERCE_PAYPAL_DESCRIPTION');
    }
    
    function getPaymentInfo(){
        $db = JFactory::getDBO();

        $db->setQuery("Select * From " . $this->table . " Order By `".$this->identity_column."` Desc Limit 1");
        $row = $db->loadObject();

        if(!($row instanceof stdClass)){
            $row = new stdClass();
            $row->info = JText::_('No payment info available');
        }
        
        $id = $this->identity_column;
        
        $result = CrBcHelpers::loadTranslation($row->$id, 'plugin_payment_paypal');
        
        if($result){
            
            $row->info = $result->body;
        }
        
        return $row->info;
    }
}